/**
 * Defines if the extension is active or inactive.
 * @type {boolean}
 */
let extension_active = true;

/**
 * If true all images are down-scaled to 320x320 px before they are given to the AI preperation.
 * This step might be redundant as the AI preperation does also down-scale the generated Tensor, but
 * tests have shown that it will generate similar (sometimes better / sometimes worse) results while taking
 * a few ms (~10) less and spare a lot of MEMORY (<- important for the batch converter).
 * @type {boolean}
 */
let prescale = true;

/**
 * If true initializes the AI when Firefox is started.
 * @type {boolean}
 */
let ai_warm_up = true;


let labels = null;
let censor_type = null;
let file_types = null;

let default_labels = ["BELLYEXPOSED",
    "BELLYCOVERED",
    "BUTTOCKSEXPOSED",
    "BUTTOCKSCOVERED",
    "FEMALEBREASTEXPOSED",
    "FEMALEBREASTCOVERED",
    "FEMALEGENITALIAEXPOSED",
    "FEMALEGENITALIACOVERED",
    "MALEGENITALIACOVERED",
    "MALEGENITALIAEXPOSED",
    "MALEBREASTEXPOSED",
    "MALEBREASTCOVERED",
    "FEETCOVERED",
    "FEETEXPOSED",
    "ARMPITSCOVERED",
    "ARMPITSEXPOSED",
    "ANUSCOVERED",
    "ANUSEXPOSED"];

let all_labels = [
    "FACEFEMALE",
    "FACEMALE",
    "BELLYEXPOSED",
    "BELLYCOVERED",
    "BUTTOCKSEXPOSED",
    "BUTTOCKSCOVERED",
    "FEMALEBREASTEXPOSED",
    "FEMALEBREASTCOVERED",
    "FEMALEGENITALIAEXPOSED",
    "FEMALEGENITALIACOVERED",
    "MALEGENITALIACOVERED",
    "MALEGENITALIAEXPOSED",
    "MALEBREASTEXPOSED",
    "MALEBREASTCOVERED",
    "FEETCOVERED",
    "FEETEXPOSED",
    "ARMPITSCOVERED",
    "ARMPITSEXPOSED",
    "ANUSCOVERED",
    "ANUSEXPOSED"
];

function getStickerCollection(callback){
    let sticker_request = browser.storage.local.get(['sticker_collections']);
    sticker_request.then((res) => {
        callback(res);
    });
}

function loadDefaultStickers(callback){
    browser.storage.local.get(['sticker_collections']).then((res) => {
        if(!res.sticker_collections){
            res.sticker_collections = [];
        }
        let fruits = [
            {name: 'eggplant', path:'images/stickers/fruits/fruits_eggplant_border.png',
                klasses: [
                    klasses.MALE_GENITALIA_COVERED,
                    klasses.MALE_GENITALIA_EXPOSED
                ],
                scale: 1.2,
                chance: 1,
            },
            {name: 'banana', path:'images/stickers/fruits/fruits_banana_border.png',
                klasses: [
                    klasses.MALE_GENITALIA_COVERED,
                    klasses.MALE_GENITALIA_EXPOSED
                ],
                scale: 1.4,
                chance: 1,
            },
            {name: 'cucumber', path:'images/stickers/fruits/fruits_cucumber_border.png',
                klasses: [
                    klasses.MALE_GENITALIA_COVERED,
                    klasses.MALE_GENITALIA_EXPOSED
                ],
                scale: 1,
                chance: 1,
            },
            {name: 'cherry', path:'images/stickers/fruits/fruits_cherry_border.png',
                klasses: [
                    klasses.FEMALE_GENITALIA_COVERED,
                    klasses.FEMALE_GENITALIA_EXPOSED
                ],
                scale: 2,
                chance: 1,
            },
            {name: 'peach', path:'images/stickers/fruits/fruits_peach_border.png',
                klasses: [
                    klasses.BUTTOCKS_COVERED,
                    klasses.BUTTOCKS_EXPOSED
                ],
                scale: 1,
                chance: 1,
            },
            {name: 'watermelon', path:'images/stickers/fruits/fruits_watermelon_border.png',
                klasses: [
                    klasses.FEMALE_BREAST_COVERED,
                    klasses.FEMALE_BREAST_EXPOSED
                ],
                scale: 1,
                chance: 1,
            },
            {name: 'half orange', path:'images/stickers/fruits/fruits_half_orange_border.png',
                klasses: [
                    klasses.FEMALE_BREAST_COVERED,
                    klasses.FEMALE_BREAST_EXPOSED,
                    klasses.ANUS_COVERED,
                    klasses.ANUS_EXPOSED
                ],
                scale: 1,
                chance: 0.5,
            },
            {name: 'half angled orange', path:'images/stickers/fruits/fruits_angled_half_orange_border.png',
                klasses: [
                    klasses.FEMALE_BREAST_COVERED,
                    klasses.FEMALE_BREAST_EXPOSED,
                    klasses.ANUS_COVERED,
                    klasses.ANUS_EXPOSED
                ],
                scale: 1,
                chance: 0.5,
            },
            {name: 'apple', path:'images/stickers/fruits/fruits_apple_border.png',
                klasses: [
                    klasses.BUTTOCKS_COVERED,
                    klasses.BUTTOCKS_EXPOSED
                ],
                scale: 1,
                chance: 1,
            },
            {name: 'half apple', path:'images/stickers/fruits/fruits_half_apple_border.png',
                klasses: [
                    klasses.FEMALE_GENITALIA_COVERED,
                    klasses.FEMALE_GENITALIA_EXPOSED
                ],
                scale: 1,
                chance: 1,
            },
            {name: 'coconut', path:'images/stickers/fruits/fruits_coconut_border.png',
                klasses: [
                    klasses.FEMALE_BREAST_COVERED,
                    klasses.FEMALE_BREAST_EXPOSED,
                ],
                scale: 1.1,
                chance: 1,
            },
            {name: 'pear', path:'images/stickers/fruits/fruits_pear_border.png',
                klasses: [
                    klasses.BUTTOCKS_COVERED,
                    klasses.BUTTOCKS_EXPOSED
                ],
                scale: 1,
                chance: 1,
            },
            {name: 'strawberry', path:'images/stickers/fruits/fruits_strawberry_border.png',
                klasses: [
                    klasses.FEMALE_GENITALIA_COVERED,
                    klasses.FEMALE_GENITALIA_EXPOSED
                ],
                scale: 1.2,
                chance: 1,
            },
            {name: 'kiwi', path:'images/stickers/fruits/fruits_kiwi_border.png',
                klasses: [
                    klasses.FEMALE_GENITALIA_COVERED,
                    klasses.FEMALE_GENITALIA_EXPOSED,
                    klasses.ANUS_COVERED,
                    klasses.ANUS_EXPOSED
                ],
                scale: 1,
                chance: 1,
            },
            {name: 'pumpkin', path:'images/stickers/fruits/fruits_pumpkin_border.png',
                klasses: [
                    klasses.FACE_MALE,
                    klasses.FACE_FEMALE
                ],
                scale: 1,
                chance: 1,
            },
            {name: 'pumpkin face 01', path:'images/stickers/fruits/fruits_pumpkin_face_01_border.png',
                klasses: [
                    klasses.FACE_MALE,
                    klasses.FACE_FEMALE
                ],
                scale: 1,
                chance: 1,
            },
            {name: 'pumpkin face 02', path:'images/stickers/fruits/fruits_pumpkin_face_02_border.png',
                klasses: [
                    klasses.FACE_MALE,
                    klasses.FACE_FEMALE
                ],
                scale: 1,
                chance: 1,
            },
            {name: 'pumpkin face 03', path:'images/stickers/fruits/fruits_pumpkin_face_03_border.png',
                klasses: [
                    klasses.FACE_MALE,
                    klasses.FACE_FEMALE
                ],
                scale: 1,
                chance: 1,
            },
        ];
        let klses = Object.values(klasses);
        let emoji_hearts = [
            {name: 'heart', path:'images/stickers/emojis/2764.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'heart on fire', path:'images/stickers/emojis/2764-fe0f.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'black heart', path:'images/stickers/emojis/1f5a4.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'broken heart', path:'images/stickers/emojis/1f494.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'sparkling heart', path:'images/stickers/emojis/1f496.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'cupid', path:'images/stickers/emojis/1f498.png',
                klasses: klses,
                scale: 1,chance:1,
            },
        ];
        let emoji_smilies = [
            {name: 'hot face', path:'images/stickers/emojis/1f975.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'smiling face with 3 hearts', path:'images/stickers/emojis/1f970.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'heart eyes', path:'images/stickers/emojis/1f60d.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'kissing heart', path:'images/stickers/emojis/1f618.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'yum', path:'images/stickers/emojis/1f60b.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'innocent', path:'images/stickers/emojis/1f607.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'stuck out tongue winking eye', path:'images/stickers/emojis/1f61c.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'relieved', path:'images/stickers/emojis/1f60c.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'smirk', path:'images/stickers/emojis/1f60f.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'grimacing', path:'images/stickers/emojis/1f62c.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'star struck', path:'images/stickers/emojis/1f929.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'face with symbols over mouth', path:'images/stickers/emojis/1f92c.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'rage', path:'images/stickers/emojis/1f621.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'grin', path:'images/stickers/emojis/1f601.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'smiling imp', path:'images/stickers/emojis/1f608.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'wink', path:'images/stickers/emojis/1f609.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'confounded', path:'images/stickers/emojis/1f616.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'triumph', path:'images/stickers/emojis/1f624.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'face with spiral eyes', path:'images/stickers/emojis/1f635-200d-1f4ab.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'drooling face', path:'images/stickers/emojis/1f924.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'face with raised eye brow', path:'images/stickers/emojis/1f928.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'melting face', path:'images/stickers/emojis/1fae0.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'peeking', path:'images/stickers/emojis/1fae3.png',
                klasses: klses,
                scale: 1,chance:1,
            },
        ];
        let emoji_hands = [
            {name: 'middle finger', path:'images/stickers/emojis/1f595.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'raised hand', path:'images/stickers/emojis/1f590.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'pinching hand', path:'images/stickers/emojis/1f90f.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'punch', path:'images/stickers/emojis/1f44a.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'sign of the horns', path:'images/stickers/emojis/1f918.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'call me', path:'images/stickers/emojis/1f919.png',
                klasses: klses,
                scale: 1,chance:1,
            },
        ];
        let emoji_signs = [
            {name: 'no entry sign', path:'images/stickers/emojis/1f6ab.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'octagonal', path:'images/stickers/emojis/1f6d1.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'warning', path:'images/stickers/emojis/26a0.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'no entry', path:'images/stickers/emojis/26d4.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'underage', path:'images/stickers/emojis/1f51e.png',
                klasses: klses,
                scale: 1,chance:1,
            },
        ];
        let emoji_other = [
            {name: 'skull crossbones', path:'images/stickers/emojis/2620.png',
                klasses: klses,
                scale: 1.2,
            },
            {name: 'boom', path:'images/stickers/emojis/1f4a5.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'lip bite', path:'images/stickers/emojis/1fae6.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'lock', path:'images/stickers/emojis/1f512.png',
                klasses: klses,
                scale: 1,chance:1,
            },
            {name: 'fire', path:'images/stickers/emojis/1f525.png',
                klasses: klses,
                scale: 1,chance:1,
            },
        ];
        let collections = [fruits, emoji_smilies, emoji_hearts, emoji_hands, emoji_other, emoji_signs];
        let names = ['fruits', 'emoji smilies','emoji hearts','emoji hands','emoji other' , 'emoji signs'];
        let promises = [];
        collections.forEach(function (col,idc) {
            let collection = new StickerCollection(names[idc],[], true,true);
            col.forEach(function (element, idx, array) {
                if(!res.sticker_collections.some(e => {
                    return e._name === collection.name;
                })){
                    let p = new Promise(resolve => {
                        let localUrl = browser.runtime.getURL(element.path);
                        let imageData = new Image();
                        imageData.onload = function () {
                            const canvas = document.createElement('canvas');
                            canvas.width = imageData.width;
                            canvas.height = imageData.height;
                            let ctx = canvas.getContext("2d");
                            ctx.drawImage(imageData, 0, 0);
                            let dataurl = canvas.toDataURL("image/png");
                            //let data = ctx.getImageData(0, 0, canvas.width, canvas.height);//.data.buffer;
                            let sticker = {
                                name: element.name,
                                //file: data.data,
                                file: null,
                                url: dataurl,
                                width: canvas.width,
                                height: canvas.height,
                                klasses: element.klasses,
                                scale: element.scale,
                                locked: true,
                                chance: element.chance,
                                groups: [],
                            };
                            collection._stickers.push(sticker);

                            if (idx == col.length - 1) {
                                if (!res.sticker_collections.some(e => {
                                    return e._name === collection.name;
                                })) {
                                    res.sticker_collections.push(collection);
                                } else {
                                    //Should not happen
                                }
                            }
                            resolve();
                        }
                        imageData.src = localUrl;
                    });
                    promises.push(p);
                }
            });
        });
        Promise.all(promises).then((values) => {
            browser.storage.local.set({
                sticker_collections: res.sticker_collections,
            }).then(() => {callback()});
        });
    });
}

function renameKey(obj, old_key, new_key) {
    obj[new_key] = obj[old_key];
}

function moveSettingsToPreset(config) {
    Object.assign(config, {
        presets: [{ config: {...config}, name: "Regular" }],
        selected_preset_index: 0,
    });
    config.presets[0].config._name = undefined;
}

// If the configuration is from a version older than this we just assume the regular repairConfigurations made it compatible with 7.6.1
function repairConfigurationFrom7_6_1() {
    moveSettingsToPreset(bar_configuration);
    renameKey(bar_configuration.presets[0].config, '_shape', 'shape');
    renameKey(bar_configuration.presets[0].config, '_scale', 'scale');
    renameKey(bar_configuration.presets[0].config, '_feathering', 'feathering');
    renameKey(bar_configuration.presets[0].config, '_color', 'color');

    moveSettingsToPreset(blur_configuration);
    renameKey(blur_configuration.presets[0].config, '_shape', 'shape');
    renameKey(blur_configuration.presets[0].config, '_scale', 'scale');
    renameKey(blur_configuration.presets[0].config, '_feathering', 'feathering');
    renameKey(blur_configuration.presets[0].config, '_blur_strength', 'strength');
    renameKey(blur_configuration.presets[0].config, '_grayscale', 'grayscale');

    moveSettingsToPreset(pixel_configuration);
    renameKey(pixel_configuration.presets[0].config, '_shape', 'shape');
    renameKey(pixel_configuration.presets[0].config, '_scale', 'scale');
    renameKey(pixel_configuration.presets[0].config, '_feathering', 'feathering');
    renameKey(pixel_configuration.presets[0].config, '_distance', 'spacing');
    renameKey(pixel_configuration.presets[0].config, '_geometry', 'type');
    renameKey(pixel_configuration.presets[0].config, '_resolution_aware', 'resolution_aware');
    renameKey(pixel_configuration.presets[0].config, '_detection_size_aware', 'detection_size_aware');
    renameKey(pixel_configuration.presets[0].config, '_pixel_size', 'strength');
    renameKey(pixel_configuration.presets[0].config, '_grayscale', 'grayscale');

    moveSettingsToPreset(triangle_configuration);
    renameKey(triangle_configuration.presets[0].config, '_shape', 'shape');
    renameKey(triangle_configuration.presets[0].config, '_scale', 'scale');
    renameKey(triangle_configuration.presets[0].config, '_feathering', 'feathering');
    renameKey(triangle_configuration.presets[0].config, 'blur', 'blurring');
    renameKey(triangle_configuration.presets[0].config, 'vertexMode', 'resolution_aware');
    renameKey(triangle_configuration.presets[0].config, 'vertexCount', 'vertex_count');
    renameKey(triangle_configuration.presets[0].config, 'fillColor', 'fill_color');
    renameKey(triangle_configuration.presets[0].config, 'strokeColor', 'stroke_color');
    switch (triangle_configuration.presets[0].config.fill) {
        case false:
            triangle_configuration.presets[0].config.fill_color = '#00000000';
            triangle_configuration.presets[0].config.fill_color_mode = 1;
            break;
        case true:
            triangle_configuration.presets[0].config.fill_color_mode = 0;
            break;
        case 'color':
            triangle_configuration.presets[0].config.fill_color_mode = 1;
            break;
    }
    switch (triangle_configuration.presets[0].config.stroke) {
        case false:
            triangle_configuration.presets[0].config.stroke_color = '#00000000';
            triangle_configuration.presets[0].config.stroke_color_mode = 1;
            break;
        case true:
            triangle_configuration.presets[0].config.stroke_color_mode = 0;
            break;
        case 'color':
            triangle_configuration.presets[0].config.stroke_color_mode = 1;
            break;
    }
    renameKey(triangle_configuration.presets[0].config, 'strokeWidth', 'stroke_width');
    renameKey(triangle_configuration.presets[0].config, 'gradientStops', 'gradient_stops');
    switch (triangle_configuration.presets[0].config.lineJoin) {
        case 'miter':
            triangle_configuration.presets[0].config.line_join = 0;
            break;
        case 'round':
            triangle_configuration.presets[0].config.line_join = 1;
            break;
        case 'bevel':
            triangle_configuration.presets[0].config.line_join = 2;
            break;
    }
    renameKey(triangle_configuration.presets[0].config, 'transparentColor', 'transparent_color');
    
    moveSettingsToPreset(box_configuration);
    renameKey(box_configuration.presets[0].config, '_shape', 'shape');
    renameKey(box_configuration.presets[0].config, '_scale', 'scale');
    renameKey(box_configuration.presets[0].config, '_feathering', 'feathering');
    switch (box_configuration.presets[0].config.fill) {
        case false:
            box_configuration.presets[0].config.fill = 1;
            break;
        case true:
            box_configuration.presets[0].config.fill = 0;
            break;
    }
    switch (box_configuration.presets[0].config.always_print_border) {
        case false:
            box_configuration.presets[0].config.border = 1;
            break;
        case true:
            box_configuration.presets[0].config.border = 0;
            break;
    }

    moveSettingsToPreset(glitch_configuration);
    renameKey(glitch_configuration.presets[0].config, '_shape', 'shape');
    renameKey(glitch_configuration.presets[0].config, '_scale', 'scale');
    renameKey(glitch_configuration.presets[0].config, '_feathering', 'feathering');
    renameKey(glitch_configuration.presets[0].config, 'glitch_type', 'type');
    if (glitch_configuration.presets[0].config.glitch_types?.[0] != null) {
        glitch_configuration.presets[0].config.chromatic_aberration_shift_intensity = glitch_configuration.presets[0].config.glitch_types[0].intensity_shift;
        glitch_configuration.presets[0].config.chromatic_aberration_color_intensity = glitch_configuration.presets[0].config.glitch_types[0].intensity_color;
        glitch_configuration.presets[0].config.chromatic_aberration_vertical_shift = glitch_configuration.presets[0].config.glitch_types[0].shift_vertical;
        glitch_configuration.presets[0].config.chromatic_aberration_horizontal_shift = glitch_configuration.presets[0].config.glitch_types[0].shift_horizontal;
    }
    if (glitch_configuration.presets[0].config.glitch_types?.[1] != null) {
        glitch_configuration.presets[0].config.multiple_panels_amount = glitch_configuration.presets[0].config.glitch_types[1].number_of_layers;
        glitch_configuration.presets[0].config.multiple_panels_min_size = glitch_configuration.presets[0].config.glitch_types[1].min_scale;
        glitch_configuration.presets[0].config.multiple_panels_max_size = glitch_configuration.presets[0].config.glitch_types[1].max_scale;
        glitch_configuration.presets[0].config.multiple_panels_scatter = glitch_configuration.presets[0].config.glitch_types[1].scatter;
        glitch_configuration.presets[0].config.multiple_panels_split_chance = glitch_configuration.presets[0].config.glitch_types[1].split_chance;
        glitch_configuration.presets[0].config.multiple_panels_border = glitch_configuration.presets[0].config.glitch_types[1].border;
        glitch_configuration.presets[0].config.multiple_panels_gradient = glitch_configuration.presets[0].config.glitch_types[1].gradients;
    }

    moveSettingsToPreset(sticker_configuration);
    renameKey(sticker_configuration.presets[0].config, '_cached', 'cached');
    // moveSettingsToPreset copied _cached too, but that one stays in sticker_configuration
    sticker_configuration.presets[0].config._cached = undefined;
    renameKey(sticker_configuration.presets[0].config, '_shape', 'shape');
    renameKey(sticker_configuration.presets[0].config, '_scale', 'scale');
    renameKey(sticker_configuration.presets[0].config, '_feathering', 'feathering');
    switch (sticker_configuration.presets[0].config._draw_mode) {
        case 0:
            sticker_configuration.presets[0].config.sample_single_soruce = false;
            break;
        case 1:
            sticker_configuration.presets[0].config.sample_single_soruce = true;
            break;
        case 2:
            sticker_configuration.presets[0].config.sample_single_soruce = true;
    }
    switch (sticker_configuration.presets[0].config._consistency) {
        case false:
            sticker_configuration.presets[0].config.draw_mode = 1;
            break;
        case true:
            sticker_configuration.presets[0].config.draw_mode = 0;
            break;
    }

    moveSettingsToPreset(sobel_configuration);
    renameKey(sobel_configuration.presets[0].config, '_shape', 'shape');
    renameKey(sobel_configuration.presets[0].config, '_scale', 'scale');
    renameKey(sobel_configuration.presets[0].config, '_feathering', 'feathering');
    renameKey(sobel_configuration.presets[0].config, '_color_inverted', 'inverted_color');
    
    moveSettingsToPreset(splatter_configuration);
    renameKey(splatter_configuration.presets[0].config, '_shape', 'shape');
    renameKey(splatter_configuration.presets[0].config, '_scale', 'scale');
    renameKey(splatter_configuration.presets[0].config, '_feathering', 'feathering');
    renameKey(splatter_configuration.presets[0].config, '_splatter_iterations', 'recursions');
    if (splatter_configuration.presets[0].config._splatter_color_scheme != null) {
        if (!splatter_configuration.presets[0].config._splatter_color_scheme.length) {
            splatter_configuration.presets[0].config.color_mode = 0;
        } else {
            splatter_configuration.presets[0].config.color_mode = 1;
            splatter_configuration.presets[0].config.color_pool = splatter_configuration.presets[0].config._splatter_color_scheme.slice(0, 6).map((color) => ({ color: color, weight: 1 }));
            for (let i = splatter_configuration.presets[0].config.color_pool.length; i < 6; i++) {
                splatter_configuration.presets[0].config.color_pool.push({ color: "#000000", weight: 0 });
            }
        }
    }
    renameKey(splatter_configuration.presets[0].config, '_splatter_transparency', 'opacity');
    renameKey(splatter_configuration.presets[0].config, '_splatter_centering', 'centering');
    renameKey(splatter_configuration.presets[0].config, '_splatter_size', 'size');
    renameKey(splatter_configuration.presets[0].config, '_splatter_amount', 'amount');
    renameKey(splatter_configuration.presets[0].config, '_splatter_sub_size', 'sub_size');
    renameKey(splatter_configuration.presets[0].config, '_splatter_sub_amount', 'sub_amount');

    moveSettingsToPreset(random_configuration);
    renameKey(random_configuration.presets[0].config, '_shape', 'shape');
    renameKey(random_configuration.presets[0].config, '_scale', 'scale');
    renameKey(random_configuration.presets[0].config, '_feathering', 'feathering');
    renameKey(random_configuration.presets[0].config, 'random_mode', 'mode');
    renameKey(random_configuration.presets[0].config, 'censor_modes', 'censor_weights');
    
    moveSettingsToPreset(mixed_configuration);
    renameKey(mixed_configuration.presets[0].config, '_shape', 'shape');
    renameKey(mixed_configuration.presets[0].config, '_scale', 'scale');
    renameKey(mixed_configuration.presets[0].config, '_feathering', 'feathering');
    if (mixed_configuration.presets[0].config.censor_types != null) {
        mixed_configuration.presets[0].config.censor_preset_indexes = {};
        for (let klass_index = 0; klass_index < 20; klass_index++) {
            // if the censor type is normal and the label is not enabled or there is no reverse mode, then don't assign
            if (mixed_configuration.presets[0].config.censor_types[klass_index]?.[0] != null && (mixed_configuration.presets[0].config.censor_types[klass_index][0] !== effects.NONE.index || (mixed_configuration.presets[0].config.reverse_mode !== effects.NONE.index && labels.includes(parseKlassFromIndex(klass_index).key)))) {
                mixed_configuration.presets[0].config.censor_types[klass_index] = [mixed_configuration.presets[0].config.censor_types[klass_index][0]];
                mixed_configuration.presets[0].config.censor_preset_indexes[klass_index] = [0];
            } else {
                mixed_configuration.presets[0].config.censor_types[klass_index] = [];
                mixed_configuration.presets[0].config.censor_preset_indexes[klass_index] = [];
            }
        }
    }
    if (mixed_configuration.presets[0].config.reverse_mode != null && mixed_configuration.presets[0].config.reverse_mode !== effects.NONE.index) {
        mixed_configuration.presets[0].config.reverse_censor_types = [mixed_configuration.presets[0].config.reverse_mode];
        mixed_configuration.presets[0].config.reverse_censor_preset_indexes = [0];
    } else {
        mixed_configuration.presets[0].config.reverse_censor_types = [];
        mixed_configuration.presets[0].config.reverse_censor_preset_indexes = [];
    }

    switch (reverse_mode_configuration.process_no_result) {
        case false:
            switch (reverse_mode_configuration.process_no_result_censor_selection) {
                case 0:
                    reverse_mode_configuration.mode = 2;
                case 1:
                    reverse_mode_configuration.mode = 1;
                case 2:
                    reverse_mode_configuration.mode = 2;
            }
            break;
        case true:
            reverse_mode_configuration.mode = 0;
            break;
    }

    renameKey(word_wall_configuration, 'text', 'word_pool');
    renameKey(word_wall_configuration, 'text_mode', 'draw_mode');
    renameKey(word_wall_configuration, 'font_size', 'size');
    renameKey(word_wall_configuration, 'distance_horizontal', 'horizontal_spacing');
    renameKey(word_wall_configuration, 'distance_vertical', 'vertical_spacing');
}

function repairConfigurations(settings_version = browser.runtime.getManifest().version) {
    if (settings_version == null || compareVersions(settings_version.replace("B", ''), "0.7.6.1") <= 0) {
        repairConfigurationFrom7_6_1();
    }

    let configs = [
        bar_configuration,
        blur_configuration,
        pixel_configuration,
        glitch_configuration,
        triangle_configuration,
        box_configuration,
        sticker_configuration,
        sobel_configuration,
        splatter_configuration,
        mixed_configuration,
        random_configuration,
        gif_configuration,
        reverse_mode_configuration,
        clustering_configuration,
        word_wall_configuration,
        only_once_mode_configuration,
        lock_configuration,
        icon_configuration,
        remote_configuration,
    ];

    let defaults = [
        default_bar_configuration,
        default_blur_configuration,
        default_pixel_configuration,
        default_glitch_configuration,
        default_triangle_configuration,
        default_box_configuration,
        default_sticker_configuration,
        default_sobel_configuration,
        default_splatter_configuration,
        default_mixed_configuration,
        default_random_configuration,
        default_gif_configuration,
        default_reverse_mode_configuration,
        default_clustering_configuration,
        default_word_wall_configuration,
        default_only_once_mode_configuration,
        default_lock_configuration,
        default_icon_configuration,
        default_remote_configuration,
    ];

    for (let i = 0; i < configs.length; i++) {
        for (let prop in defaults[i]) {
            if (Object.prototype.hasOwnProperty.call(defaults[i], prop)) {
                if (configs[i][prop] == null) {
                    configs[i][prop] = defaults[i][prop];
                }
            }
        }
        for (let prop in configs[i]) {
            if (!Object.prototype.hasOwnProperty.call(defaults[i], prop)) {
                delete configs[i][prop];
                log("DELETED CONFIG PROPERTY", prop);
            }
        }

        // Remove and init properties of presets
        if (defaults[i].presets?.[0] != null) {
            if (configs[i].presets == null) {
                configs[i].presets = defaults[i].presets;
                continue;
            }
            for (let preset_prop in defaults[i].presets?.[0].config) {
                if (Object.prototype.hasOwnProperty.call(defaults[i].presets[0].config, preset_prop)) {
                    for (let config_preset of configs[i].presets) {
                        if (config_preset == null) continue;
                        if (config_preset.config[preset_prop] == null) {
                            config_preset.config[preset_prop] = defaults[i].presets[0].config[preset_prop];
                        }
                    }
                }
            }
            for (let config_preset of configs[i].presets) {
                if (config_preset == null) continue;
                for (let preset_prop in config_preset.config) { 
                    if (!Object.prototype.hasOwnProperty.call(defaults[i].presets[0].config, preset_prop)) {
                        delete config_preset.config[preset_prop];
                        log("DELETED PRESET CONFIG PROPERTY", preset_prop);
                    }
                }
            }
            
        }
    }

    // Repair OOM sub configs
    for (let prop in only_once_mode_configuration.mode_configuration) {
        if (Object.prototype.hasOwnProperty.call(default_only_once_mode_configuration.mode_configuration, prop)) {
            if (only_once_mode_configuration.mode_configuration[prop] === null || typeof only_once_mode_configuration.mode_configuration[prop] === "undefined") {
                only_once_mode_configuration.mode_configuration[prop] = default_only_once_mode_configuration.mode_configuration[prop];
            }
            for (let sub_prop in default_only_once_mode_configuration.mode_configuration[prop]) {
                if (only_once_mode_configuration.mode_configuration[prop][sub_prop] === null || typeof only_once_mode_configuration.mode_configuration[prop][sub_prop] === "undefined") {
                    only_once_mode_configuration.mode_configuration[prop][sub_prop] = default_only_once_mode_configuration.mode_configuration[prop][sub_prop];
                }
            }
        }
    }

    return browser.storage.sync.set({
        bar_configuration: bar_configuration,
        blur_configuration: blur_configuration,
        pixel_configuration: pixel_configuration,
        glitch_configuration: glitch_configuration,
        triangle_configuration: triangle_configuration,
        box_configuration: box_configuration,
        sticker_configuration: sticker_configuration,
        sobel_configuration: sobel_configuration,
        splatter_configuration: splatter_configuration,
        mixed_configuration: mixed_configuration,
        random_configuration: random_configuration,
        gif_configuration: gif_configuration,
        reverse_mode_configuration: reverse_mode_configuration,
        clustering_configuration: clustering_configuration,
        word_wall_configuration: word_wall_configuration,
        only_once_mode_configuration: only_once_mode_configuration,
        lock_configuration: lock_configuration,
        icon_configuration: icon_configuration,
        remote_configuration: remote_configuration,
        version: browser.runtime.getManifest().version,
    });
}

function bundleConfiguration(){
    return {
        bar_configuration: bar_configuration,
        blur_configuration: blur_configuration,
        pixel_configuration: pixel_configuration,
        glitch_configuration: glitch_configuration,
        triangle_configuration: triangle_configuration,
        box_configuration: box_configuration,
        sticker_configuration: sticker_configuration,
        sobel_configuration: sobel_configuration,
        splatter_configuration: splatter_configuration,
        mixed_configuration: mixed_configuration,
        random_configuration: random_configuration,
        gif_configuration: gif_configuration,
        reverse_mode_configuration: reverse_mode_configuration,
        word_wall_configuration: word_wall_configuration,
        clustering_configuration: clustering_configuration,
        sticker_collections: sticker_collections,
        lock_configuration: lock_configuration,
        icon_configuration: icon_configuration,
        remote_configuration: remote_configuration,
    };
}

function hexConverter(hex) {
    return hex.replace('#','0xFF');
}

const default_bar_configuration = 
    new CensorTypeConfig("bar", [generateDefaultPreset[effects.BAR.index]()]);
var bar_configuration = default_bar_configuration;

const default_pixel_configuration = 
    new CensorTypeConfig("pixel", [generateDefaultPreset[effects.PIXEL.index]()]);
var pixel_configuration = default_pixel_configuration;

const default_blur_configuration = 
    new CensorTypeConfig("blur", [generateDefaultPreset[effects.BLUR.index]()]);
var blur_configuration = default_blur_configuration;

const default_triangle_configuration = 
    new CensorTypeConfig("triangle", [generateDefaultPreset[effects.TRIANGLE.index]()]);
var triangle_configuration = default_triangle_configuration;

const default_box_configuration = 
    new CensorTypeConfig("box", [generateDefaultPreset[effects.BOX.index]()]);
var box_configuration = default_box_configuration;

const default_glitch_configuration = 
    new CensorTypeConfig("glitch", [generateDefaultPreset[effects.GLITCH.index]()]);
var glitch_configuration = default_glitch_configuration;

const default_sticker_configuration = 
    new StickerCensorTypeConfig("sticker", false, [generateDefaultPreset[effects.STICKER.index]()]);
var sticker_configuration = default_sticker_configuration;

const default_sobel_configuration = 
    new CensorTypeConfig("sobel", [generateDefaultPreset[effects.SOBEL.index]()]);
var sobel_configuration = default_sobel_configuration;

const default_splatter_configuration = 
    new CensorTypeConfig("splatter", [generateDefaultPreset[effects.SPLATTER.index]()]);
var splatter_configuration = default_splatter_configuration;

const default_mixed_configuration = 
    new CensorTypeConfig("mixed", [generateDefaultPreset[effects.MIXED.index]()]);
var mixed_configuration = default_mixed_configuration;

const default_random_configuration = 
    new CensorTypeConfig("random", [generateDefaultPreset[effects.RANDOM.index]()]);
var random_configuration = default_random_configuration;

const default_gif_configuration = new GIFTypeConfiguration("gif", 0, true, true, 1000, 50000, 10000000);
var gif_configuration = default_gif_configuration;

const default_png_configuration = new PNGTypeConfiguration("png");
var png_configuration = default_png_configuration;

const default_jpg_configuration = new JPGTypeConfiguration("jpg");
var jpg_configuration = default_jpg_configuration;

const default_bmp_configuration = new BMPTypeConfiguration("bmp");
var bmp_configuration = default_bmp_configuration;

const default_webp_configuration = new WEBPTypeConfiguration("webp");
var webp_configuration = default_webp_configuration;

const default_avif_configuration = new AVIFTypeConfiguration("avif");
var avif_configuration = default_avif_configuration;

const default_whiteblacklist_configuration = new WhiteBlackListConfiguration();
var whiteblacklist_configuration = default_whiteblacklist_configuration;

const default_clustering_configuration = new ClusteringConfiguration(false, 0);
var clustering_configuration = default_clustering_configuration;

const default_reverse_mode_configuration = new ReverseModeConfiguration(false, 0);
var reverse_mode_configuration = default_reverse_mode_configuration;

const default_word_wall_configuration = new WordWallConfiguration(false, ["PURYFI"], 0, 'Segoe UI',
1, 4, 1.5, 1, 0, 0, "#FFFFFF");
var word_wall_configuration = default_word_wall_configuration;

const default_remote_configuration = new RemoteConfiguration();
var remote_configuration = default_remote_configuration;

const default_only_once_mode_configuration = new OnlyOnceModeConfiguration();
var only_once_mode_configuration = default_only_once_mode_configuration;

const default_only_once_mode_storage_manager = new OnlyOnceModeStorageManager();
var only_once_mode_storage_manager = default_only_once_mode_storage_manager;

const default_icon_configuration = {
    "16": '/icons/icon_16.png',
    "32": '/icons/icon_32.png',
    "64": '/icons/icon_64.png',
    "save_mode": true, // Don't you mean safe mode?
};
var icon_configuration = JSON.parse(JSON.stringify(default_icon_configuration));

const default_lock_configuration = new LockConfiguration();
var lock_configuration = default_lock_configuration;

const default_look_and_feel_configuration = new LookAndFeelConfiguration();
var look_and_feel_configuration = default_look_and_feel_configuration;

function restoreDefaultConfiguration(leave_out = []){
    if(!leave_out.includes("bar_configuration"))            bar_configuration       = default_bar_configuration;
    if(!leave_out.includes("pixel_configuration"))          pixel_configuration     = default_pixel_configuration;
    if(!leave_out.includes("blur_configuration"))           blur_configuration      = default_blur_configuration;
    if(!leave_out.includes("glitch_configuration"))         glitch_configuration    = default_glitch_configuration;
    if(!leave_out.includes("triangle_configuration"))       triangle_configuration  = default_triangle_configuration;
    if(!leave_out.includes("box_configuration"))            box_configuration       = default_box_configuration;
    if(!leave_out.includes("sticker_configuration"))        sticker_configuration   = default_sticker_configuration;
    if(!leave_out.includes("sobel_configuration"))          sobel_configuration     = default_sobel_configuration;
    if(!leave_out.includes("splatter_configuration"))       splatter_configuration  = default_splatter_configuration;
    if(!leave_out.includes("mixed_configuration"))          mixed_configuration     = default_mixed_configuration;
    if(!leave_out.includes("random_configuration"))          random_configuration   = default_random_configuration;

    if(!leave_out.includes("gif_configuration"))            gif_configuration       = default_gif_configuration;
    if(!leave_out.includes("png_configuration"))            png_configuration       = default_png_configuration;
    if(!leave_out.includes("jpg_configuration"))            jpg_configuration       = default_jpg_configuration;
    if(!leave_out.includes("bmp_configuration"))            bmp_configuration       = default_bmp_configuration;
    if(!leave_out.includes("webp_configuration"))           webp_configuration      = default_webp_configuration;
    if(!leave_out.includes("avif_configuration"))           avif_configuration      = default_avif_configuration;
    if(!leave_out.includes("whiteblacklist_configuration")) whiteblacklist_configuration    = default_whiteblacklist_configuration;
    if(!leave_out.includes("reverse_mode_configuration"))   reverse_mode_configuration      = default_reverse_mode_configuration;
    if(!leave_out.includes("clustering_configuration"))     clustering_configuration        = default_clustering_configuration;
    if(!leave_out.includes("only_once_mode_configuration")) only_once_mode_configuration    = default_only_once_mode_configuration;
    if(!leave_out.includes("lock_configuration"))           lock_configuration              = default_lock_configuration;
    if(!leave_out.includes("word_wall_configuration"))      word_wall_configuration         = default_word_wall_configuration;
    if(!leave_out.includes("remote_configuration"))         remote_configuration            = default_remote_configuration;
    if(!leave_out.includes("look_and_feel_configuration"))  look_and_feel_configuration     = default_look_and_feel_configuration;


    censor_type = 'black';
    browser.storage.sync.set({
        censor_type: 'black',
        labels : ["BELLYEXPOSED",
            "BELLYCOVERED",
            "BUTTOCKSEXPOSED",
            "BUTTOCKSCOVERED",
            "FEMALEBREASTEXPOSED",
            "FEMALEBREASTCOVERED",
            "FEMALEGENITALIAEXPOSED",
            "FEMALEGENITALIACOVERED",
            "MALEGENITALIACOVERED",
            "MALEGENITALIAEXPOSED",
            "MALEBREASTEXPOSED",
            "MALEBREASTCOVERED",
            "FEETCOVERED",
            "FEETEXPOSED",
            "ARMPITSCOVERED",
            "ARMPITSEXPOSED",
            "ANUSCOVERED",
            "ANUSEXPOSED"],
        file_types: ['jpg', 'png', 'bmp', 'webp', 'avif'],
        debug: false,
        do_cache: true,
        active: true,
        prescale: true,
        base64_scanner: true,
        bar_configuration : bar_configuration,
        pixel_configuration : pixel_configuration,
        blur_configuration : blur_configuration,
        glitch_configuration : glitch_configuration,
        triangle_configuration : triangle_configuration,
        box_configuration: box_configuration,
        sticker_configuration : sticker_configuration,
        sobel_configuration : sobel_configuration,
        splatter_configuration: splatter_configuration,
        mixed_configuration: mixed_configuration,
        random_configuration: random_configuration,
        gif_configuration : gif_configuration,
        png_configuration : png_configuration,
        jpg_configuration : jpg_configuration,
        bmp_configuration : bmp_configuration,
        webp_configuration : webp_configuration,
        avif_configuration: avif_configuration,
        whiteblacklist_configuration : whiteblacklist_configuration,
        reverse_mode_configuration : reverse_mode_configuration,
        clustering_configuration : clustering_configuration,
        word_wall_configuration: word_wall_configuration,
        only_once_mode_configuration: only_once_mode_configuration,
        remote_configuration: remote_configuration,
        lock_configuration: lock_configuration,
        statistics: statistics,
        look_and_feel_configuration: look_and_feel_configuration,
    });
}


//Todo: needs better rules and context awareness e.g.
// box = big -> do not scale much
// low scale -> do not feather too much
let rnd_rules = {
    'scale': {'min': 1.0, 'max': 1.8, 'average': 1.3},
    'shape': [0,1,2],
    'feathering': {'min': 0, 'max': 50, 'average': 0, 'integer': true},
    'color': true,
    'spacing': {'min': 0, 'max': 5, 'integer': true},
    'type': [0,1,2],
    'strength': {'min': 3, 'max': 50, 'average': 20, 'integer': true},
    //'resolution_aware': [1], this freezes triang, need extra rule to downscale vertex count with this
    'channel_colors': {'min': 1, 'max': 256, 'integer': true},
    // 'detection_size_aware': [1],
    'grayscale': [0,0,0,0,0,1],
    'blurring': {'min': 3, 'max': 50, 'average': 20, 'integer': true},
    'inverted_color': [true, false],
    'centering': [1,2,3,4,5,6,7,8,9,10],
    'opacity': {'min': 0.1, 'max': 1, 'average': 0.8},
    'color_pool': true,
    'color_mode': [0,1],
    'amount': [1,1,2,2,3,3,3,4,4,4,4,5,5,5,5,6,6,6,7,7,8,9,10],
    'size': {'min': 0.5, 'max': 5, 'average':1},
    'sub_amount': [1,2,3,4,5,6,6,7,7,8,8,9,9,10,10,10,11,11,12,12,13,13,14,15],
    'sub_size': {'min': 0.5, 'max': 5, 'average':1},

    'accuracy': {'min': 0, 'max': 1, 'average': 0.7},
    // 'blur': {'min': 0, 'max': 100, 'average': 0},
    'gradients': [true, false, false, false, false, false, false, false, false, false],
    'stroke_width': { 'min': 0, 'max': 5, 'average': 0.5 },
    'threshold': {'min': 0, 'max': 100, 'average': 50, 'integer': true },
    'vertex_count': {'min': 500, 'max': 3000, 'average': 1000, 'integer': true },
    // 'vertexMode': [false],
    //'fill': [true, true, true, true, true, true, true, true, true, false],
    'fill': [1,1,1,1,1,0],
    'border': [0,1],
    'border_color': true,
    'display': [0,0,0,1,2,3,3,3],
    'fill_color_mode': [0,0,0,0,0,1],
    'stroke_color_mode': [0,0,0,0,0,1],
    'stroke_color': true,
    'fill_color': true,
}

function getRandomColor(transparency = false) {
    const letters = '0123456789ABCDEF';
    let color = '#';
    for (let i = 0; i < 6; i++) {
        color += letters[Math.floor(Math.random() * 16)];
    }
    let alphaHex = '';
    if(transparency) {
        const alpha = Math.floor(Math.random() * 256); // Generate a random alpha value (0 to 255)
        alphaHex = alpha.toString(16).toUpperCase(); // Convert alpha to hexadecimal
    }
    return color + alphaHex;
}

function randomizeConfigObject(obj) {
    const randomizedObj = {...obj};
    if(user && user.patreon_tier >= 0){
        rnd_rules['shape'] = [0,1,2,3]; //todo: move this logik to patreon.js
    }
    for (const key in obj) {
        // Check if the key exists in the rules object
        if (key in rnd_rules) {
            if (key === 'color_pool' && rnd_rules[key] === true) {
                const randomColorCount = Math.floor(Math.random() * 5) + 1;
                const colors = [];
                for (let i = 0; i < randomColorCount; i++) {
                    colors.push({'color': getRandomColor(true), weight: Math.random() < 0.5 ? 1 : 0});
                }
                randomizedObj[key] = colors;
            }else if ((key === 'color' || key === 'fill_color' || key === 'stroke_color' || key === 'border_color') && rnd_rules[key] === true) {
                // If the key is '_color' and the rule is set to true, generate a random color
                randomizedObj[key] = getRandomColor();
            } else if (Array.isArray(rnd_rules[key])) {
                // If the rule is an array, select a random value from it
                const randomIndex = Math.floor(Math.random() * rnd_rules[key].length);
                randomizedObj[key] = rnd_rules[key][randomIndex];
            } else if (typeof obj[key] === 'number' && Number.isFinite(obj[key])) {
                if (typeof rnd_rules[key] === 'object' && 'min' in rnd_rules[key] && 'max' in rnd_rules[key]
                    //&& 'average' in rnd_rules[key]
                ) {
                    // If the rule is an object with 'min', 'max', and 'average', generate a weighted random value within that range
                    const min = rnd_rules[key].min;
                    const max = rnd_rules[key].max;
                    const average = rnd_rules[key].average ?? null;
                    
                    randomizedObj[key] = rnd_rules[key].integer === true ? Math.min(Math.max(Math.floor(weightedRandom(min, max, average)), min), max) : weightedRandom(min, max, average);
                }
            }
        }
    }
    return randomizedObj;
}

function weightedRandom(min, max, average) {
    if(!average){
        return Math.random() * (max - min) + min;
    }
    const range = max - min;
    const weightedRange = (range * 0.6); // Adjust the factor to control the weight
    const randomValue = Math.random() * weightedRange + average - weightedRange / 2;
    return Math.min(Math.max(randomValue, min), max);
}
