let remote_check_timestamp = 0; // can be loaded from last-checked

let remote_check_interval = null;

let remote_check_interval_ms = 30000;

function initialize_remote_interval(){
    if((remote_configuration.check_mode === 0 || remote_configuration.check_mode === 1) && remote_configuration.remote_token_identifier !== null){
        remote_configuration_check();
    }
    update_remote_interval();
}

function update_remote_interval(){
    remote_check_timestamp = remote_configuration.last_checked ?? 0;
    if(remote_check_interval != null){
        clearInterval(remote_check_interval);
    }
    if(remote_configuration.remote_token_identifier !== null && remote_configuration.check_mode === 0) {
        remote_check_interval = setInterval(function () {
            if(remote_configuration.remote_token_identifier === null){
                clearInterval(remote_check_interval);
                return;
            }
            remote_configuration_check();
        }, remote_check_interval_ms);
    }
}


function remote_configuration_check() {
        let now = Date.now();
        if (now - remote_check_timestamp > Math.max(remote_check_interval_ms, remote_configuration.check_period) //remote_check_interval_ms
        ) {
            periodicRemoteSettingsRefresh().then(() => {
            }).catch(e => {
                log(e)
            }).finally(() => {
                remote_check_timestamp = now;
                remote_configuration.last_checked = remote_check_timestamp;
                browser.storage.sync.set({
                    remote_configuration: remote_configuration
                });
            });
        }
}

function periodicRemoteSettingsRefresh(){
    return new Promise((resolve, reject) => {
        if (remote_configuration.remote_token_identifier != null) {
            let remote_identifier = remote_configuration.remote_token_identifier;
            puryfiNetworkRequest("https://pury.fi/site/wp-json/remote_control/settings_info", {
                'username' : encodeURIComponent(user.username),
                'password' : encodeURIComponent(user.password),
                'identifier': encodeURIComponent(remote_identifier),
            }).then((data) => {
                let last_updated_externally = utcToLocalDate(new Date(data.last_updated)).getTime();
                let last_updated_internally = remote_configuration.last_updated;
                if (data.identifier && (last_updated_externally > last_updated_internally)) {
                    // Settings might have been changed
                    puryfiNetworkRequest("https://pury.fi/site/wp-json/cloud_settings/load",{
                        'identifier': data.identifier,
                    }).then((data) => {
                        importRemoteSettings(data['settings'], data['version'], remote_configuration.allow_older_settings).then(()=>{
                            resolve(data);
                        }).catch((e) => {
                            reject(e);
                        });
                    }).catch(e => {
                        reject(e);
                    });
                } else if (data.identifier === null) {
                    // No Settings attached -> Do nothing
                    reject('NO_SETTINGS');
                }else{
                    reject('UP_TO_DATE');
                }
            }).catch(e => {
                if (e === 'INVALIDATED') {
                    // Do Nothing
                }
                reject(e);
            });
        }
    });
}

function importRemoteSettings(data, version, allow_older_version){
    return new Promise((resolve, reject) => {
        let v = version.replace("B", "");
        let compare = compareVersions(browser.runtime.getManifest().version, v);
        if (compare === 0 || (compare >= 0 && allow_older_version) ) {
            const base64String = decrypt(version, data);
            try {
                const decodedString = atob(base64String);
                const settings_import = JSON.parse(decodedString);
                let c = browser.storage.sync.get(null);
                c.then((res) => {
                    let setting = {};
                    let keys = Object.keys(res);
                    for (const property in settings_import) {
                        if (shape_settings.includes(property)) {
                            settingsCheck(res.user, [settings_import[property]]);
                        }
                        if (keys.includes(property) && !skip_settings.includes(property)) {
                            setting[property] = settings_import[property];
                            if(property === 'lock_configuration' && settings_import[property].enabled){
                                setting[property].remote_lock = true;
                            }
                        }
                    }

                    let affected_settings = Object.keys(setting).join(', ');
                    browser.storage.sync.set(setting).then(function () {
                        remote_configuration.last_updated = Date.now();
                        browser.storage.sync.set({
                            remote_configuration: remote_configuration
                        }).then(() => {
                            updateOptions(false).then(() => {
                                repairConfigurations();
                            });
                            resolve(affected_settings);
                        });
                    });
                });
            } catch (e) {
                reject(e);
            }
        }else if (compare !== 0) {
            // Settings are too new or too old
            reject('SETTINGS_VERSION_MISMATCH');
        }
    });
}

function puryfiNetworkRequest(url, parameters){
    return new Promise((resolve, reject) => {
        if(user == null){
            return;
        }
        let xmlhttp = new XMLHttpRequest();
        let params = '';
        for (const [key, value] of Object.entries(parameters)) {
            if(params !== ''){
                params += '&';
            }
            params += key+'='+value;
        }
        xmlhttp.onreadystatechange = function () {
            if (this.readyState == 4 && this.status == 200) {
                try {
                    let data = JSON.parse(this.responseText);
                    if (data) {
                        if (data.hasOwnProperty('error')) {
                            reject(data.error);
                            return;
                        } else {
                            resolve(data);
                        }
                    }
                } catch (e) {
                    reject(e);
                    return;
                }
            }
        };
        xmlhttp.ontimeout = function (e) {
            reject(e);
            return;
        };
        xmlhttp.open('POST', url, true);
        xmlhttp.timeout = 1000;
        xmlhttp.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
        xmlhttp.send(params);
    });
}
