// Don't polute the global namespace
(function () {
  // Don't want to clash with the local file handler
  if (location.protocol === "file:") return;

  let pre_observer_config = {
    childList: true,
    subtree: true,
    attributes: true,
    attributeFilter: ["src"],
  };

  let pre_blurred_elems = [];
  let pre_observer = new MutationObserver((mutations) => {
    for (let mutation of mutations) {
      if (mutation.addedNodes) {
        for (let addedNode of mutation.addedNodes) {
          if (addedNode.nodeName.toLowerCase() === "img") {
            let img_elem = $(addedNode);
            if (img_elem.prop("src")?.match(/^(data:image|blob:)/)) {
              img_elem.css("filter", "blur(50px)");
              pre_blurred_elems.push(addedNode);
            }
          }
        }
      }

      if (mutation.removedNodes) {
        for (let removedNode of mutation.removedNodes) {
          if (removedNode.nodeName.toLowerCase() === "img") {
            let index = pre_blurred_elems.indexOf(removedNode);
            if (index !== -1) {
              pre_blurred_elems.splice(index, 1);
            }
          }
        }
      }

      if (
        mutation.type === "attributes" &&
        mutation.target.nodeName.toLowerCase() === "img"
      ) {
        let img_elem = $(mutation.target);
        if (img_elem.prop("src")?.match(/^(data:image|blob:)/)) {
          img_elem.css("filter", "blur(50px)");
          pre_blurred_elems.push(mutation.target);
        }
      }
    }
  });
  pre_observer.observe(document.documentElement, pre_observer_config);

  browser.storage.sync
    .get(["active", "base64_scanner", "whiteblacklist_configuration"])
    .then(function (res) {
      let observer_config = {
        childList: true,
        subtree: true,
        attributes: true,
        attributeFilter: ["src"],
      };

      let observer = new MutationObserver(async (mutations) => {
        let elems_to_censor = [];
        for (let mutation of mutations) {
          if (mutation.addedNodes) {
            for (let addedNode of mutation.addedNodes) {
              if (addedNode.nodeName.toLowerCase() === "img") {
                let img_elem = $(addedNode);
                if (img_elem.prop("src")?.match(/^(data:image|blob:)/)) {
                  img_elem.css("filter", "blur(50px)");
                  elems_to_censor.push(img_elem);
                }
              }
            }
          }

          if (
            mutation.type === "attributes" &&
            mutation.target.nodeName.toLowerCase() === "img"
          ) {
            let img_elem = $(mutation.target);
            if (img_elem.prop("src")?.match(/^(data:image|blob:)/)) {
              img_elem.css("filter", "blur(50px)");
              elems_to_censor.push(img_elem);
            }
          }
        }
        elems_to_censor.forEach(handleImage);
      });

      let { active, base64_scanner } = res;
      let listed = checkWhiteBlacklist(
        res.whiteblacklist_configuration,
        window.location.origin,
        window.location.origin
      );

      pre_observer.disconnect();
      if (active && base64_scanner && listed) {
        $.unique(pre_blurred_elems).forEach((elem) => {
          handleImage($(elem));
        });
        observer.observe(document.documentElement, observer_config);
      } else {
        pre_blurred_elems.forEach((elem) => $(elem).css("filter", ""));
      }
      pre_blurred_elems = [];

      // NOTE: we keep up with the settings which determine if the observer is on actively rather than lazily, this is so images get blurred as soon as possible, without the observer first having to await a call to storage
      browser.storage.onChanged.addListener(function (changes, area_name) {
        if (area_name === "sync") {
          active = changes.active?.newValue ?? active;
          base64_scanner = changes.base64_scanner?.newValue ?? base64_scanner;
          if (changes.whiteblacklist_configuration !== undefined) {
            listed = checkWhiteBlacklist(
              changes.whiteblacklist_configuration.newValue,
              window.location.origin,
              window.location.origin
            );
          }

          if (active && base64_scanner && listed) {
            observer.observe(document.body, observer_config);
          } else {
            observer.disconnect();
          }
        }
      });

      function handleImage(img_elem) {
        let url = img_elem.prop("src");
        if (url == null) return;

        if (url.startsWith("data:image")) {
          toDataURL(url, function (data, contenttype) {
            let blob = new Blob([data]);
            censorBlob(blob, url, img_elem);
          });
        } else {
          // Must be a blob
          fetch(url)
            .then((response) => response.blob())
            .then((blob) => {
              censorBlob(blob, url, img_elem);
            })
            .catch((error) => console.error(error));
        }
      }

      function censorBlob(blob, url, img_elem) {
        let fileReader = new FileReader();
        fileReader.onloadend = function (e) {
          let arr = new Uint8Array(e.target.result);
          let header = "";
          for (let i = 0; i < arr.length; i++) {
            header += arr[i].toString(16);
          }
          checkMimeHeader(
            header,
            async function (type, ext, header, cont) {
              let res = await browser.storage.sync.get([
                "file_types",
                "labels",
                "png_configuration",
                "jpg_configuration",
                "webp_configuration",
                "avif_configuration",
                "censor_type",
                "bar_configuration",
                "blur_configuration",
                "pixel_configuration",
                "gif_configuration",
                "glitch_configuration",
                "triangle_configuration",
                "box_configuration",
                "sticker_configuration",
                "splatter_configuration",
                "sobel_configuration",
                "mixed_configuration",
                "random_configuration",
                "reverse_mode_configuration",
                "word_wall_configuration",
                "clustering_configuration",
                "lock_configuration",
              ]);

              let full_type = type;
              //Repair short content types
              if (res.file_types.includes(type)) {
                full_type = "image/" + type;
              }
              if (
                !processableContentType(
                  full_type,
                  res.file_types,
                  res.gif_configuration
                )
              ) {
                img_elem.css("filter", "");
                return;
              }

              let imageData = new Image();
              imageData.onload = async function () {
                let out = document.createElement("canvas");
                out.width = this.naturalWidth;
                out.height = this.naturalHeight;
                let outctx = out.getContext("2d");
                outctx.drawImage(imageData, 0, 0, out.width, out.height);
                let frame = outctx.getImageData(0, 0, out.width, out.height);

                let ai_res = await browser.runtime.sendMessage({
                  ai_request: true,
                  frame: frame,
                  width: out.width,
                  height: out.height,
                });

                let { valid_detections, classes, scores, boxes } = ai_res;
                if (
                  full_type === "image/gif" &&
                  !res.gif_configuration._thumbnails
                ) {
                  let gif_res = browser.runtime.sendMessage({
                    gif_request: true,
                    data: [valid_detections, classes, scores, boxes],
                    blob: blob,
                  });
                  assignCensoredBlob(img_elem, gif_res.blob);
                } else {
                  if (
                    [
                      "triangle",
                      "sticker",
                      "mixed",
                      "random",
                      "splatter",
                    ].includes(res.censor_type)
                  ) {
                    let paint_res = await browser.runtime.sendMessage({
                      paint_request: true,
                      data: [valid_detections, classes, scores, boxes],
                      blob: blob,
                    });
                    assignCensoredBlob(img_elem, paint_res.blob);
                  } else {
                    let img_config = extractImgConfig(res, full_type);
                    if(!checkImageFileSize(img_config, blob.size)){
                      log("rejected by checkImageFileSize:", img_config, full_type, blob.size);
                      img_elem.css('filter', '');
                      return;
                    }
                    if(!checkImageDimensions(img_config, this.naturalWidth, this.naturalHeight)){
                      log("rejected by checkImageDimensions:", img_config, full_type, this.naturalWidth, this.naturalHeight);
                      img_elem.css('filter', '');
                      return;
                    }

                    let cache = {};
                    paintCensor(
                      out,
                      outctx,
                      res.censor_type,
                      res.labels,
                      valid_detections,
                      classes,
                      scores,
                      boxes,
                      res,
                      cache
                    );

                    out.toBlob(function (blob) {
                      assignCensoredBlob(img_elem, blob);
                    }, img_config._file_output_type);
                  }
                }
              };
              imageData.src = url;
            },
            e.target.result
          );
        };
        fileReader.readAsArrayBuffer(blob.slice(0, 4));
      }

      function assignCensoredBlob(img_elem, blob) {
        let censored_url = URL.createObjectURL(blob);
        observer.disconnect();
        img_elem.css("background-color", "#00000000");
        img_elem.one("load", function () {
          img_elem.css("filter", "");
          // URL.revokeObjectURL(censored_url);
        });
        img_elem.attr("src", censored_url);
        observer.observe(document.body, observer_config);
      }
    }).catch((err) => {
      console.error("Error in base64-handler: " + err);
      pre_observer.disconnect();
      pre_blurred_elems.forEach((elem) => $(elem).css("filter", ""));
    });
})();
