$(document).ready(function() {
    if(location.protocol === 'file:' && puryfi_active){
        let file_url = $("img").attr("src");
        censorLocalFile(file_url, null);
    }
})

function censorLocalFile(file_url, censor_type = null) {
    toDataURL(file_url, function (data, contenttype) {
        let blob = new Blob([data]);
        let fileReader = new FileReader();
        fileReader.onloadend = function(e) {
            let arr = new Uint8Array(e.target.result);
            let header = '';
            for (let i = 0; i < arr.length; i++) {
                header += arr[i].toString(16);
            }
            checkMimeHeader(header, function (type, ext, header, cont){
                let options = browser.storage.sync.get(['file_types', 'png_configuration','jpg_configuration', 'webp_configuration', 'avif_configuration', 'censor_type','bar_configuration', 'labels', 'censor_type',
                    'blur_configuration','pixel_configuration','gif_configuration','glitch_configuration',
                    'triangle_configuration','box_configuration','sticker_configuration','sobel_configuration','splatter_configuration',
                    'mixed_configuration', 'random_configuration',
                    'reverse_mode_configuration','word_wall_configuration','clustering_configuration','lock_configuration']);
                options.then((res) => {
                    if(!censor_type){
                        censor_type = res.censor_type;
                    }
                    let full_type = type;
                    //Repair short content types
                    if(res.file_types.includes(type)){
                        full_type = 'image/' + type;
                    }
                    if (!processableContentType(full_type, res.file_types, res.gif_configuration)) {
                        return;
                    }
                    $("img").css('filter','blur(50px)');
                    let image = new Image();
                    image.onload = function () {
                        let out = document.createElement('canvas');
                        out.width = this.naturalWidth;
                        out.height = this.naturalHeight;
                        let outctx = out.getContext('2d');
                        outctx.drawImage(image, 0, 0, out.width, out.height);
                        let frame = outctx.getImageData(0, 0, out.width, out.height);
                        let sending = browser.runtime.sendMessage({
                            ai_request: true,
                            local_file: true,
                            frame: frame,
                            width: out.width,
                            height: out.height
                        });
                        sending.then((response) => {
                            let valid_detections = response.valid_detections;
                            let classes = response.classes;
                            let scores = response.scores;
                            let boxes = response.boxes;
                            if(full_type === "image/gif" && !res.gif_configuration._thumbnails){
                                let gif_request = browser.runtime.sendMessage({
                                    gif_request: true,
                                    blob: blob,
                                });
                                gif_request.then(gif_response => {
                                    let censored_url = URL.createObjectURL(gif_response.blob);
                                    $("img").css('background-color', '#00000000');
                                    $("img").attr("src", censored_url);
                                    $("img").css('filter', '');
                                });
                            }else{
                                if(censor_type === "triangle" || censor_type === "sticker" ||
                                    censor_type === "mixed" || censor_type === "random" || censor_type === "splatter") {
                                    let paint_request = browser.runtime.sendMessage({
                                        paint_request: true,
                                        blob: blob,
                                        censor_type: censor_type,
                                        content_type_header: full_type,
                                        ext: ext,
                                    });
                                    paint_request.then(paint_response => {
                                        let censored_url = URL.createObjectURL(paint_response.blob);
                                        $("img").css('background-color', '#00000000');
                                        $("img").attr("src", censored_url);
                                        $("img").css('filter', '');
                                    });
                                }else{
                                    let img_elem = $("img");
                                    let img_config = extractImgConfig(res, full_type);
                                    if(!checkImageFileSize(img_config, blob.size)){
                                        log("rejected by checkImageFileSize:", img_config, full_type, blob.size);
                                        img_elem.css('filter', '');
                                        return;
                                    }
                                    if(!checkImageDimensions(img_config, this.naturalWidth, this.naturalHeight)){
                                        log("rejected by checkImageDimensions:", img_config, full_type, this.naturalWidth, this.naturalHeight);
                                        img_elem.css('filter', '');
                                        return;
                                    }

                                    let cache = {};
                                    paintCensor(out, outctx, censor_type, res.labels, valid_detections, classes, scores, boxes, res, cache);

                                    out.toBlob(function(blob) {
                                        let censored_url = URL.createObjectURL(blob);
                                        img_elem.css('background-color', '#00000000');
                                        img_elem.attr("src", censored_url);
                                        img_elem.css('filter', '');
                                    }, img_config._file_output_type);
                                }
                            }
                        });
                    }
                    image.src = file_url;
                });
            }, e.target.result);
        };
        fileReader.readAsArrayBuffer(blob.slice(0, 4))
    });
}