$(".color-scheme-picker").on("change", function (event) {
   saveTheme();
});

function restoreTheme() {
   let labels = browser.storage.sync.get(["look_and_feel_configuration"]);
   labels.then((res) => {
      applyTheme(res);

      $("#main_background_color_picker").val(
         res.look_and_feel_configuration.main_bg_color
      );
      $("#main_background_color_picker").css(
         "background-color",
         res.look_and_feel_configuration.main_bg_color
      );

      $("#secondary_background_color_picker").val(
         res.look_and_feel_configuration.secondary_bg_color
      );
      $("#secondary_background_color_picker").css(
         "background-color",
         res.look_and_feel_configuration.secondary_bg_color
      );

      $("#secondary_background_variant_color_picker").val(
         res.look_and_feel_configuration.secondary_bg_color_variant_1
      );
      $("#secondary_background_variant_color_picker").css(
         "background-color",
         res.look_and_feel_configuration.secondary_bg_color_variant_1
      );

      $("#main_accent_color_picker").val(
         res.look_and_feel_configuration.main_accent_color
      );
      $("#main_accent_color_picker").css(
         "background-color",
         res.look_and_feel_configuration.main_accent_color
      );

      $("#font_color_picker").val(
         res.look_and_feel_configuration.main_text_color
      );
      $("#font_color_picker").css(
         "background-color",
         res.look_and_feel_configuration.main_text_color
      );
   });
}

function hexToRGBA(hex, alpha) {
   const isValidHex = (hex) => /^#([A-Fa-f0-9]{3,4}){1,2}$/.test(hex);

   const getChunksFromString = (st, chunkSize) =>
      st.match(new RegExp(`.{${chunkSize}}`, "g"));

   const convertHexUnitTo256 = (hexStr) =>
      parseInt(hexStr.repeat(2 / hexStr.length), 16);

   const getAlphafloat = (a, alpha) => {
      if (typeof a !== "undefined") {
         return a / 255;
      }
      if (typeof alpha != "number" || alpha < 0 || alpha > 1) {
         return 1;
      }
      return alpha;
   };

   if (!isValidHex(hex)) {
      throw new Error("Invalid HEX");
   }
   const chunkSize = Math.floor((hex.length - 1) / 3);
   const hexArr = getChunksFromString(hex.slice(1), chunkSize);
   const [r, g, b, a] = hexArr.map(convertHexUnitTo256);
   return { r: r, g: g, b: b, a: getAlphafloat(a, alpha) };
   //return `rgba(${r}, ${g}, ${b}, ${getAlphafloat(a, alpha)})`
}

function rgba2hex(rgb) {
   let hex =
      (rgb.r | (1 << 8)).toString(16).slice(1) +
      (rgb.g | (1 << 8)).toString(16).slice(1) +
      (rgb.b | (1 << 8)).toString(16).slice(1);
   let a = rgb.a;
   if (a === undefined) {
      return "#" + hex;
   }
   // multiply before convert to HEX
   a = ((a * 255) | (1 << 8)).toString(16).slice(1);
   hex = hex + a;

   return "#" + hex;
}

function calcColorDistanceRGB(main_color, variant_color) {
   let main = hexToRGBA(main_color);
   let variant = hexToRGBA(variant_color);
   let r = main.r - variant.r;
   let g = main.g - variant.g;
   let b = main.b - variant.b;
   let a = main.a - variant.a;
   if (!a) {
      a = 0;
   }
   let merge = {
      r: r,
      g: g,
      b: b,
      a: a,
   };
   return merge;
}
function addColorDistanceRGB(main_color, dif) {
   let main = hexToRGBA(main_color);
   if (dif.a === undefined) {
      dif.a = 0;
   }
   if (main.a === undefined) {
      main.a = 1;
   }
   main.r -= dif.r;
   main.g -= dif.g;
   main.b -= dif.b;
   main.a -= dif.a;

   main.r = Math.min(255, Math.max(0, main.r));
   main.g = Math.min(255, Math.max(0, main.g));
   main.b = Math.min(255, Math.max(0, main.b));
   /*
    main.r = Math.max(0,main.r) % 255;
    main.g = Math.max(0,main.g) % 255;
    main.b = Math.max(0,main.b) % 255;
    */
   main.a = Math.min(1, Math.max(0, main.a));
   return main;
}

function calcColorDistance(main_color, variant_color) {
   main_color = main_color.replace("#", "");
   variant_color = variant_color.replace("#", "");
   main_color = parseInt(main_color, 16);
   variant_color = parseInt(variant_color, 16);
   return main_color - variant_color;
   //return Math.abs(main_color-variant_color);
}

function addColorDistance(main_color, dif, subs = 1) {
   main_color = main_color.replace("#", "");
   main_color = parseInt(main_color, 16);
   dif = Math.abs(dif);
   dif = dif * subs;
   let variant_color = main_color + dif;
   //variant_color = Math.abs(variant_color);
   variant_color = Math.min(16777215, Math.max(0, variant_color));
   let variant_color_str = variant_color.toString(16);
   while (variant_color_str.length < 6) {
      variant_color_str = "0" + variant_color_str;
   }
   return "#" + variant_color_str;
}

function addHsl(val, diff) {
   let { h, s, l } = val;
   h = (h + diff.h) % 360;
   s = Math.min(100, Math.max(0, s + diff.s));
   l = Math.min(100, Math.max(0, l + diff.l));
   return { h, s, l };
}

function addColorInHslSpace(a, b) {
   let result = addHsl(a, b);
   return `hsl(${result.h}, ${result.s}%, ${result.l}%)`;
}

function hexToHsl(val) {
   var result = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(val);
   r = parseInt(result[1], 16);
   g = parseInt(result[2], 16);
   b = parseInt(result[3], 16);
   (r /= 255), (g /= 255), (b /= 255);
   var max = Math.max(r, g, b),
      min = Math.min(r, g, b);
   var h,
      s,
      l = (max + min) / 2;
   if (max == min) {
      h = s = 0; // achromatic
   } else {
      var d = max - min;
      s = l > 0.5 ? d / (2 - max - min) : d / (max + min);
      switch (max) {
         case r:
            h = (g - b) / d + (g < b ? 6 : 0);
            break;
         case g:
            h = (b - r) / d + 2;
            break;
         case b:
            h = (r - g) / d + 4;
            break;
      }
      h /= 6;
   }
   var HSL = new Object();
   HSL["h"] = h * 360;
   HSL["s"] = s * 100;
   HSL["l"] = l * 100;
   return HSL;
}

function calcHslColorDifference(val, other) {
   val = hexToHsl(val);
   other = hexToHsl(other);
   let h =
      other.s !== 0 && other.l !== 0 && other.l !== 100 ? other.h - val.h : 0;
   let s = other.s - val.s;
   let l = other.l - val.l;
   return { h, s, l };
}

function addHslColorDifference(val, diff) {
   let { h, s, l } = hexToHsl(val);
   h = (h + diff.h) % 360;
   s = Math.min(100, Math.max(0, s + diff.s));
   l = Math.min(100, Math.max(0, l + diff.l));
   return `hsl(${h}, ${s}%, ${l}%)`;
}

function applyTheme(res) {
   let r = document.querySelector(":root");
   r.style.setProperty(
      "--main-background-color",
      res.look_and_feel_configuration.main_bg_color
   );
   $("#main_background_color_picker").val(
      res.look_and_feel_configuration.main_bg_color
   );
   $("#main_background_color_picker").css(
      "background-color",
      res.look_and_feel_configuration.main_bg_color
   );

   r.style.setProperty(
      "--secondary-background-color",
      res.look_and_feel_configuration.secondary_bg_color
   );
   $("#secondary_background_color_picker").val(
      res.look_and_feel_configuration.secondary_bg_color
   );
   $("#secondary_background_color_picker").css(
      "background-color",
      res.look_and_feel_configuration.secondary_bg_color
   );

   r.style.setProperty(
      "--secondary-background-color-variant-1",
      res.look_and_feel_configuration.secondary_bg_color_variant_1
   );
   $("#secondary_background_variant_color_picker").val(
      res.look_and_feel_configuration.secondary_bg_color_variant_1
   );
   $("#secondary_background_variant_color_picker").css(
      "background-color",
      res.look_and_feel_configuration.secondary_bg_color_variant_1
   );

   r.style.setProperty(
      "--secondary-background-color-variant-2",
      res.look_and_feel_configuration.secondary_bg_color_variant_2
   );
   r.style.setProperty(
      "--secondary-background-color-variant-3",
      res.look_and_feel_configuration.secondary_bg_color_variant_3
   );
   r.style.setProperty(
      "--secondary-background-color-variant-4",
      res.look_and_feel_configuration.secondary_bg_color_variant_4
   );

   r.style.setProperty(
      "--main-accent-color",
      res.look_and_feel_configuration.main_accent_color
   );
   $("#main_accent_color_picker").val(
      res.look_and_feel_configuration.main_accent_color
   );
   $("#main_accent_color_picker").css(
      "background-color",
      res.look_and_feel_configuration.main_accent_color
   );

   r.style.setProperty(
      "--main-accent-color-variant-1",
      res.look_and_feel_configuration.main_accent_color_variant_1
   );
   r.style.setProperty(
      "--main-accent-color-variant-2",
      res.look_and_feel_configuration.main_accent_color_variant_2
   );
   r.style.setProperty(
      "--main-accent-color-variant-3",
      res.look_and_feel_configuration.main_accent_color_variant_3
   );
   r.style.setProperty(
      "--main-accent-color-variant-4",
      res.look_and_feel_configuration.main_accent_color_variant_4
   );

   r.style.setProperty(
      "--main-input-color",
      res.look_and_feel_configuration.main_input_color
   );
   r.style.setProperty(
      "--main-input-color-variant-1",
      res.look_and_feel_configuration.main_input_color_variant_1
   );

   r.style.setProperty(
      "--main-border-color",
      res.look_and_feel_configuration.main_border_color
   );
   r.style.setProperty(
      "--border-color-variant-1",
      res.look_and_feel_configuration.border_color_variant_1
   );
   r.style.setProperty(
      "--border-color-variant-2",
      res.look_and_feel_configuration.border_color_variant_2
   );
   r.style.setProperty(
      "--border-color-variant-3",
      res.look_and_feel_configuration.border_color_variant_3
   );

   r.style.setProperty(
      "--main-text-color",
      res.look_and_feel_configuration.main_text_color
   );
   r.style.setProperty(
      "--main-text-color-disabled",
      res.look_and_feel_configuration.main_text_color_disabled
   );

   r.style.setProperty(
      "--tab-text-color",
      res.look_and_feel_configuration.tab_text_color
   );
}

function saveTheme(override = null) {
   let labels = browser.storage.sync.get(["look_and_feel_configuration"]);
   labels.then((res) => {
      res.look_and_feel_configuration.main_bg_color = $(
         "#main_background_color_picker"
      ).val();

      res.look_and_feel_configuration.secondary_bg_color = $(
         "#secondary_background_color_picker"
      ).val();

      res.look_and_feel_configuration.main_accent_color = $(
         "#main_accent_color_picker"
      ).val();

      res.look_and_feel_configuration.main_text_color =
         $("#font_color_picker").val();

      res.look_and_feel_configuration.secondary_bg_color_variant_1 =
         addHslColorDifference(
            res.look_and_feel_configuration.secondary_bg_color,
            calcHslColorDifference("#151719", "#2c3135")
         );

      res.look_and_feel_configuration.secondary_bg_color_variant_2 =
         addHslColorDifference(
            res.look_and_feel_configuration.secondary_bg_color,
            calcHslColorDifference("#151719", "#212428")
         );

      res.look_and_feel_configuration.secondary_bg_color_variant_3 =
         addHslColorDifference(
            res.look_and_feel_configuration.secondary_bg_color,
            calcHslColorDifference("#151719", "#14171a")
         );

      res.look_and_feel_configuration.secondary_bg_color_variant_4 =
         addHslColorDifference(
            res.look_and_feel_configuration.secondary_bg_color,
            calcHslColorDifference("#151719", "#15181b")
         );

      res.look_and_feel_configuration.main_accent_color_variant_1 =
         addHslColorDifference(
            res.look_and_feel_configuration.main_accent_color,
            calcHslColorDifference("#009879", "#017a62")
         );

      res.look_and_feel_configuration.main_accent_color_variant_2 =
         addHslColorDifference(
            res.look_and_feel_configuration.main_accent_color,
            calcHslColorDifference("#009879", "#0b5e50")
         );

      res.look_and_feel_configuration.main_accent_color_variant_3 =
         addHslColorDifference(
            res.look_and_feel_configuration.main_accent_color,
            calcHslColorDifference("#009879", "#213e3a")
         );

      res.look_and_feel_configuration.main_accent_color_variant_4 =
         addHslColorDifference(
            res.look_and_feel_configuration.main_accent_color,
            calcHslColorDifference("#009879", "#4ac193")
         );

      res.look_and_feel_configuration.main_input_color =
         res.look_and_feel_configuration.secondary_bg_color_variant_1;

      res.look_and_feel_configuration.main_input_color_variant_1 =
         addHslColorDifference(
            res.look_and_feel_configuration.secondary_bg_color,
            calcHslColorDifference("#151719", "#1f2a2d")
         );

      res.look_and_feel_configuration.main_border_color = addHslColorDifference(
         res.look_and_feel_configuration.main_text_color,
         calcHslColorDifference("#bbc9df", "#bbc9df")
      );

      res.look_and_feel_configuration.border_color_variant_1 =
         addHslColorDifference(
            res.look_and_feel_configuration.secondary_bg_color,
            calcHslColorDifference("#151719", "#646d78")
         );

      res.look_and_feel_configuration.border_color_variant_2 =
         addHslColorDifference(
            res.look_and_feel_configuration.secondary_bg_color,
            calcHslColorDifference("#151719", "#3a3f45")
         );

      res.look_and_feel_configuration.border_color_variant_3 =
         addHslColorDifference(
            res.look_and_feel_configuration.secondary_bg_color,
            calcHslColorDifference("#151719", "#111111")
         );

      res.look_and_feel_configuration.tab_text_color =
         res.look_and_feel_configuration.main_text_color;

      let main_text_color_hsl = hexToHsl(
         res.look_and_feel_configuration.main_text_color
      );
      res.look_and_feel_configuration.main_text_color_disabled = `hsla(${main_text_color_hsl.h}, ${main_text_color_hsl.s}%, ${main_text_color_hsl.l}%, 0.6)`;

      override?.(res);
      applyTheme(res);

      browser.storage.sync.set({
         look_and_feel_configuration: res.look_and_feel_configuration,
      });
   });
}

$("#tealThemeButton").on("click", function () {
   $("#main_background_color_picker").val("#0B0E11");
   $("#main_background_color_picker").css("background-color", "#0B0E11");
   $("#secondary_background_color_picker").val("#151719");
   $("#secondary_background_color_picker").css("background-color", "#151719");
   $("#main_accent_color_picker").val("#009879");
   $("#main_accent_color_picker").css("background-color", "#009879");
   $("#font_color_picker").val("#BBC9DF");
   $("#font_color_picker").css("background-color", "#BBC9DF");
   saveTheme();
});

$("#blueThemeButton").on("click", function () {
   let labels = browser.storage.sync.get(["look_and_feel_configuration"]);
   labels.then((res) => {
      res.look_and_feel_configuration.main_bg_color = "#1A1127";
      $("#main_background_color_picker").val("#1A1127");
      $("#main_background_color_picker").css("background-color", "#1A1127");

      res.look_and_feel_configuration.secondary_bg_color = "#0E2337";
      $("#secondary_background_color_picker").val("#0E2337");
      $("#secondary_background_color_picker").css(
         "background-color",
         "#0E2337"
      );

      res.look_and_feel_configuration.main_accent_color = "#166082";
      $("#main_accent_color_picker").val("#166082");
      $("#main_accent_color_picker").css("background-color", "#166082");

      res.look_and_feel_configuration.main_text_color = "#BBC9DF";
      $("#font_color_picker").val("#BBC9DF");
      $("#font_color_picker").css("background-color", "#BBC9DF");

      res.look_and_feel_configuration.tab_text_color = "#BBC9DF";

      let main_text_color_hsl = hexToHsl(
         res.look_and_feel_configuration.main_text_color
      );
      res.look_and_feel_configuration.main_text_color_disabled = `hsla(${main_text_color_hsl.h}, ${main_text_color_hsl.s}%, ${main_text_color_hsl.l}%, 0.6)`;

      res.look_and_feel_configuration.secondary_bg_color_variant_1 = "#284259";
      res.look_and_feel_configuration.secondary_bg_color_variant_2 = "#193046";
      res.look_and_feel_configuration.secondary_bg_color_variant_3 = "#0D2238";
      res.look_and_feel_configuration.secondary_bg_color_variant_4 = "#0E2439";

      res.look_and_feel_configuration.main_accent_color_variant_1 = "#134C66";
      res.look_and_feel_configuration.main_accent_color_variant_2 = "#153B51";
      res.look_and_feel_configuration.main_accent_color_variant_3 = "#00354C";
      res.look_and_feel_configuration.main_accent_color_variant_4 = "#728AC4";

      res.look_and_feel_configuration.main_input_color = "#253D53";
      res.look_and_feel_configuration.main_input_color_variant_1 = "#082029";

      res.look_and_feel_configuration.main_border_color = "#ABCDEF";
      res.look_and_feel_configuration.border_color_variant_1 = "#3680C5";
      res.look_and_feel_configuration.border_color_variant_2 = "#306297";
      res.look_and_feel_configuration.border_color_variant_3 = "#0E1D2B";

      applyTheme(res);
      browser.storage.sync.set({
         look_and_feel_configuration: res.look_and_feel_configuration,
      });
   });
});

$("#purpleThemeButton").on("click", function () {
   $("#main_background_color_picker").val("#000000");
   $("#main_background_color_picker").css("background-color", "#000000");
   $("#secondary_background_color_picker").val("#120519");
   $("#secondary_background_color_picker").css("background-color", "#120519");
   $("#main_accent_color_picker").val("#530281");
   $("#main_accent_color_picker").css("background-color", "#530281");
   $("#font_color_picker").val("#A03DD9");
   $("#font_color_picker").css("background-color", "#A03DD9");
   saveTheme();
});

$("#greenThemeButton").on("click", function () {
   $("#main_background_color_picker").val("#000000");
   $("#main_background_color_picker").css("background-color", "#000000");
   $("#secondary_background_color_picker").val("#0E0B11");
   $("#secondary_background_color_picker").css("background-color", "#0E0B11");
   $("#main_accent_color_picker").val("#02834E");
   $("#main_accent_color_picker").css("background-color", "#02834E");
   $("#font_color_picker").val("#BBC9DF");
   $("#font_color_picker").css("background-color", "#BBC9DF");
   saveTheme();
});

$("#cyberpunkThemeButton").on("click", function () {
   let labels = browser.storage.sync.get(["look_and_feel_configuration"]);
   labels.then((res) => {
      res.look_and_feel_configuration.main_bg_color = "#F3E600";
      $("#main_background_color_picker").val("#F3E600");
      $("#main_background_color_picker").css("background-color", "#F3E600");

      res.look_and_feel_configuration.secondary_bg_color = "#0A262D";
      $("#secondary_background_color_picker").val("#0A262D");
      $("#secondary_background_color_picker").css(
         "background-color",
         "#0A262D"
      );

      res.look_and_feel_configuration.main_accent_color = "#0070FF";
      $("#main_accent_color_picker").val("#0070FF");
      $("#main_accent_color_picker").css("background-color", "#0070FF");

      res.look_and_feel_configuration.main_text_color = "#F5FF00";
      $("#font_color_picker").val("#F5FF00");
      $("#font_color_picker").css("background-color", "#F5FF00");

      res.look_and_feel_configuration.tab_text_color = "#0071C5";

      let main_text_color_hsl = hexToHsl(
         res.look_and_feel_configuration.main_text_color
      );
      res.look_and_feel_configuration.main_text_color_disabled = `hsla(${main_text_color_hsl.h}, ${main_text_color_hsl.s}%, ${main_text_color_hsl.l}%, 0.6)`;

      res.look_and_feel_configuration.secondary_bg_color_variant_1 = "#030303";
      res.look_and_feel_configuration.secondary_bg_color_variant_2 = "#06060C";
      res.look_and_feel_configuration.secondary_bg_color_variant_3 = "#060D24";
      res.look_and_feel_configuration.secondary_bg_color_variant_4 = "#001126";

      res.look_and_feel_configuration.main_accent_color_variant_1 = "#0260C8";
      res.look_and_feel_configuration.main_accent_color_variant_2 = "#104999";
      res.look_and_feel_configuration.main_accent_color_variant_3 = "#003075";
      res.look_and_feel_configuration.main_accent_color_variant_4 = "#00C3FF";

      res.look_and_feel_configuration.main_input_color = "#260004";
      res.look_and_feel_configuration.main_input_color_variant_1 = "#00161C";

      res.look_and_feel_configuration.main_border_color = "#54D7FF";
      res.look_and_feel_configuration.border_color_variant_1 = "#0086C8";
      res.look_and_feel_configuration.border_color_variant_2 = "#0070E6";
      res.look_and_feel_configuration.border_color_variant_3 = "#003395";

      applyTheme(res);
      browser.storage.sync.set({
         look_and_feel_configuration: res.look_and_feel_configuration,
      });
   });
});

$("#pinkDollThemeButton").on("click", function () {
   $("#main_background_color_picker").val("#FF36AF");
   $("#main_background_color_picker").css("background-color", "#FF36AF");
   $("#secondary_background_color_picker").val("#FF5CBE");
   $("#secondary_background_color_picker").css("background-color", "#FF5CBE");
   $("#main_accent_color_picker").val("#FFC4F3");
   $("#main_accent_color_picker").css("background-color", "#FFC4F3");
   $("#font_color_picker").val("#FFEEF7");
   $("#font_color_picker").css("background-color", "#FFEEF7");
   saveTheme();
});

$("#pinkThemeButton").on("click", function () {
   let labels = browser.storage.sync.get(["look_and_feel_configuration"]);
   labels.then((res) => {
      res.look_and_feel_configuration.main_bg_color = "#5F2744";
      $("#main_background_color_picker").val("#5F2744");
      $("#main_background_color_picker").css("background-color", "#5F2744");

      res.look_and_feel_configuration.secondary_bg_color = "#421B2F";
      $("#secondary_background_color_picker").val("#421B2F");
      $("#secondary_background_color_picker").css(
         "background-color",
         "#421B2F"
      );

      res.look_and_feel_configuration.main_accent_color = "#DF55D0";
      $("#main_accent_color_picker").val("#DF55D0");
      $("#main_accent_color_picker").css("background-color", "#DF55D0");

      res.look_and_feel_configuration.main_text_color = "#FFD9FF";
      $("#font_color_picker").val("#FFD9FF");
      $("#font_color_picker").css("background-color", "#FFD9FF");

      res.look_and_feel_configuration.tab_text_color =
         res.look_and_feel_configuration.main_text_color;

      let main_text_color_hsl = hexToHsl(
         res.look_and_feel_configuration.main_text_color
      );
      res.look_and_feel_configuration.main_text_color_disabled = `hsla(${main_text_color_hsl.h}, ${main_text_color_hsl.s}%, ${main_text_color_hsl.l}%, 0.6)`;

      res.look_and_feel_configuration.secondary_bg_color_variant_1 = "#DE577E";
      res.look_and_feel_configuration.secondary_bg_color_variant_2 = "#C43690";
      res.look_and_feel_configuration.secondary_bg_color_variant_3 = "#411A30";
      res.look_and_feel_configuration.secondary_bg_color_variant_4 = "#421C31";

      res.look_and_feel_configuration.main_accent_color_variant_1 = "#FB00A2";
      res.look_and_feel_configuration.main_accent_color_variant_2 = "#C70E80";
      res.look_and_feel_configuration.main_accent_color_variant_3 = "#862C7C";
      res.look_and_feel_configuration.main_accent_color_variant_4 = "#842D76";

      res.look_and_feel_configuration.main_input_color = "#31132D";
      res.look_and_feel_configuration.main_input_color_variant_1 = "#483043";

      res.look_and_feel_configuration.main_border_color = "#F97EDD";
      res.look_and_feel_configuration.border_color_variant_1 = "#FF90FF";
      res.look_and_feel_configuration.border_color_variant_2 = "#FF6390";
      res.look_and_feel_configuration.border_color_variant_3 = "#752957";

      applyTheme(res);
      browser.storage.sync.set({
         look_and_feel_configuration: res.look_and_feel_configuration,
      });
   });
});
