$("[data-exact-clicks").on("click mousedown mouseup", function (e) {
   if (e.target !== this) {
      e.preventDefault();
      e.stopPropagation();
      e.stopImmediatePropagation();
   }
});

function updateInputRadio($inputs, val, do_highlight) {
   let $input_to_enable = $inputs.filter(`[value="${val}"]`);
   if (!$input_to_enable.prop("checked")) {
      if (do_highlight) animateHighlight($inputs);
      $inputs.prop("checked", false);
      $input_to_enable.prop("checked", true);
   }
}

function updateInputCheckbox($input, val, do_highlight) {
   if (do_highlight && $input.is(":checked") !== val) {
      animateHighlight($input);
   }

   $input.prop("checked", val);
}

// Sets the value of an input and validate it, doesn't trigger a change event
function updateInputField($input, val, do_highlight) {
   if (do_highlight && $input.val() !== val.toString()) {
      if ($input.is('.general-setting-field input[type="text"]')) {
         animateHighlight($input.closest(".general-setting-field"));
      } else {
         animateHighlight($input);
      }
   }

   $input.val(val);
   $input.removeAttr("data-fallback-value");
   validateInputValueOnChange.call($input);
}

function removeMixedCensorType($entry, do_highlight) {
   if ($entry.attr("data-censor-type") == null) return;

   let $select = $entry.children(".mixed-entry-select");
   if (do_highlight) {
      animateHighlight($select.add($entry.find(".mixed-entry-button")));
   }
   $entry.removeAttr("data-censor-type");

   $select.attr("disabled", true);
   $select.find("a").html("&nbsp");
   $select.find("ul").empty();
}

function setMixedCensorType($entry, censor_type, do_highlight) {
   if (parseInt($entry.attr("data-censor-type")) === censor_type) return;

   let $button = $entry.find(".mixed-entry-button");
   let $select = $entry.children(".mixed-entry-select");
   if (do_highlight) animateHighlight($button.add($select));

   $entry.attr("data-censor-type", censor_type);

   $select.removeAttr("disabled");

   const $select_items_container = $select.children("ul");
   $select_items_container.empty();
   for (
      let preset_index = 0;
      preset_index < censor_presets[censor_type]?.length;
      preset_index++
   ) {
      if (censor_presets[censor_type][preset_index] == null) continue;
      $select_items_container.append(
         `<li value="${preset_index}"><a>${
            escapeHTMLSpecialChars(
               censor_presets[censor_type][preset_index].name
            ) || "&nbsp"
         }</a></li>`
      );
   }
   setMixedCensorPreset($select, 0);
}

function setMixedCensorPreset($select, val, do_highlight) {
   if (parseInt($select.attr("value")) !== val && do_highlight) {
      animateHighlight($select);
   }

   $select.attr("value", val);
   $select.children("a").html($select.find(`ul li[value="${val}"] a`).html());
}

function doInputStep($target, delta_y, shift_key, ctrl_key) {
   let val = parseFloat($target.val());

   let step = parseFloat($target.attr("data-input-step"));
   if (step && $target.is(":focus")) {
      // TODO: hanndle step being a decimal when input type is integral
      // NOTE: every precision fix is for the sake of avoiding rounding errors
      if (shift_key) {
         step = parseFloat((step / 10).toFixed(7));
      } else if (ctrl_key) {
         step *= 10;
      }

      const precision = $target.attr("data-input-precision");
      if (precision == null) {
         const input_type = $target.attr("data-input-type");
         if (input_type === "int" || input_type === "absolute int") {
            step = Math.max(1, step);
         }
      } else {
         step =
            0 <= precision
               ? Math.max(fix(1 / (precision + 1)), step)
               : Math.max(1, step);
      }

      if (0 < delta_y) {
         val = fix((Math.ceil(fix(val / step)) - 1) * step);
      } else {
         val = fix((Math.floor(fix(val / step)) + 1) * step);
      }

      $target.val(val);
      $target.trigger("change");

      // Set precision of value to precision of step
      val = $target.val();
      val = parseFloat(val).toFixed(countDecimals(step.toString(), true));

      $target.val(val);
      return true;
   }
   return false;
}

function approveInputKeypress(code) {
   let val = $(this).val();

   const input_type = $(this).attr("data-input-type");
   if (input_type != null) {
      if (input_type === "int" || input_type === "absolute int") {
         const selectionStart = $(this).prop("selectionStart");
         const selectionEnd = $(this).prop("selectionEnd");
         const notSelectedValue =
            val.substring(0, selectionStart) + val.substring(selectionEnd);

         // If it's numeric
         if (48 <= code && code <= 57) {
            let val_len = notSelectedValue.replace("-", "").length;

            // If there is padding, remove it from the length
            let padding_char = $(this).attr("data-padding-char");
            if (padding_char != null) {
               let padding_position = $(this).attr("data-padding-position");
               if (padding_position === "start") {
                  val_len -=
                     val.match(new RegExp(`^${padding_char}+`))?.[0].length ??
                     0;
               } else {
                  val_len -=
                     val.match(new RegExp(`${padding_char}+$`))?.[0].length ??
                     0;
               }
            }

            // Reject if the max number of integral digits has already been reached
            const max_int_digits = Math.max(
               ...($(this)
                  .attr("data-input-range")
                  ?.split(",")
                  .map((n) => Math.abs(parseInt(n)).toString().length) ?? [])
            );
            if (max_int_digits <= val_len) {
               return false;
            }
         }
         // If it's a minus sign and the type is not absolute
         else if (input_type === "int" && code === 45) {
            // Reject if not typing at the first character or if there is another minus sign already, excluding any in the selected area
            if (notSelectedValue.indexOf("-") !== -1 || selectionStart !== 0) {
               return false;
            }
         }
         // Reject if it's none of the previous
         else {
            return false;
         }
      } else if (input_type === "float" || input_type === "absolute float") {
         const selectionStart = $(this).prop("selectionStart");
         const selectionEnd = $(this).prop("selectionEnd");
         const notSelectedValue =
            val.substring(0, selectionStart) + val.substring(selectionEnd);

         // If it's numeric
         if (48 <= code && code <= 57) {
            const maxIntegralDigits = Math.max(
               ...($(this)
                  .attr("data-input-range")
                  ?.split(",")
                  .map((n) => Math.abs(parseInt(n)).toString().length) ?? [])
            );
            const maxDecimalDigits = parseInt(
               $(this).attr("data-input-precision")
            );

            // Reject if the relevant max number of digits has already been reached
            const indexOfDot = notSelectedValue.indexOf(".");
            if (indexOfDot !== -1) {
               if (selectionStart <= indexOfDot) {
                  if (maxIntegralDigits <= indexOfDot) {
                     return false;
                  }
               } else {
                  if (
                     maxDecimalDigits <=
                     notSelectedValue.length - 1 - indexOfDot
                  ) {
                     return false;
                  }
               }
            } else {
               if (
                  maxIntegralDigits <= notSelectedValue.replace("-", "").length
               ) {
                  return false;
               }
            }
         }
         // If it's a dot
         else if (code === 46) {
            // Reject if there is another dot already, excluding any in the selected area
            if (notSelectedValue.indexOf(".") !== -1) {
               return false;
            }
         }
         // If it's a minus sign and the type is not absolute
         else if (input_type === "float" && code === 45) {
            // Reject if not typing at the first character or if there is another minus sign already, excluding any in the selected area
            if (notSelectedValue.indexOf("-") !== -1 || selectionStart !== 0) {
               return false;
            }
         }
         // Reject if it's none of the previous
         else {
            return false;
         }
      } else if (input_type === "color") {
         const selectionStart = $(this).prop("selectionStart");
         const selectionEnd = $(this).prop("selectionEnd");
         const notSelectedValue =
            val.substring(0, selectionStart) + val.substring(selectionEnd);
         // If it's a hashtag
         if (code === 35) {
            // Reject if not typing at the first character or if there is another hashtag already, excluding any in the selected area
            if (notSelectedValue.indexOf("#") !== -1 || selectionStart !== 0) {
               return false;
            }
         } // Reject if it's also not numeric, A-F or a-f
         else if (
            (48 <= code && code <= 57) ||
            (65 <= code && code <= 70) ||
            (97 <= code && code <= 102)
         ) {
            // Reject if the max number of chars has already been reached
            if (8 <= notSelectedValue.replace("#", "").length) {
               return false;
            }
         } else {
            return false;
         }
      }
   }

   return true;
}

function validateInputValueOnInput() {
   let val = $(this).val();

   const max_int_digits = Math.max(
      ...($(this)
         .attr("data-input-range")
         ?.split(",")
         .map((n) => Math.abs(parseInt(n)).toString().length) ?? [])
   );

   let padding_char = $(this).attr("data-padding-char");
   if (padding_char != null && max_int_digits !== -Infinity) {
      let padding_position = $(this).attr("data-padding-position");
      if (padding_position === "start") {
         val = val
            .padStart(max_int_digits, padding_char)
            .slice(-max_int_digits);
      } else {
         val = val
            .padEnd(max_int_digits, padding_char)
            .slice(0, max_int_digits);
      }
   }

   $(this).val(val);
}

function validateInputValueOnChange() {
   let val = $(this).val();
   let fallback = $(this).attr("data-fallback-value");
   if (fallback != null && val === fallback) return;

   fallback = fallback || 0;

   const input_type = $(this).attr("data-input-type");
   if (input_type != null) {
      switch (input_type) {
         case "int": {
            val = parseInt(val);
            if (isNaN(val)) {
               $(this).val(fallback);
               return;
            }
            break;
         }
         case "absolute int": {
            val = parseInt(val);
            if (isNaN(val)) {
               $(this).val(fallback);
               return;
            }

            if (val < 0) {
               $(this).val(fallback);
               return;
            }
            break;
         }
         case "float": {
            val = parseFloat(val);
            if (isNaN(val)) {
               $(this).val(fallback);
               return;
            }

            const precision = $(this).attr("data-input-precision");
            if (precision != null) {
               val = parseFloat(val).toFixed(parseFloat(precision));
            }
            break;
         }
         case "absolute float": {
            val = parseFloat(val);
            if (isNaN(val)) {
               $(this).val(fallback);
               return;
            }

            let precision = $(this).attr("data-input-precision");
            if (precision != null) {
               val = parseFloat(val).toFixed(parseFloat(precision));
            }

            if (val < 0) {
               $(this).val(fallback);
               return;
            }
            break;
         }
         case "color": {
            val = val.toUpperCase();

            // Collapse any multiple hashtags
            val = val.replace(/#+/, "#");

            // Prepend a hashtag if there isn't one
            if (val.charAt(0) !== "#") val = "#" + val;

            // If the alpha part of the value is incomplete, remove that char
            if (val.length === 8) {
               val = val.slice(0, -1);
            }

            // Validate the value after applying all potential fixes, if it doesn't pass, revert to the last valid value
            if (!val.match(/^#([0-9A-F]{8}|[0-9A-F]{6})$/)) {
               $(this).val(fallback);
               return;
            }

            break;
         }
      }
   }

   const range = $(this).attr("data-input-range")?.split(",");
   if (range != null) {
      const [min, max] = range;
      val = clamp(parseFloat(val), parseFloat(min), parseFloat(max));
   }

   const max_int_digits = Math.max(
      ...($(this)
         .attr("data-input-range")
         ?.split(",")
         .map((n) => Math.abs(parseInt(n)).toString().length) ?? [])
   );

   let padding_char = $(this).attr("data-padding-char");
   if (padding_char != null && max_int_digits !== -Infinity) {
      let padding_position = $(this).attr("data-padding-position");
      if (padding_position === "start") {
         val = val.toString().padStart(max_int_digits, padding_char);
      } else {
         val = val.toString().padEnd(max_int_digits, padding_char);
      }
   }

   $(this).val(val);
   $(this).attr("data-fallback-value", val);
}

// NOTE: because jquery doesn't support wheel event
document.addEventListener(
   "wheel",
   function (e) {
      let $target = $(e.target).is("label")
         ? $(`#${$(e.target).attr("for")}`)
         : $(e.target);
      if (doInputStep($target, e.deltaY, e.shiftKey, e.ctrlKey)) {
         e.preventDefault();
         $target.trigger("input");
      }
   },
   { passive: false }
);

$(document)
   .keydown(function (e) {
      let delta_y = e.which === 38 ? -1 : e.which === 40 ? 1 : null;
      if (delta_y != null) {
         if (doInputStep($(e.target), delta_y, e.shiftKey, e.ctrlKey)) {
            e.preventDefault();
            $(e.target).trigger("input");
         }
      }
   })
   .keypress(function (e) {
      // Blur if key is enter
      if ($(e.target).is("input") && e.which === 13) {
         $(e.target).blur();
      } else if (!approveInputKeypress.call(e.target, e.which)) {
         e.preventDefault();
      }
      // NOTE: called both on change and focusout for the sake of reformatting values after doing steps, such as to remove redundant zeroes after a dot
   })
   .on("input", function (e) {
      if (!$(e.target).is('input[type="text"], input[type="number"]')) return;

      let $target = $(e.target);
      validateInputValueOnInput.call($target);
   })
   .on("change focusout", function (e) {
      if (!$(e.target).is('input[type="text"], input[type="number"]')) return;

      let $target = $(e.target);
      validateInputValueOnChange.call($target);
      if ($target.attr("data-input-type") === "color") {
         $target.css("background-color", $target.val());
      }

      resetHorScroll.call(e.target);
   })
   .change(function (e) {
      if ($(e.target).hasClass("save-config")) {
         saveConfig();
      } else if ($(e.target).hasClass("save-file-config")) {
         saveFileConfig();
      }
   });
