const LOCK_SALT = 26794318;

$('#lock_password_toggle_vis').click(function(){
    const type = $('#lock_password').attr('type') === 'password' ? 'text' : 'password';
    $('#lock_password').attr('type', type);
    $('#lock_confirm_password').attr('type', type);
    if(type == 'text'){
        $(this).find('.eye').hide();
        $(this).find('.eye-off').show();
    } else {
        $(this).find('.eye').show();
        $(this).find('.eye-off').hide();
    }
});

const password_valid_chars = [
    'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 
    'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', 
    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 
    'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', 
    '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 
    '!', '@', '#', '$', '%', '^', '&', '*', '(', ')', '_', '+', '-', 
    '=', '{', '}', '[', ']', '|', ':', ';', '<', '>', '?', '/', '.', 
    '`', '~'
];

$('#lock_password_randomize').click(function() {
    let rand_password = '';
    for (let i = 0; i < 20; i++) {
        const rand_i = Math.floor(Math.random() * password_valid_chars.length);
        rand_password += password_valid_chars[rand_i];
    }

    $('#lock_password').val(rand_password).trigger('input');
    $('#lock_confirm_password').val(rand_password).trigger('input');

    navigator.clipboard.writeText(rand_password)
        .catch((err) => {
            console.error('Failed to copy randomized password to clipboard: ' + err);
        });

    let $notification = $(document.createElement('div'));
    $notification.addClass('notification');
    $notification.html(getToolTipText('password-copied-to-clipboard'));
    $(this).append($notification);
    $notification.position({
        my: 'center bottom-12',
        at: 'center top',
        of: $(this),
        collision: 'fit fit',
    });

    animateHighlight($notification.add('#lock_password, #lock_confirm_password'));

    setTimeout(function() {
        $notification.animate({
            top: '-=6px',
            opacity: 0,
        }, {
            duration: 500,
            queue: false,
        }, function() {
            $notification.remove();
        });
    }, 750);
});

$('#lock_password, #lock_confirm_password').on('input',function(){
    checkLockSettings();
});

$('#lock-show-advanced').click(function(){
    $('#lock-advanced').toggle();
    $(this).find('.toggle-div').html($('#lock-advanced').is(':visible')?"-":"+");
});

$('.lock-timer-plus-icon').click(function(){
    $('.lock-timer-plus-popup').toggle();
});

$('#lock_password_check').on('click',function(){
    checkLockSettings();
});

$('#lock_timer_check').on('click',function(){
    if($(this).attr("check")){
        $('#lock-timer-plus-advanced').show();
    } else {
        $('#lock-timer-plus-advanced').hide();
    }
    checkLockSettings();
});

function checkLockSettings(){
    let failure_reason = '';
    let sum = getTimeSum();
    if(!$('#lock_timer_check').attr('check') && !$('#lock_password_check').attr('check')) {
        failure_reason = 'Cannot lock without either a timer or a password!';
    } else {
        if ($('#lock_password_check').attr('check')) {
            if ($('#lock_password').val() === '') {
                failure_reason = 'Password cannot be empty!';
            } else if ($('#lock_password').val() !== $('#lock_confirm_password').val()) {
                failure_reason = 'Password confirmation must match the password!';
            }
        }

        if($('#lock_timer_check').attr('check')) {
            if (sum <= 0){
                failure_reason = 'Timer cannot be set to no duration!';
            }
        }
    }
    if(!failure_reason){
        $('#lock-button').removeClass('disabled');
        $('#lock_button_disabled_reason').html('');
    }else{
        $('#lock_button_disabled_reason').html(failure_reason);
        $('#lock-button').addClass('disabled');
    }
    const currentDate = new Date();
    const newDate = new Date(currentDate.getTime() + sum);
    $('#lock-timer-date').html(newDate.toLocaleString());
}

$('.lock-timer-input').on('keyup keypress blur change',function(){
    checkLockSettings();
});

function getTimeSum() {
    let d = $('#lock-timer-days').val() > 0?$('#lock-timer-days').val()*24*60*60:0;
    let h = $('#lock-timer-hours').val() > 0?$('#lock-timer-hours').val()*60*60:0;
    let m = $('#lock-timer-minutes').val() > 0?$('#lock-timer-minutes').val()*60:0;
    return (d+h+m)*1000;
}

let security_token = null;

$('#unlock-token').on('input',function(){
    let unlock_token = $(this).val();
    if(security_token == null){
        let c = browser.storage.sync.get(['lock_configuration']);
        c.then((res) => {
            security_token = res.lock_configuration.token;
            if(checkUnlockToken(unlock_token,security_token)){
                unlockExtension();
            }
        });
    }else{
        if(checkUnlockToken(unlock_token,security_token)){
            unlockExtension();
        }
    }
});
$('#unlock-extension-patreon').click(function(){
    let user_req = browser.storage.sync.get(['user']);
    user_req.then((res) => {
        if (res.user != null && res.user.permissions.permission_unlock_extension <= res.user.patreon_tier) {
            unlockExtension();
        }
    });
});
$('#emergency_unlock_help').click(function(){
    $('#emergency_unlock_info').toggle();
});

function checkUnlockToken(unlock_token, security_token){
    let u = parseInt(unlock_token);
    let s = parseInt(security_token);
    if((""+unlock_token).length === (""+security_token).length &&
        validateUnlockToken(u,s) //&&
        //(u+s) % 2 == 0
    ){
        $('#unlock-token').removeClass('unlock-token-invalid');
        return true;
    }else{
        $('#unlock-token').removeClass('unlock-token-invalid').addClass('unlock-token-invalid');
        return false;
    }
}

function validateUnlockToken(unlock_token, security_token){
    let u = ""+unlock_token;
    let s = ""+security_token;
    for (let i = 0; i < s.length; i++) {
        let cs = s.charAt(i);
        let cu = u.charAt(i);
        let ns = parseInt(cs);
        let nu = parseInt(cu);
        if(((ns+nu)%10)%2 != i%2){
            return false;
        }
    }
    return true;
}

function generateUnlockToken(security_token){
    let s = ""+security_token;
    let r = "";
    for (let i = 0; i < s.length; i++) {
        let ung = [1,3,5,7,9].sort(function() {return 0.5 - Math.random()})
        let ger = [2,4,6,8].sort(function() {return 0.5 - Math.random()});
        let c = s.charAt(i);
        let n = parseInt(c);
        let goal = i%2;
        let p = ung[0];
        if(goal == 0){
            if(n%2 == 0){
                p = ger[0];
            }
        }else{
            if(n%2 == 1){
                p = ger[0];
            }
        }
        let d = p;
        r += d;
    }
    return r;
}

$('#unlock-button').click(function(){
    let pwd = $('#unlock_password').val();
    unlockExtension(pwd);
});

function unlockExtension(pwd = null){
    let c = browser.storage.sync.get(['lock_configuration']);
    c.then((res) => {
        if(pwd === null || pwd === res.lock_configuration.password || hashFnv32a(pwd, true, LOCK_SALT) === res.lock_configuration.password){
            res.lock_configuration.enabled = false;
            res.lock_configuration.password = null;
            res.lock_configuration.token = null;
            res.lock_configuration.duration = null;
            res.lock_configuration.remote_lock = false;
            res.lock_configuration.locked_options = [];
            browser.storage.sync.set({
                lock_configuration: res.lock_configuration,
            }).then(function(){
                updateOptionsContentScript();
                window.location.reload();
            });
        }else{
            alert("Wrong password!");
        }
    });
}

$("input:radio[name='lock-timer-format']").on("change",function (event) {
    if(parseInt($(this).val()) === 1){
        $('#lock-timer-date').show();
        const currentDate = new Date();
        const newDate = new Date(currentDate.getTime() + getTimeSum());
        $('#lock-timer-date').html(newDate.toLocaleString());
    }else{
        $('#lock-timer-date').hide();
    }
});

$('#lock-button').click(function(){
    let pwd = $('#lock_password').val();
    if(!$('#lock_password_check').attr('check')){
        pwd = null;
    }
    let timer_mode = parseInt($("input:radio[name='lock-timer-format']:checked").val());
    const d = $('#lock-timer-days').val() > 0?$('#lock-timer-days').val()*24*60*60:0;
    const h = $('#lock-timer-hours').val() > 0?$('#lock-timer-hours').val()*60*60:0;
    const m = $('#lock-timer-minutes').val() > 0?$('#lock-timer-minutes').val()*60:0;
    let sum = (d+h+m)*1000;
    if(!$('#lock_timer_check').attr('check')){
        sum = null;
    }
    if(pwd == null && sum == null){
        return;
    }
    let c = browser.storage.sync.get(['lock_configuration']);
    c.then((res) => {
        res.lock_configuration.enabled = true;
        let hash_pwd = null;
        if(pwd != null){
            hash_pwd = hashFnv32a(pwd, true, LOCK_SALT);
        }
        res.lock_configuration.password = hash_pwd;
        res.lock_configuration.token = generateSecurityToken(res.lock_configuration.password);
        res.lock_configuration.duration = Number.isInteger(sum) && sum != null && sum > 0? sum : null;
        res.lock_configuration.duration_timestamp = Number.isInteger(sum) && sum != null && sum > 0? Date.now() : null;
        if(timer_mode !== 1){
            res.lock_configuration.timer_mode = res.lock_configuration.duration_timestamp;
        }else{
            res.lock_configuration.timer_mode = 1;
        }

        if($('#lock-option-censor-type').is(":checked")){
            res.lock_configuration.locked_options.push('censor_type');
        }
        if($('#lock-option-video').is(":checked")){
            res.lock_configuration.locked_options.push('video');
        }
        if($('#lock-option-settings').is(":checked")){
            res.lock_configuration.locked_options.push('settings');
        }
        res.lock_configuration.timer_plus = $('.timer-plus-toggle').attr('check')? true : false;
        res.lock_configuration.timer_plus_weight_box = $('#lock-timer-plus-box-size-show').is(":checked")? true : false;
        res.lock_configuration.timer_plus_weight_box_size = parseInt($('#lock-timer-plus-box-size').val());
        $('.timer-plus-label').each(function() {
            let key = $( this ).attr("label");
            let r = Object.values(klasses).find((e) => e.key == key);
            let label = Object.keys(klasses).find(k=>klasses[k].key === key);
            if(r){
                let val = parseInt($( this ).val());
                res.lock_configuration.timer_plus_data[label] = val;
            }
        });
        security_token = res.lock_configuration.token;
        browser.storage.sync.set({
                lock_configuration: res.lock_configuration,
            }).then(function(){
                updateOptionsContentScript();
                window.location.reload();
            });
    })
});

$('io-toggle.timer-plus-toggle').click(function(e){
    e.stopImmediatePropagation();
});

$('#lock-timer-plus-increase-format').change(function(e) {
    let format = $('#lock-timer-plus-increase-format').val();
    switch (format) {
        case 's':
            $("#lock-timer-plus-increase-input").attr({
                "max" : 100000,
            });
            break;
        case 'm':
            $("#lock-timer-plus-increase-input").attr({
                "max" : 10000,
            });
            break;
        case 'h':
            $("#lock-timer-plus-increase-input").attr({
                "max" : 1000,
            });
            break;
        case 'd':
            $("#lock-timer-plus-increase-input").attr({
                "max" : 100,
            });
            break;
    }
});

$('#lock-timer-plus-increase').click(function(e){
    let ms = Math.max(0,$('#lock-timer-plus-increase-input').val());
    let format = $('#lock-timer-plus-increase-format').val();
    if( ms == 0){
       return;
    }
    let c = browser.storage.sync.get(['lock_configuration']);
    c.then((res) => {
        switch (format) {
            case 's':
                ms = ms*1000;
                break;
            case 'm':
                ms = ms*1000*60;
                break;
            case 'h':
                ms = ms*1000*60*60;
                break;
            case 'd':
                ms = ms*1000*60*60*24;
                break;
        }
        $('#lock-timer-plus-increase-input').val(0);
        res.lock_configuration.duration += ms;
        browser.storage.sync.set({
            lock_configuration: res.lock_configuration,
        }).then(function(){
            window.location.reload();
        });
    });
});

function generateSecurityToken(pass){
    let i = performance.now();
    if(pass == null){
        pass = i+"";
    }
    return hashFnv32a(pass, false, i);
}

/**
 * Calculate a 32 bit FNV-1a hash
 *
 * @param {string} str the input value
 * @param {boolean} [asString=false] set to true to return the hash value as
 *     8-digit hex string instead of an integer
 * @param {number} [seed] optionally pass the hash of the previous chunk
 * @returns {integer | string}
 */
function hashFnv32a(str, asString, seed) {
    let i, l, hval = (seed === undefined) ? 0x811c9dc5 : seed;

    for (i = 0, l = str.length; i < l; i++) {
        hval ^= str.charCodeAt(i);
        hval += (hval << 1) + (hval << 4) + (hval << 7) + (hval << 8) + (hval << 24);
    }
    if( asString ){
        // Convert to 8 digit hex string
        return ("0000000" + (hval >>> 0).toString(16)).substr(-8);
    }
    return hval >>> 0;
}

