document.addEventListener('DOMContentLoaded', restoreOptions);

function escapeHTMLSpecialChars(str) {
	return str.replace('&', '&amp');
}

const MAX_CENSOR_PRESETS = 4;

let sticker_collections = null;

function restoreOptions() {
	restoreTheme();
	restoreVersion();
	restoreONOFF();
	restoreLoginState();
	restoreLockUnlock();
	updateRemoteSubscription().then(r => {});
	restoreIcons();
	restoreLabels();
	restoreFiletypes();
	restoreCensorOptions();
	restoreFileOptions();
	restoreCensorConfigs();
	restoreDebug();
	restoreAdvanced();
	restoreVideo();
	restoreWhiteBlackList();
	updateQuickSettings();
	updateHelp();
	updateStatistics();
	startUpdateInterval();
	restoreView();
}

let lock_configuration, remote_configuration; 
browser.storage.sync.get(['lock_configuration', 'remote_configuration']).then((res) => {
	lock_configuration = res.lock_configuration;
	remote_configuration = res.remote_configuration;

	browser.storage.onChanged.addListener((changes, area) => {
		if(area === 'sync') {
			lock_configuration = changes.lock_configuration?.newValue ?? lock_configuration;
			remote_configuration = changes.remote_configuration?.newValue ?? remote_configuration;
		}
	});
});

function restoreVersion(){
	$('#version').html(getVersion());
}

function getVersion(){
	let v = browser.runtime.getManifest().version;
	if(browser.runtime.getManifest().browser_specific_settings.gecko.id === 'pury.fi@pury.fi'){
		v += 'B';
	}
	return v;
}

function restoreMascot(user){
	let mascot_elem = $('#mascot');
	if(user === null || user.patreon_tier === 0){
		if(!mascot_elem.prop('src').endsWith('girl_small_patreon.png')){
			mascot_elem.fadeOut(200, function() {
				mascot_elem.attr('src', '../images/purychan/girl_small_patreon.png');
			}).fadeIn(200);
		}
	}else{
		if(!mascot_elem.prop('src').endsWith('girl_small_patreon_small.png')) {
			mascot_elem.fadeOut(200, function () {
				mascot_elem.attr('src', '../images/purychan/girl_small_patreon_small.png');
			}).fadeIn(200);
		}
	}
}

function restoreLabels() {
	let labels = browser.storage.sync.get('labels');
	labels.then((res) => {
		$('.bodypart-grid .labelButton').each(function(i, obj) {
			if(res.labels.includes(obj.id)){
				$(this).addClass("selected");
				let btn = $('button[label='+$(this).attr('id')+']');
				btn.attr('check', true);
				btn.parent().attr('check', true);
			}else{
				$(this).removeClass("selected");
				let btn = $('button[label='+$(this).attr('id')+']');
				btn.removeAttr('check');
				btn.parent().removeAttr('check');
			}
		});
	});
}

function restoreCensorOptions() {
	let censor_type = browser.storage.sync.get(['censor_type']);
	censor_type.then((res) => {
		enableCensorType($(`.general-accordion-container[data-censor-type="${parseEffectFromName(res.censor_type).index}"]`));
	});
}

function restoreLoginState() {
	let user_req = browser.storage.sync.get(['user', 'username', 'password']);
	user_req.then((res) => {
		if (res.user) {
			loginSuccess(res.user);
			$('#video_overlay_log_in_warning').hide();
		} else {
			$('#video_overlay_log_in_warning').show();
			if (res.username) {
				$("#profile_name").html(escapeHTMLSpecialChars(res.username));
				$("#profile_tier").html('');
			}
			$("#login_form :input").prop("disabled", false);
			$("#login_form").show("slow");
			$("#profile").hide();
		}
		restoreMascot(res.user??null);
	});
}

function restoreDebug() {
	let debug = browser.storage.sync.get('debug');
	debug.then((res) => {
		$('.debugToggle').each(function(i, obj) {
			if(res.debug){
				$(this).attr('check', true);
				$('#'+$(this).attr('label')).addClass("selected");
			}else{
				$(this).removeAttr('check');
				$('#'+$(this).attr('label')).removeClass("selected");
			}

		});
	});
	browser.storage.sync.getBytesInUse(null).then(store => {
		$('#storage-synced-size').text(formatBytes(store));
	});
	browser.storage.local.get(null).then(res => {
		let bs = 0;
		let ss = 0;
		let ooms = 0;
		let oomcount = 0;
		let oomtreecount = 0;
		res.sticker_collections.forEach(function (collection, key) {
			collection._stickers.forEach(function (sticker, name) {
				if (sticker.file) {
					console.error(sticker.name, sticker.file.length)
				}
				bs += new Blob([sticker.url]).size
			});
		});
		$('#storage-local-size').text(formatBytes(bs));
		ss = bs;
		if(res.only_once_mode_storage_manager && res.only_once_mode_storage_manager.trees){
			let local_request = browser.storage.local.get(res.only_once_mode_storage_manager.trees);
			local_request.then((local) => {
				oomtreecount = res.only_once_mode_storage_manager.trees.length;
				for (const tree_reference of res.only_once_mode_storage_manager.trees) {
					if(local.hasOwnProperty(tree_reference)) {
						let compressed = local[tree_reference];
						let decompressed = LZString.decompress(compressed);
						const obj = JSON.parse(decompressed);
						oomcount += obj.count;
						ooms += decompressed.length;
					}else{
						console.error("Corrupted oom data storage!", tree_reference);
					}
				}
				$('#oom-cache-size').text(formatBytes(ooms));
				$('#oom-containers').text(oomtreecount);
				$('#oom-image-hashes').text(oomcount);
				$('#statistics-oom').text(oomcount);

			});

		}
		$('#storage-local-sticker-size').text(formatBytes(ss));

	});
}

function countLeaves(obj) {
	return Object.values(obj).reduce((count,v) => count += (typeof(v) === 'object') ? countLeaves(v): 1, 0);
}

$('#remote-settings-subscribe, #settings-import-cloud, #settings-load').on('click mousedown mouseup submit', function(e) {
	if (lock_configuration.enabled) {
		e.preventDefault();
		e.stopImmediatePropagation();
		e.stopPropagation();
	}
});

$('#settings-load').on('click mousedown mouseup submit', function(e) {
	if (lock_configuration.enabled && lock_configuration.locked_options.includes('settings')) {
		e.preventDefault();
		e.stopImmediatePropagation();
		e.stopPropagation();
	}
});

$('#remote-settings-identifier, #settings-import-cloud-identifier').on('keydown keyup keypress input change submit', function(e) {
	if (lock_configuration.enabled) {
		e.preventDefault();
		e.stopImmediatePropagation();
		e.stopPropagation();
	}
});

$('#settings_import_dropzone').on('dragover dragenter drop', function(e) {
	if (lock_configuration.enabled) {
		e.preventDefault();
		e.stopImmediatePropagation();
		e.stopPropagation();
	}
});

$('#remote-settings-unsubscribe').on('click mousedown mouseup submit', function(e) {
	if (remote_configuration.lock) {
		e.preventDefault();
		e.stopImmediatePropagation();
		e.stopPropagation();
	}
});

function updateOnCensorShapeChange($input) {
	let name = parseEffectFromIndex(parseInt($input.closest('.general-accordion-container').attr('data-censor-type'))).name;
	if (name === "black") name = "bar";

	if (parseInt($input.val()) === 0) {
		$(`#${name}_rounding`).closest('.general-setting').show();
	} else {
		$(`#${name}_rounding`).closest('.general-setting').hide();
	}
}

$('#bar_shape, #pixel_shape, #blur_shape, #triangle_shape, #sobel_shape').change(function(e) {
	updateOnCensorShapeChange($(this));
});

function updateOnBarTypeChange() {
	if (parseInt($('#bar_type').val()) === 0) {
		$('#bar_color').closest('.general-setting').show();
		$('#bar_opacity').closest('.general-setting').hide();
	} else {
		$('#bar_color').closest('.general-setting').hide();
		$('#bar_opacity').closest('.general-setting').show();
	}
}

$('#bar_type').change(function(e) {
	updateOnBarTypeChange();
});

$('#content_toggle_all').on('mousedown', function() {
	browser.storage.sync.get(['lock_configuration']).then((res) => {
		if(res.lock_configuration.enabled) {
			return;
		}

		let labellist = [];
		let $toggles = $(this).closest('#content-general').find('io-toggle.labelToggle');
		$toggles.each(function() {
			let label = $(this).attr('label');
			$(this).attr('check', true);
			$(this).children('button').attr('check', true);
			$('.labelButton[id="'+label+'"]').addClass('selected');
			
			labellist.push(label);
			setMixedLabelTypeRowSelected(parseKlassFromKey(label).index, true);
		})

		browser.storage.sync.set({
			labels: labellist
		}).then(function(){
			updateOptionsContentScript();
		});
	});
});

$('#content_toggle_none').on('mousedown', function() {
	browser.storage.sync.get(['lock_configuration']).then((res) => {
		if(res.lock_configuration.enabled) {
			return;
		}

		let labellist = [];
		let $toggles = $(this).closest('#content-general').find('io-toggle.labelToggle');
		$toggles.each(function() {
			let label = $(this).attr('label');
			$(this).removeAttr('check');
			$(this).children('button').removeAttr('check');
			$('.labelButton[id="'+label+'"]').removeClass('selected');

			setMixedLabelTypeRowSelected(parseKlassFromKey(label).index, false);
		})

		browser.storage.sync.set({
			labels: labellist
		}).then(function(){
			updateOptionsContentScript();
		});
	});
});

$('#content_toggle_flip').on('mousedown', function() {
	browser.storage.sync.get(['lock_configuration']).then((res) => {
		if(res.lock_configuration.enabled) {
			return;
		}

		let labellist = [];
		let $toggles = $(this).closest('#content-general').find('io-toggle.labelToggle');
		$toggles.each(function() {
			if ($(this).attr('check')) {
				let label = $(this).attr('label');
				$(this).removeAttr('check');
				$(this).children('button').removeAttr('check');
				$('.labelButton[id="'+label+'"]').removeClass('selected');

				setMixedLabelTypeRowSelected(parseKlassFromKey(label).index, false);
			} else {
				let label = $(this).attr('label');
				$(this).attr('check', true);
				$(this).children('button').attr('check', true);
				$('.labelButton[id="'+label+'"]').addClass('selected');
				
				labellist.push(label);
				setMixedLabelTypeRowSelected(parseKlassFromKey(label).index, true);
			}
		})

		browser.storage.sync.set({
			labels: labellist
		}).then(function(){
			updateOptionsContentScript();
		});
	});
});

let mixed_multiselect_info = null;

$(document).click(function(e) {
	if ($(e.target).closest('.mixed-censor-type-popup').length) return;

	let $entry = $(e.target).closest('.mixed-entry');
	if (!$entry.length) {
		$('.mixed-entry').removeClass('selected');
		mixed_multiselect_info = null;
		return;
	}
	
	if (!e.shiftKey && !e.ctrlKey) {
		if (!$entry.hasClass('selected') || !$(e.target).closest('.mixed-entry-select, .mixed-entry-button-container').length) {
			$('.mixed-entry').removeClass('selected');
			mixed_multiselect_info = null;
			return;
		}
	} else {
		if (e.shiftKey) {
			// If no anchor is set
			if (mixed_multiselect_info == null) {
				// Select this entry and set is as the anchor
				$entry.addClass('selected');
				mixed_multiselect_info = {
					anchor: $entry,
				}
			} else {
				let $table_cols = $('.entries-table').find('tr');
				let $entries_rows;
				// Get the rows for the table section in which the click happened
				if ($entry.parent().attr('data-label-type')) {
					$entries_rows = $table_cols.find('.mixed-entries-row[data-label-type]:eq(0)');
					if (!$entries_rows.find($entry).length) {
						$entries_rows = $table_cols.find('.mixed-entries-row[data-label-type]:eq(1)');
					}
				} else {
					$entries_rows = $entry.parent();
				}

				let anchor_row_index = $entries_rows.index(mixed_multiselect_info.anchor.parent());
				// If the anchor is in the same table section as this click
				if (anchor_row_index !== -1) {
					// Get only the entries rows between the anchor and this entry
					let row_index = $entries_rows.index($entry.parent());
					if (row_index < anchor_row_index) {
						$entries_rows = $entries_rows.slice(row_index, anchor_row_index + 1);
					} else {
						$entries_rows = $entries_rows.slice(anchor_row_index, row_index + 1);
					}

					// Get only the entries between the anchor and this entry
					let layer = parseInt($entry.attr('data-index'));
					let anchor_layer = parseInt(mixed_multiselect_info.anchor.attr('data-index'));
					let [min_layer, max_layer] = layer <= anchor_layer ? [layer, anchor_layer] : [anchor_layer, layer];
					let $entries = $entries_rows.find('.mixed-entry').filter(function() {
						let other_layer = parseInt($(this).attr('data-index'));
						return min_layer <= other_layer && other_layer <= max_layer;
					})
					
					// If ctrl+click, set this entry as the anchor, otherwise, deselect the previous selection and store the new one
					if (e.ctrlKey) {
						mixed_multiselect_info = {
							anchor: $entry
						}
					} else {
						mixed_multiselect_info.selection?.not(mixed_multiselect_info.anchor).removeClass('selected');
						mixed_multiselect_info.selection = $entries;
					}

					// Finally select the selection
					$entries.addClass('selected');
				} else {
					// If ctrl+click, set this entry as the anchor, otherwise, deselect the previous selection and store this entry as the selection
					if (e.ctrlKey) {
						mixed_multiselect_info = {
							anchor: $entry
						}
					} else {
						mixed_multiselect_info.selection?.not(mixed_multiselect_info.anchor).removeClass('selected');
						mixed_multiselect_info.selection = $entry;
					}

					// Finally select this entry
					$entry.addClass('selected');
				}
			}
		} else {
			// Toggle selecting this entry and set it as the anchor
			$entry.toggleClass('selected');
			mixed_multiselect_info = {
				anchor: $entry,
			}
		}
	}
})

// On shift/ctrl + dblclick, deselect all but the clicked mixed entry
$(document).dblclick(function(e) {
	if ($(e.target).closest('.mixed-censor-type-popup').length) return;
	if (e.shiftKey || e.ctrlKey) {
		$('.mixed-entry').removeClass('selected');
		let $mixed_entry = $(e.target).closest('.mixed-entry');
		if ($mixed_entry.length) {
			$mixed_entry.addClass('selected');
			mixed_multiselect_info = {
				anchor: $mixed_entry,
			}
		}
	}
})

$('.type-button-s').click(function(e) {
	const $popup = $(this).closest('.mixed-censor-type-popup')
	const label_type = $popup.attr('data-label-type');
	const index = $popup.attr('data-index');
	const censor_type = parseInt($(this).attr('data-censor-type'));
	let mixed_entry;
	if (label_type != null) {
		mixed_entry = $('#mixed_setting_group').find(`.mixed-entries-row[data-label-type="${label_type}"] .mixed-entry[data-index="${index}"]`);
	} else if ($popup.is('[data-reverse-type]')) {
		mixed_entry = $('#mixed_setting_group').find(`.mixed-entries-row[data-reverse-type] .mixed-entry[data-index="${index}"]`);
	} else return;

	setMixedCensorType(mixed_entry.add('.mixed-entry.selected'), censor_type);

	$popup.remove();
	saveConfig();
})

$('.mixed-entry-select ul').hide();

$(document).click(function(e) {
	if (e.shiftKey || e.ctrlKey) return;
	let $select = $(e.target).closest('.mixed-entry-select');
	if (!$select.length) {
		$('.mixed-entry-select ul').hide();
		return;
	}

	let $select_option_label = $(e.target).closest('.mixed-entry-select ul li a');
	if ($select_option_label.length) {
		$('.mixed-entry-select ul').hide();
		let censor_type = parseInt($select.closest('.mixed-entry').attr('data-censor-type'));
		setMixedCensorPreset($select.add(`.mixed-entry.selected[data-censor-type="${censor_type}"] .mixed-entry-select`), $select_option_label.parent().val());

		saveConfig();
		return;
	}

	let $select_name = $(e.target).closest('.mixed-entry-select a');
	if ($select_name.length) {
		let $select_items = $select.children('ul');
		$('.mixed-entry-select ul').not($select_items).hide();
		$select_items.toggle();
		return;
	}
})

$(document).click(function(e) {
	if ($(e.target).closest('.mixed-censor-type-popup').length) return;
	$('.mixed-censor-type-popup').not('#mixed_censor_type_popup_template').remove();
})

function spawnCensorTypeSelector($button) {
	const $entry = $button.closest('.mixed-entry');
	const $popup = $('#mixed_censor_type_popup_template').clone({deepWithDataAndEvents: true});
	$popup.attr('id', '');
	const $entries_row = $entry.closest('.mixed-entries-row');
	const label_type = $entries_row.attr('data-label-type');
	if (label_type != null) {
		$popup.attr('data-label-type', label_type);
	} else if ($entries_row.is('[data-reverse-type]')) {
		$popup.attr('data-reverse-type', 'true');
	} else return;
	$popup.attr('data-index', $entry.attr('data-index'));
	$popup.prependTo($('body'));

	const $first_popup_item = $popup.find('.type-button-s').first();
	const first_popup_item_center = $first_popup_item.offset().left + ($first_popup_item.outerWidth() / 2) - $popup.offset().left;
	const button_bottom = $button.offset().top + $button.outerHeight();
	const button_center = $button.offset().left + ($button.outerWidth() / 2);

	$popup.position({
		my: 'left top+4',
		at: 'left bottom',
		of: $button,
		collision: 'fit flipfit',
		// 'left': button_center - first_popup_item_center + 'px',
		// 'top': button_bottom + 4 +'px',
	});
	
	$popup.addClass('visible');
}

$(document).click(function(e) {
	const $remove_button = $(e.target).closest('.mixed-entry-remove-button');
	if (!$remove_button.length) return;
	if (e.shiftKey || e.ctrlKey) return;
	removeMixedCensorType($remove_button.closest('.mixed-entry').add('.mixed-entry.selected'));
	saveConfig();
}).click(function(e) {
	const $button = $(e.target).closest('.mixed-entry-button');
	if (!$button.length) return;
	if (e.shiftKey || e.ctrlKey) return;
	spawnCensorTypeSelector($button);
})

function resetHorScroll() {
	$(this).scrollLeft(0);
}

function updateOnPresetGroupChange($tab_preset_group, preset_index_removed = -1) {
	const censor_type = parseInt($tab_preset_group.attr('data-censor-type'));

	// If a preset was removed, update mixed entries set to this censor type
	if (preset_index_removed !== -1) {
		censor_presets[effects.MIXED.index].forEach(preset => {
			if (preset == null) return;

			for (let label_type in preset.config.censor_types) {
				for (let layer = 0; layer < MIXED_CENSOR_LAYERS_COUNT; layer++) {
					if (preset.config.censor_types[label_type][layer] === censor_type) {
						let curr_index = preset.config.censor_preset_indexes[label_type][layer];
						// If the removed preset was below the set preset, shift the preset index down by one
						if (preset_index_removed < curr_index) {
							preset.config.censor_preset_indexes[label_type][layer] = curr_index - 1;
						} // If the removed preset was the set preset, reset the preset index to zero
						else if (curr_index === preset_index_removed) {
							preset.config.censor_preset_indexes[label_type][layer] = 0;
						}
					}
				}
			}

			for (let layer = 0; layer < MIXED_CENSOR_LAYERS_COUNT; layer++) {
				if (preset.config.reverse_censor_types[layer] === censor_type) {
					let curr_index = preset.config.reverse_censor_preset_indexes[layer];
					// If the removed preset was below the set preset, shift the preset index down by one
					if (preset_index_removed < curr_index) {
						preset.config.reverse_censor_preset_indexes[layer] = curr_index - 1;
					} // If the removed preset was the set preset, reset the preset index to zero
					else if (curr_index === preset_index_removed) {
						preset.config.reverse_censor_preset_indexes[layer] = 0;
					}
				}
			}
		});
	}

	const select_items_html = $tab_preset_group.find('.tab-edit-field').map(function(i) {
		return `<li value="${i}"><a>${escapeHTMLSpecialChars($(this).val()) || "&nbsp"}</a></li>`;
	}).get();

	// Update UI of currently selected mixed mode preset
	const $selects = $(`#mixed_setting_group .mixed-entries-row .mixed-entry[data-censor-type="${censor_type}"] .mixed-entry-select`);
	$selects.each(function() {
		let select_val = parseInt($(this).attr('value'));
		let $select_items_container = $(this).children('ul');
		$select_items_container.empty();
		$select_items_container.append(select_items_html);

		if (preset_index_removed === -1) {
			setMixedCensorPreset($(this), select_val);
		} else if (preset_index_removed < select_val) {
			setMixedCensorPreset($(this), select_val - 1);
		} else if (select_val === preset_index_removed) {
			setMixedCensorPreset($(this), 0);
		} else {
			setMixedCensorPreset($(this), select_val);
		}
	})
}

function selectPreset($tab_preset) {
	$tab_preset.siblings('.tab-preset').removeClass("tab-preset-selected");
	$tab_preset.addClass("tab-preset-selected");
	
	const censorType = $tab_preset.parent().attr("data-censor-type");
	const index = $tab_preset.attr("data-index");
	updateCensorPresetConfig[censorType](censor_presets[censorType][index]);
}

function deletePreset($tab_preset) {
	if ($tab_preset.siblings('.tab-preset').length <= 0) return;

	const $next_tab_presets = $tab_preset.nextAll(".tab-preset");
	if ($tab_preset.hasClass("tab-preset-selected")) {
		let $other_tab_preset = $tab_preset.prevAll(".tab-preset").first();
		if (!$other_tab_preset.length) {
			$other_tab_preset = $next_tab_presets.first();
		}
		selectPreset($other_tab_preset);
	}

	const $tab_preset_group = $tab_preset.parent();
	const censor_type = $tab_preset_group.attr("data-censor-type");
	const index = parseInt($tab_preset.attr("data-index"));
	censor_presets[censor_type].splice(index, 1);
	$next_tab_presets.each(function() {
		$(this).attr("data-index", parseInt($(this).attr("data-index")) - 1);
	});

	deleteTabPreset($tab_preset);
	updateOnPresetGroupChange($tab_preset_group, index);
}

function deleteTabPreset($tab_preset) {
	$tab_preset.remove();
}

function createPreset($tab_preset_group, name = "") {
	const $tab_presets = $tab_preset_group.children('.tab-preset');
	if (MAX_CENSOR_PRESETS <= $tab_presets.length) return;

	const $new_tab_preset = createTabPreset($tab_preset_group, name);

	const censor_type = $tab_preset_group.attr("data-censor-type");
	const index = $new_tab_preset.attr("data-index");
	censor_presets[censor_type][index] = generateDefaultPreset[censor_type]();
	
	$new_tab_preset.children('input').focus();
	selectPreset($new_tab_preset, false);
}

function createTabPreset($tab_preset_group, name = "") {
	const $tab_presets = $tab_preset_group.children('.tab-preset');

	const $new_tab_preset = $tab_presets.first().clone();
	$new_tab_preset.insertAfter($tab_presets.last());
	$new_tab_preset.attr("data-index", $tab_presets.length);
	$new_tab_preset.children('input').val(name);

	return $new_tab_preset;
}

$(document).click(function(e) { 
	const $button = $(e.target).closest('.tab-preset-plus');
	if (!$button.length) return;
	createPreset($button.parent());
	saveConfig();
}).click(function(e) {
	const $button = $(e.target).closest('.tab-preset-x');
	if (!$button.length) return;
	deletePreset($button.parent());
	saveConfig();
}).click(function(e) {
	const $tab_preset = $(e.target).closest('.tab-preset');
	if (!$tab_preset.length) return;
	if ($(e.target).closest('.tab-preset-x, .tab-edit-field').length) return;
	selectPreset($tab_preset);
	saveConfig();
}).on('change focusout', function(e) {
	const $input = $(e.target);
	if (!$input.hasClass('tab-edit-field') || !$input.closest('.tab-preset').length) return;
	updateOnPresetGroupChange($input.closest('.presets'));
	saveConfig();
})

function setMixedLabelTypeRowSelected(label_type, selected) { 
	const $label_type_button = $(`#mixed_setting_group .mixed-label-type-button[data-label-type="${label_type}"]`);
	const $censor_type_button = $(`#mixed_setting_group .mixed-entries-row[data-label-type="${label_type}"] .mixed-entry-button`);

	if (selected) {
		$label_type_button.addClass('selected');
		$censor_type_button.addClass('selected');
	} else {
		$label_type_button.removeClass('selected');
		$censor_type_button.removeClass('selected');
	}
}

function setMixedReverseRowSelected(selected) { 
	const $censor_type_button = $(`#mixed_setting_group .mixed-entries-row[data-reverse-type] .mixed-entry-button`);

	if (selected) {
		$censor_type_button.addClass('selected');
	} else {
		$censor_type_button.removeClass('selected');
	}
}

const censor_presets = {
	0: null,
	1: new Array(MAX_CENSOR_PRESETS),
	2: new Array(MAX_CENSOR_PRESETS),
	3: new Array(MAX_CENSOR_PRESETS),
	4: new Array(MAX_CENSOR_PRESETS),
	5: new Array(MAX_CENSOR_PRESETS),
	6: new Array(MAX_CENSOR_PRESETS),
	7: new Array(MAX_CENSOR_PRESETS),
	8: new Array(MAX_CENSOR_PRESETS),
	9: new Array(MAX_CENSOR_PRESETS),
	10: new Array(MAX_CENSOR_PRESETS),
	11: new Array(MAX_CENSOR_PRESETS),
}

const updateCensorPresetConfig = {
	0: null,
	1: (preset, do_highlight = false) => { 
		updateInputField($('#pixel_shape'), preset.config.shape, do_highlight);
		updateOnCensorShapeChange($('#pixel_shape'));
		updateInputField($('#pixel_rounding'), preset.config.rounding, do_highlight);
		updateInputField($('#pixel_scale'), preset.config.scale * 100, do_highlight);
		updateInputField($('#pixel_feathering'), preset.config.feathering, do_highlight);
		updateInputField($('#pixel_border_width'), preset.config.border_width, do_highlight);
		updateInputField($('#pixel_border_color'), preset.config.border_color, do_highlight);
		$('#pixel_border_color').css("background-color", preset.config.border_color);
		updateInputField($('#pixel_type'), preset.config.type, do_highlight);
		updateInputField($('#pixel_strength'), preset.config.strength, do_highlight);
		updateInputField($('#pixel_spacing'), preset.config.spacing, do_highlight);
		updateInputField($('#pixel_channel_colors'), preset.config.channel_colors, do_highlight);
		updateInputCheckbox($('#pixel_grayscale'), preset.config.grayscale, do_highlight);
		updateInputCheckbox($('#pixel_resolution_aware'), preset.config.resolution_aware, do_highlight);
		updateInputCheckbox($('#pixel_detection_size_aware'), preset.config.detection_size_aware, do_highlight);
		updateInputCheckbox($('#pixel_accurate_sampling'), preset.config.accurate_sampling, do_highlight);
	},
	2: (preset, do_highlight = false) => { 
		updateInputField($('#blur_shape'), preset.config.shape, do_highlight);
		updateOnCensorShapeChange($('#blur_shape'));
		updateInputField($('#blur_rounding'), preset.config.rounding, do_highlight);
		updateInputField($('#blur_scale'), preset.config.scale * 100, do_highlight);
		updateInputField($('#blur_feathering'), preset.config.feathering, do_highlight);
		updateInputField($('#blur_border_width'), preset.config.border_width, do_highlight);
		updateInputField($('#blur_border_color'), preset.config.border_color, do_highlight);
		$('#blur_border_color').css("background-color", preset.config.border_color);
		updateInputField($('#blur_strength'), preset.config.strength, do_highlight);
		updateInputCheckbox($('#blur_grayscale'), preset.config.grayscale, do_highlight);
		updateInputCheckbox($('#blur_resolution_aware'), preset.config.resolution_aware, do_highlight);
	},
	3: (preset, do_highlight = false) => {
		updateInputField($('#bar_shape'), preset.config.shape, do_highlight);
		updateOnCensorShapeChange($('#bar_shape'));
		updateInputField($('#bar_rounding'), preset.config.rounding, do_highlight);
		updateInputField($('#bar_scale'), preset.config.scale * 100, do_highlight);
		updateInputField($('#bar_feathering'), preset.config.feathering, do_highlight);
		updateInputField($('#bar_border_width'), preset.config.border_width, do_highlight);
		updateInputField($('#bar_border_color'), preset.config.border_color, do_highlight);
		$('#bar_border_color').css("background-color", preset.config.border_color);
		updateInputField($('#bar_type'), preset.config.type, do_highlight);
		updateOnBarTypeChange();
		updateInputField($('#bar_color'), preset.config.color, do_highlight);
		$('#bar_color').css("background-color", preset.config.color);
		updateInputField($('#bar_opacity'), preset.config.opacity * 100, do_highlight);
	},
	4: (preset, do_highlight = false) => {
		updateInputField($('#triangle_shape'), preset.config.shape, do_highlight);
		updateOnCensorShapeChange($('#triangle_shape'));
		updateInputField($('#triangle_rounding'), preset.config.rounding, do_highlight);
		updateInputField($('#triangle_scale'), preset.config.scale * 100, do_highlight);
		updateInputField($('#triangle_feathering'), preset.config.feathering, do_highlight);
		updateInputField($('#triangle_border_width'), preset.config.border_width, do_highlight);
		updateInputField($('#triangle_border_color'), preset.config.border_color, do_highlight);
		$('#triangle_border_color').css("background-color", preset.config.border_color);
		updateInputCheckbox($('#triangle_resolution_aware'), preset.config.resolution_aware, do_highlight);
		updateInputField($('#triangle_vertex_count'), preset.config.vertex_count, do_highlight);
		updateInputField($('#triangle_threshold'), preset.config.threshold, do_highlight);
		updateInputField($('#triangle_accuracy'), preset.config.accuracy * 100, do_highlight);
		updateInputField($('#triangle_blurring'), preset.config.blurring, do_highlight);
		updateInputField($('#triangle_fill_color_mode'), preset.config.fill_color_mode, do_highlight);
		updateInputField($('#triangle_fill_color'), preset.config.fill_color, do_highlight);
		$('#triangle_fill_color').css("background-color", preset.config.fill_color);
		updateInputField($('#triangle_stroke_color_mode'), preset.config.stroke_color_mode, do_highlight);
		updateInputField($('#triangle_stroke_color'), preset.config.stroke_color, do_highlight);
		$('#triangle_stroke_color').css("background-color", preset.config.stroke_color);
		updateInputField($('#triangle_stroke_width'), preset.config.stroke_width, do_highlight);
		updateInputCheckbox($('#triangle_gradients'), preset.config.gradients, do_highlight);
		updateInputField($('#triangle_gradient_stops'), preset.config.gradient_stops, do_highlight);
		updateInputField($('#triangle_line_join'), preset.config.line_join, do_highlight);
		updateInputCheckbox($('#triangle_transparent_color'), preset.config.transparent_color, do_highlight);

		updateOnTriangleFillColorModeChange.call($('#triangle_fill_color_mode'));
		updateOnTriangleStrokeColorModeChange.call($('#triangle_stroke_color_mode'));
		updateOnTriangleGradientsChange.call($('#triangle_gradients'));
	},
	5: (preset, do_highlight = false) => {
		updateInputField($('#box_scale'), preset.config.scale * 100, do_highlight);
		updateInputField($('#box_display'), preset.config.display, do_highlight);
		updateInputField($('#box_fill'), preset.config.fill, do_highlight);
		updateInputField($('#box_fill_color'), preset.config.fill_color, do_highlight);
		$('#box_fill_color').css("background-color", preset.config.fill_color);
		updateInputField($('#box_border'), preset.config.border, do_highlight);
		updateInputField($('#box_border_color'), preset.config.border_color, do_highlight);
		$('#box_border_color').css("background-color", preset.config.border_color);
	},
	6: (preset, do_highlight = false) => {
		updateInputField($('#glitch_type'), preset.config.type, do_highlight);
		updateInputField($('#glitch_scale'), preset.config.scale * 100, do_highlight);
		updateInputField($('#glitch_chromatic_aberration_color_intensity'), preset.config.chromatic_aberration_color_intensity * 100, do_highlight);
		updateInputField($('#glitch_chromatic_aberration_shift_intensity'), preset.config.chromatic_aberration_shift_intensity * 100, do_highlight);
		updateInputCheckbox($('#glitch_chromatic_aberration_horizontal_shift'), preset.config.chromatic_aberration_horizontal_shift, do_highlight);
		updateInputCheckbox($('#glitch_chromatic_aberration_vertical_shift'), preset.config.chromatic_aberration_vertical_shift, do_highlight);
		updateInputField($('#glitch_multiple_panels_amount'), preset.config.multiple_panels_amount, do_highlight);
		updateInputField($('#glitch_multiple_panels_min_size'), preset.config.multiple_panels_min_size * 100, do_highlight);
		updateInputField($('#glitch_multiple_panels_max_size'), preset.config.multiple_panels_max_size * 100, do_highlight);
		updateInputField($('#glitch_multiple_panels_scatter'), preset.config.multiple_panels_scatter * 100, do_highlight);
		updateInputField($('#glitch_multiple_panels_split_chance'), preset.config.multiple_panels_split_chance * 100, do_highlight);
		updateInputCheckbox($('#glitch_multiple_panels_border'), preset.config.border, do_highlight);
		for (let i = 0; i < preset.config.multiple_panels_gradient.length; i++) {
			updateInputField($(`#glitch_multiple_panels_gradient_color_${i}`), preset.config.multiple_panels_gradient[i].color, do_highlight);
			$(`#glitch_multiple_panels_gradient_color_${i}`).css("background-color", preset.config.multiple_panels_gradient[i].color);
			updateInputField($(`#glitch_multiple_panels_gradient_offset_${i}`), preset.config.multiple_panels_gradient[i].offset * 100, do_highlight);
		}

		updateOnGlitchTypeChange.call($('#glitch_type'));
	},
	7: (preset, do_highlight = false) => {
		updateInputField($('#sticker_scale'), preset.config.scale * 100, do_highlight);
		updateInputField($('#sticker_draw_mode'), preset.config.draw_mode, do_highlight);
		updateInputCheckbox($('#sticker_sample_single_source'), preset.config.sample_single_source, do_highlight);
		updateInputField($('#sticker_enabled_sources'), preset.config.enabled_sources.join(';'), do_highlight);
	},
	8: (preset, do_highlight = false) => {
		updateInputField($('#sobel_shape'), preset.config.shape, do_highlight);
		updateOnCensorShapeChange($('#sobel_shape'));
		updateInputField($('#sobel_rounding'), preset.config.rounding, do_highlight);
		updateInputField($('#sobel_scale'), preset.config.scale * 100, do_highlight);
		updateInputField($('#sobel_feathering'), preset.config.feathering, do_highlight);
		updateInputField($('#sobel_border_width'), preset.config.border_width, do_highlight);
		updateInputField($('#sobel_border_color'), preset.config.border_color, do_highlight);
		$('#sobel_border_color').css("background-color", preset.config.border_color);
		updateInputCheckbox($('#sobel_inverted_color'), preset.config.inverted_color, do_highlight);
	},
	9: (preset, do_highlight = false) => {
		updateInputField($('#splatter_scale'), preset.config.scale * 100, do_highlight);
		updateInputField($('#splatter_color_mode'), preset.config.color_mode, do_highlight);
		updateInputField($('#splatter_opacity'), preset.config.opacity * 100, do_highlight);
		for (let i = 0; i < preset.config.color_pool.length; i++) {
			updateInputField($(`#splatter_color_pool_color_${i}`), preset.config.color_pool[i].color, do_highlight);
			$(`#splatter_color_pool_color_${i}`).css("background-color", preset.config.color_pool[i].color);
			updateInputField($(`#splatter_color_pool_weight_${i}`), preset.config.color_pool[i].weight, do_highlight);
		}
		updateInputField($('#splatter_centering'), preset.config.centering, do_highlight);
		updateInputField($('#splatter_size'), preset.config.size * 100, do_highlight);
		updateInputField($('#splatter_amount'), preset.config.amount, do_highlight);
		updateInputField($('#splatter_sub_size'), preset.config.sub_size * 100, do_highlight);
		updateInputField($('#splatter_sub_amount'), preset.config.sub_amount, do_highlight);
		updateInputField($('#splatter_recursions'), preset.config.recursions, do_highlight);

		updateOnSplatterColorModeChange.call($('#splatter_color_mode'));
	},
	10: (preset, do_highlight = false) => {
		for (let label_type = 0; label_type < MIXED_LABEL_TYPE_ROWS_COUNT; label_type++) {
			const $mixed_entries_row = $(`#mixed_setting_group .mixed-entries-row[data-label-type="${label_type}"]`);
			for (let index = 0; index < MIXED_CENSOR_LAYERS_COUNT; index++) {
				const censor_type = preset.config.censor_types[label_type][index];
				const $mixed_entry = $mixed_entries_row.find(`.mixed-entry[data-index="${index}"`);
				if (censor_type == null) {
					removeMixedCensorType($mixed_entry, do_highlight);
				} else {
					setMixedCensorType($mixed_entry, censor_type, do_highlight);
					setMixedCensorPreset($mixed_entry.find('.mixed-entry-select'), preset.config.censor_preset_indexes[label_type][index] || 0, do_highlight);
				}
			}
		}
		const $mixed_entries_row = $(`#mixed_setting_group .mixed-entries-row[data-reverse-type]`);
		for (let index = 0; index < MIXED_CENSOR_LAYERS_COUNT; index++) {
			const censor_type = preset.config.reverse_censor_types[index];
			const $mixed_entry = $mixed_entries_row.find(`.mixed-entry[data-index="${index}"`);
			if (censor_type == null) {
				removeMixedCensorType($mixed_entry, do_highlight);
			} else {
				setMixedCensorType($mixed_entry, censor_type, do_highlight);
				setMixedCensorPreset($mixed_entry.find('.mixed-entry-select'), preset.config.reverse_censor_preset_indexes[index] || 0, do_highlight);
			}
		}
	},
	11: (preset, do_highlight = false) => {
		updateInputRadio($('[name="random_mode"]'), preset.config.mode, do_highlight);
		for (const key in preset.config.censor_weights) {
			updateInputField($(`input.random-chance[data-censor-type=${key}]`), preset.config.censor_weights[key], do_highlight);
		}
	}
}

const saveCensorPresetConfig = {
	0: null,
	1: (preset) => { 
		preset.config = new PixelCensorTypePresetConfig(
			parseFloat($('#pixel_shape').val()),
			parseFloat($('#pixel_rounding').val()),
			parseFloat($('#pixel_scale').val()) / 100,
			parseFloat($('#pixel_feathering').val()),
			parseFloat($('#pixel_border_width').val()),
			$('#pixel_border_color').val(),
			parseFloat($('#pixel_type').val()),
			parseFloat($('#pixel_strength').val()),
			parseFloat($('#pixel_spacing').val()),
			parseFloat($('#pixel_channel_colors').val()),
			$('#pixel_grayscale').is(':checked'),
			$('#pixel_resolution_aware').is(':checked'),
			$('#pixel_detection_size_aware').is(':checked'),
			$('#pixel_accurate_sampling').is(':checked'),
		)
	},
	2: (preset) => { 
		preset.config = new BlurCensorTypePresetConfig(
			parseFloat($('#blur_shape').val()),
			parseFloat($('#blur_rounding').val()),
			parseFloat($('#blur_scale').val()) / 100,
			parseFloat($('#blur_feathering').val()),
			parseFloat($('#blur_border_width').val()),
			$('#blur_border_color').val(),
			parseFloat($('#blur_strength').val()),
			$('#blur_grayscale').is(':checked'),
			$('#blur_resolution_aware').is(':checked'),
		)
	},
	3: (preset) => {
		preset.config = new BarCensorTypePresetConfig(
			parseFloat($('#bar_shape').val()),
			parseFloat($('#bar_rounding').val()),
			parseFloat($('#bar_scale').val()) / 100,
			parseFloat($('#bar_feathering').val()),
			parseFloat($('#bar_border_width').val()),
			$('#bar_border_color').val(),
			parseFloat($('#bar_type').val()),
			$('#bar_color').val(),
			parseFloat($('#bar_opacity').val()) / 100,
		)
	},
	4: (preset) => {
		preset.config = new TriangleCensorTypePresetConfig(
			parseFloat($('#triangle_shape').val()),
			parseFloat($('#triangle_rounding').val()),
			parseFloat($('#triangle_scale').val()) / 100,
			parseFloat($('#triangle_feathering').val()),
			parseFloat($('#triangle_border_width').val()),
			$('#triangle_border_color').val(),
			$('#triangle_resolution_aware').is(':checked'),
			parseFloat($('#triangle_vertex_count').val()),
			parseFloat($('#triangle_threshold').val()),
			parseFloat($('#triangle_accuracy').val()) / 100 ?? 0,
			parseFloat($('#triangle_blurring').val()),
			parseFloat($('#triangle_fill_color_mode').val()),
			$('#triangle_fill_color').val(),
			parseFloat($('#triangle_stroke_color_mode').val()),
			$('#triangle_stroke_color').val(),
			parseFloat($('#triangle_stroke_width').val()),
			$('#triangle_gradients').is(':checked'),
			parseFloat($('#triangle_gradient_stops').val()),
			parseFloat($('#triangle_line_join').val()),
			$('#triangle_transparent_color').is(':checked'),
		)
	},
	5: (preset) => { 
		preset.config = new BoxCensorTypePresetConfig(
			parseFloat($('#box_scale').val()) / 100,
			parseFloat($('#box_display').val()),
			parseFloat($('#box_fill').val()),
			$('#box_fill_color').val(),
			parseFloat($('#box_border').val()),
			$('#box_border_color').val(),
		)
	},
	6: (preset) => {
		const gradient = new Array(4);
		for (let i = 0; i < gradient.length; i++) {
			gradient[i] = {
				color: $(`#glitch_multiple_panels_gradient_color_${i}`).val(),
				offset: parseFloat($(`#glitch_multiple_panels_gradient_offset_${i}`).val()) / 100 ?? 0,
			};
		}
		preset.config = new GlitchCensorTypePresetConfig(
			parseFloat($('#glitch_type').val()),
			parseFloat($('#glitch_scale').val()) / 100,
			parseFloat($('#glitch_chromatic_aberration_color_intensity').val()) / 100 ?? 0,
			parseFloat($('#glitch_chromatic_aberration_shift_intensity').val()) / 100 ?? 0,
			$('#glitch_chromatic_aberration_horizontal_shift').is(':checked'),
			$('#glitch_chromatic_aberration_vertical_shift').is(':checked'),
			parseFloat($('#glitch_multiple_panels_amount').val()),
			parseFloat($('#glitch_multiple_panels_min_size').val()) / 100,
			parseFloat($('#glitch_multiple_panels_max_size').val()) / 100,
			parseFloat($('#glitch_multiple_panels_scatter').val()) / 100 ?? 0,
			parseFloat($('#glitch_multiple_panels_split_chance').val()) / 100 ?? 0,
			$('#glitch_multiple_panels_border').is(':checked'),
			gradient,
		)
	},
	7: (preset) => {
		preset.config = new StickerCensorTypePresetConfig(
			parseFloat($('#sticker_scale').val()) / 100,
			parseFloat($('#sticker_draw_mode').val()),
			$('#sticker_sample_single_source').is(':checked'),
			$('#sticker_enabled_sources').val().split(';').filter(item => item),
		)
	},
	8: (preset) => { 
		preset.config = new SobelCensorTypePresetConfig(
			parseFloat($('#sobel_shape').val()),
			parseFloat($('#sobel_rounding').val()),
			parseFloat($('#sobel_scale').val()) / 100,
			parseFloat($('#sobel_feathering').val()),
			parseFloat($('#sobel_border_width').val()),
			$('#sobel_border_color').val(),
			$('#sobel_inverted_color').is(':checked'),
		)
	},
	9: (preset) => {
		const color_pool = new Array(SPLATTER_COLOR_POOL_ENTRIES_COUNT);
		for (let i = 0; i < color_pool.length; i++) {
			color_pool[i] = {
				color: $(`#splatter_color_pool_color_${i}`).val(),
				weight: parseFloat($(`#splatter_color_pool_weight_${i}`).val()),
			};
		}
		preset.config = new SplatterCensorTypePresetConfig(
			parseFloat($('#splatter_scale').val()) / 100,
			parseFloat($('#splatter_color_mode').val()),
			parseFloat($('#splatter_opacity').val()) / 100 ?? 0,
			color_pool,
			parseFloat($('#splatter_centering').val()),
			parseFloat($('#splatter_size').val()) / 100,
			parseFloat($('#splatter_amount').val()),
			parseFloat($('#splatter_sub_size').val()) / 100,
			parseFloat($('#splatter_sub_amount').val()),
			parseFloat($('#splatter_recursions').val()),
		)
	},
	10: (preset) => {
		let censor_types = {}, censor_preset_indexes = {};
		for (let label_type = 0; label_type < MIXED_LABEL_TYPE_ROWS_COUNT; label_type++) {
			censor_types[label_type] = new Array(MIXED_CENSOR_LAYERS_COUNT);
			censor_preset_indexes[label_type] = new Array(MIXED_CENSOR_LAYERS_COUNT);
			const $mixed_entries_row = $(`#mixed_setting_group .mixed-entries-row[data-label-type="${label_type}"]`);
			for (let index = 0; index < MIXED_CENSOR_LAYERS_COUNT; index++) {
				const $mixed_entry = $mixed_entries_row.find(`.mixed-entry[data-index="${index}"`);
				const censor_type = parseInt($mixed_entry.attr('data-censor-type'));

				censor_types[label_type][index] = !isNaN(censor_type) ? censor_type : undefined;
				const censor_preset_index = parseInt($mixed_entry.find('.mixed-entry-select').attr('value'));
				censor_preset_indexes[label_type][index] = !isNaN(censor_preset_index) ? censor_preset_index : undefined;
			}
		}
		let reverse_censor_types = new Array(MIXED_CENSOR_LAYERS_COUNT);
		let reverse_censor_preset_indexes = new Array(MIXED_CENSOR_LAYERS_COUNT);
		const $mixed_entries_row = $(`#mixed_setting_group .mixed-entries-row[data-reverse-type]`);
		for (let index = 0; index < MIXED_CENSOR_LAYERS_COUNT; index++) {
			const $mixed_entry = $mixed_entries_row.find(`.mixed-entry[data-index="${index}"`);
			const censor_type = parseInt($mixed_entry.attr('data-censor-type'));

			reverse_censor_types[index] = !isNaN(censor_type) ? censor_type : undefined;
			const censor_preset_index = parseInt($mixed_entry.find('.mixed-entry-select').attr('value'));
			reverse_censor_preset_indexes[index] = !isNaN(censor_preset_index) ? censor_preset_index : undefined;
		}
		preset.config = new MixedCensorTypePresetConfig(censor_types, censor_preset_indexes, reverse_censor_types, reverse_censor_preset_indexes);
	},
	11: (preset) => {
		let censor_weights = {};
		$('input.random-chance').each(function() {
			censor_weights[$(this).attr('data-censor-type')] = parseFloat($(this).val());
		});
		preset.config = new RandomCensorTypePresetConfig(
			parseFloat($('[name="random_mode"]:checked').val()),
			censor_weights,
		)
	}
}

function restoreCensorConfigs() {
	let censor_type = browser.storage.sync.get(['bar_configuration','blur_configuration','pixel_configuration',
		'glitch_configuration','triangle_configuration','box_configuration','sticker_configuration', 'sobel_configuration',
		'splatter_configuration', 'mixed_configuration', 'random_configuration',
		'gif_configuration','png_configuration', 'jpg_configuration', 'bmp_configuration', 'webp_configuration', 'avif_configuration',
		'reverse_mode_configuration', 'word_wall_configuration', 'clustering_configuration','only_once_mode_configuration', 'remote_configuration']);
	censor_type.then((res) => {
		const censor_configs_by_type = {
			1: res.pixel_configuration,
			2: res.blur_configuration,
			3: res.bar_configuration,
			4: res.triangle_configuration,
			5: res.box_configuration,
			6: res.glitch_configuration,
			7: res.sticker_configuration,
			8: res.sobel_configuration,
			9: res.splatter_configuration,
			10: res.mixed_configuration,
			11: res.random_configuration,
		};

		// Store preset configs before updating the UI so when loading the mixed mode UI we don't read data of censor modes that were going to be loaded after mixed
		for (const censor_type in censor_configs_by_type) {
			const censor_config = censor_configs_by_type[censor_type];

			if (!censor_config.presets.length) {
				censor_config.presets.push(generateDefaultPreset[censor_type]());
			}
			
			for (let i = 0; i < MAX_CENSOR_PRESETS; i++) {
				if (censor_config.presets[i] != null) {
					censor_presets[censor_type][i] = censor_config.presets[i];
				} else {
					censor_presets[censor_type][i] = null;
				}
			}
		}

		for (const censor_type in censor_configs_by_type) {
			const censor_config = censor_configs_by_type[censor_type];

			const $tab_preset_group = $(`.presets[data-censor-type=${censor_type}]`);
			for (let i = 0; i < MAX_CENSOR_PRESETS; i++) {
				if (censor_config.presets[i] != null) {
					const $preset_tab = $tab_preset_group.children(`.tab-preset[data-index="${i}"]`);
					if ($preset_tab.length) {
						$preset_tab.children('input').val(censor_config.presets[i].name);
					} else {
						createTabPreset($tab_preset_group, censor_config.presets[i].name);
					}
				} else {
					const $preset_tab = $tab_preset_group.children(`.tab-preset[data-index="${i}"]`);
					if ($preset_tab.length) {
						deleteTabPreset($preset_tab);
					}
				}
			}
	
			const $tab_preset = $tab_preset_group.children(`.tab-preset[data-index="${censor_config.selected_preset_index}"]`);
			$tab_preset.siblings('.tab-preset').removeClass("tab-preset-selected");
			$tab_preset.addClass("tab-preset-selected");
			
			const selected_preset_index = $tab_preset.attr("data-index");
			updateCensorPresetConfig[censor_type](censor_presets[censor_type][selected_preset_index]);
		}
		
		// TODO: do the same for reverse
		$('.body-part-element .body-part-toggle io-toggle').each(function() {
			const label_type = parseKlassFromKey($(this).attr('label')).index;
			setMixedLabelTypeRowSelected(label_type, $(this).attr('check') ? true : false);
		});

		if(res.clustering_configuration.enabled){
			$('button.typecm').addClass('selected');
			$('.clusteringToggle').attr('check', true);
		}else{
			$('button.typecm').removeClass('selected');
			$('.clusteringToggle').removeAttr('check');
		}
		const $clustering_mode = $('[name="clustering_mode"]');
		$clustering_mode.prop('checked', false);
		$clustering_mode.filter(`[value="${res.clustering_configuration.mode}"]`).prop('checked', true);
		
		if(res.reverse_mode_configuration.enabled){
			$('.reverseToggle').attr('check', true);
			$('button.typerm').addClass('selected');
		}else{
			$('.reverseToggle').removeAttr('check');
			$('button.typerm').removeClass('selected');
		}
		setMixedReverseRowSelected(res.reverse_mode_configuration.enabled);
		
		const $reverse_mode_mode = $('[name="reverse_mode_mode"]');
		$reverse_mode_mode.prop('checked', false);
		$reverse_mode_mode.filter(`[value="${res.reverse_mode_configuration.mode}"]`).prop('checked', true);

		if(res.word_wall_configuration.enabled){
			$('button.typeww').addClass('selected');
			$('.word-wall-toggle').attr('check', true);
		}else{
			$('button.typeww').removeClass('selected');
			$('.word-wall-toggle').removeAttr('check');
		}
		$('#word_wall_word_pool').val(res.word_wall_configuration.word_pool.join('\n'));
		$(`[name="word_wall_draw_mode"][value="${res.word_wall_configuration.draw_mode}"`).prop('checked', true);
		$('#word_wall_size').val(res.word_wall_configuration.size);
		$('#word_wall_horizontal_spacing').val(res.word_wall_configuration.horizontal_spacing);
		$('#word_wall_vertical_spacing').val(res.word_wall_configuration.vertical_spacing);
		$('#word_wall_offset').val(res.word_wall_configuration.offset);
		$('#word_wall_angle').val(res.word_wall_configuration.angle);
		$('#word_wall_font').val(res.word_wall_configuration.font);
		$("#word_wall_color_mode").val(res.word_wall_configuration.color_mode);
		$('#word_wall_color').val(res.word_wall_configuration.color);
		$('#word_wall_color').css("background-color", res.word_wall_configuration.color);

		if(res.only_once_mode_configuration.enabled){
			$('.oomToggle').attr('check', true);
		}else{
			$('.oomToggle').removeAttr('check');
		}

		if(res.only_once_mode_configuration.precision != null){
			$('#only_once_mode_precision').val(res.only_once_mode_configuration.precision);
		}
		if(res.only_once_mode_configuration.message != null){
			$('#only_once_mode_message').val(res.only_once_mode_configuration.message);
		}
		if(res.only_once_mode_configuration.date_time_format != null){
			$('#only_once_mode_date_time_format').val(res.only_once_mode_configuration.date_time_format);
		}else{
			$('#only_once_mode_date_time_format').val("");
		}
		if(res.only_once_mode_configuration.mode != null){
			$('.oomModeRadio').prop("checked", false);
			$('.oomModeRadio[value="'+res.only_once_mode_configuration.mode+'"]').prop("checked", true);
			$('#'+$('.oomModeRadio[value="'+res.only_once_mode_configuration.mode+'"]').attr('toggle')).show();
		}
		if(res.only_once_mode_configuration.timer) {
			$('#only_once_mode_timer').prop("checked", res.only_once_mode_configuration.timer);
			$('#only_once_mode_timer_advanced').show();
		}
		if(res.only_once_mode_configuration.timer_autorefresh) {
			$('#only_once_mode_timer_refresh').prop("checked", res.only_once_mode_configuration.timer_autorefresh);
			$('#only_once_mode_timer_refresh_animations').show();
		}
		if(res.only_once_mode_configuration.timer_animation) {
			if(res.only_once_mode_configuration.timer_animation == 1){
				$('#only_once_mode_timer_animation_progressbar').prop("checked", true);
			}else if(res.only_once_mode_configuration.timer_animation == 2){
				$('#only_once_mode_timer_animation_black').prop("checked", true);
			}else{
				$('#only_once_mode_timer_animation_blur').prop("checked", true);
			}
		}
		if(res.only_once_mode_configuration.timer_min_duration != null){
			let dur = res.only_once_mode_configuration.timer_min_duration;
			let obj = msToDMHSObject(dur);
			$('#oom-min-timer-days').val(obj.days);
			$('#oom-min-timer-hours').val(obj.hours);
			$('#oom-min-timer-minutes').val(obj.minutes);
			$('#oom-min-timer-seconds').val(obj.seconds);
		}
		if(res.only_once_mode_configuration.timer_max_duration != null){
			let dur = res.only_once_mode_configuration.timer_max_duration;
			let obj = msToDMHSObject(dur);
			$('#oom-max-timer-days').val(obj.days);
			$('#oom-max-timer-hours').val(obj.hours);
			$('#oom-max-timer-minutes').val(obj.minutes);
			$('#oom-max-timer-seconds').val(obj.seconds);
		}


		if(res.only_once_mode_configuration.width_min != null){
			$('#only_once_mode_min_width').val(res.only_once_mode_configuration.width_min);
		}
		if(res.only_once_mode_configuration.height_min != null){
			$('#only_once_mode_min_height').val(res.only_once_mode_configuration.height_min);
		}
		if(res.only_once_mode_configuration.trigger != null){
			$('.oom-label').each(function() {
					let key = $(this).attr("label");
					let r = Object.values(klasses).find((e) => e.key == key);
					if(res.only_once_mode_configuration.trigger.includes(r.index)){
						$(this).prop("checked", true);
					}
			});
		}
		if(res.only_once_mode_configuration.display_classes){
			$('#only_once_mode_display_classes').prop("checked", true);
		}
		if(res.only_once_mode_configuration.mode_configuration[1].transparency != null){
			$('#only_once_mode_mode_see_trough_advanced_transparency').val(res.only_once_mode_configuration.mode_configuration[1].transparency*100);
		}

		if(res.only_once_mode_configuration.mode_configuration[2].distance != null){
			$('#only_once_mode_mode_border_advanced_distance').val(res.only_once_mode_configuration.mode_configuration[2].distance*10);
		}
		if(res.only_once_mode_configuration.mode_configuration[2].radius != null){
			$('#only_once_mode_mode_border_advanced_radius').val(res.only_once_mode_configuration.mode_configuration[2].radius);
		}
		
		if(res.only_once_mode_configuration.mode_configuration[3].blur != null){
			$('#only_once_mode_mode_thumbnail_advanced_blur').val(res.only_once_mode_configuration.mode_configuration[3].blur*100);
		}

		if(res.only_once_mode_configuration.mode_configuration[4].strength != null){
			$('#only_once_mode_mode_grid_advanced_strength').val(res.only_once_mode_configuration.mode_configuration[4].strength);
		}
		if(res.only_once_mode_configuration.mode_configuration[4].color_1 != null &&
			res.only_once_mode_configuration.mode_configuration[4].color_2 != null){
			$('#only_once_mode_mode_grid_advanced_color_1').val(res.only_once_mode_configuration.mode_configuration[4].color_1);
			$('#only_once_mode_mode_grid_advanced_color_1').css('background-color', res.only_once_mode_configuration.mode_configuration[4].color_1);
			$('#only_once_mode_mode_grid_advanced_color_2').val(res.only_once_mode_configuration.mode_configuration[4].color_2);
			$('#only_once_mode_mode_grid_advanced_color_2').css('background-color', res.only_once_mode_configuration.mode_configuration[4].color_2);
		}
		
		if(res.only_once_mode_configuration.mode_configuration[5].blur != null){
			$('#only_once_mode_mode_blur_advanced_blur').val(res.only_once_mode_configuration.mode_configuration[5].blur*100);
		}
		if(res.only_once_mode_configuration.mode_configuration[5].grayscale != null) {
			$('#only_once_mode_mode_blur_advanced_grayscale').prop("checked", res.only_once_mode_configuration.mode_configuration[5].grayscale);
		}
		
		if(res.only_once_mode_configuration.mode_configuration[6].allow_faces != null){
			$('#only_once_mode_mode_box_tease_advanced_allow_faces').prop("checked", true);
		}

		if(res.remote_configuration.check_mode != null){
			$("input[name=remoteCheckModeRadio]").val([res.remote_configuration.check_mode]);
		}
		if(res.remote_configuration.allow_older_settings){
			$("#remote-allow-old-settings").prop("checked", true);
		}

		$('.file_max_min').trigger('paste');
		$('.zero_look_disabled').trigger('paste');
	});
	let sticker_request = browser.storage.local.get(['sticker_collections']).then((res) => {
			sticker_collections = res.sticker_collections;
			//cacheStickerCollections(sticker_collections, null);
			$('#sticker_collections').empty();
			res.sticker_collections.forEach(function (collection, key) {
				$('#sticker_collections').append($('<option>', {
					value: key,
					text: capitalizeFirstLetter(collection._name),
					name: collection._name,
					class: ' ' + (collection._enabled ? 'sticker_collection_enabled' : 'sticker_collection_disabled') +
						' ' + (collection._locked ? 'sticker_collection_locked' : ''),
				}));
			});
	});

}

function restoreVideo() {
	let advanced = browser.storage.sync.get(['video_overlay']);
	advanced.then((res) => {
		if(res.video_overlay){
			$('.video_overlay').attr('check', true);
		}else{
			$('.video_overlay').removeAttr('check');
		}
	});
}

function restoreAdvanced() {
	let advanced = browser.storage.sync.get(['prescale', 'base64_scanner', 'ai_warm_up']);
	advanced.then((res) => {
		if(res.prescale){
			$('.prescale').attr('check', true);
		}else{
			$('.prescale').removeAttr('check');
		}
		if(res.base64_scanner){
			$('.base64_support').attr('check', true);
		}else{
			$('.base64_support').removeAttr('check');
		}
		if(res.ai_warm_up){
			$('.ai-warm-up').attr('check', true);
		}else{
			$('.ai-warm-up').removeAttr('check');
		}
	});
}

function restoreWhiteBlackList() {
	let whiteblacklist = browser.storage.sync.get(['whiteblacklist_configuration']);
	whiteblacklist.then((res) => {
		if(res.whiteblacklist_configuration.mode == 0){
			$("#radio-nolist").prop("checked", true);
			$('#whitelist-container').hide();
			$('#blacklist-container').hide();
		}else if(res.whiteblacklist_configuration.mode == 1){
			$("#radio-whitelist").prop("checked", true);
			$('#whitelist-container').show();
			$('#blacklist-container').hide();
		}else{
			$("#radio-blacklist").prop("checked", true);
			$('#whitelist-container').hide();
			$('#blacklist-container').show();
		}
		$('#whitelist').val(res.whiteblacklist_configuration.white_list.join('\n'));
		$('#blacklist').val(res.whiteblacklist_configuration.black_list.join('\n'));
		$('#blacklist_locked').val(res.whiteblacklist_configuration.black_list.join('\n'));
		$('#whitelist_locked').val(res.whiteblacklist_configuration.white_list.join('\n'));
	});
}

function restoreView() {
	let option_page_view = browser.storage.sync.get(['option_page_view','only_once_mode_configuration']);
	option_page_view.then((res) => {
		if(res.option_page_view){
			$('.tab[content='+res.option_page_view+']').click();
		}
		if(res.only_once_mode_configuration.mode){
			let elem = $('.oomModeRadio[value="'+res.only_once_mode_configuration.mode+'"]');
		}
	});
}

function updateHelp() {
	let cache = browser.storage.sync.get('do_cache');
	cache.then((res) => {
		$('.do_cacheToggle').each(function(i, obj) {
			if(res.do_cache){
				$(this).attr('check', true);
			}else{
				$(this).removeAttr('check');
			}
		});
	});
	let sending = browser.runtime.sendMessage({
		getdata: true,
	});
	sending.then(function(message){
		$('#ts-backend').html(message.tsbackend);
		$('#ts-tensors').html(message.tstensors);

		$('#ts-numBytes').html(bytesToSize(message.tsnumBytes));
		$('#ts-numBytesInGPU').html(bytesToSize(message.tsnumBytesInGPU));
		$('#ts-numDataBuffers').html(message.tsnumDataBuffers);
		$('#ts-unreliable').html(message.tsunreliable? 'False' : 'True');
		if(message.tsreasons){
			$('#help_error').show();
			$('#ts-reasons').html(message.tsreasons);
		}

		$('#cached-count').html(message.cache.size);

		let mem = 0;
		message.cache.forEach(function(value, key, map){
			 mem += value._imgdata.size+roughSizeOfObject(value);
		});
		mem = formatBytes(mem);
		$('#cached-size').html(mem);
	}, handleError);
}

function updateStatistics(){
	let sending = browser.runtime.sendMessage({
		syncstatictics: true,
	});
	sending.then((t) => {
		let cache = browser.storage.sync.get(['statistics','statistics_enabled']);
		cache.then((res) => {
			if(res.statistics_enabled){
				$('.statistics_enabled').attr('check', true);
			}else{
				$('.statistics_enabled').removeAttr('check');
			}
			$('#statistics-total').html(res.statistics.images.total);
			$('#statistics-total-positive').html(res.statistics.images.total_positive);
			if(res.statistics.images.total_average_duration){
				$('#statistics-total-average-duration').html(res.statistics.images.total_average_duration.toFixed(2)+' ms');
			}else{
				$('#statistics-total-average-duration').html('-');
			}
			if(res.statistics.images.total_average_ai_duration){
				$('#statistics-total-average-ai-duration').html(res.statistics.images.total_average_ai_duration.toFixed(2)+' ms');
			}else{
				$('#statistics-total-average-ai-duration').html('-');
			}
			if(res.statistics.images.total_average_paint_duration){
				$('#statistics-total-average-paint-duration').html(res.statistics.images.total_average_paint_duration.toFixed(2)+' ms');
			}else{
				$('#statistics-total-average-paint-duration').html('-');
			}
			$('#statistics-image-type-jpeg').html(res.statistics.images.type.jpeg);
			$('#statistics-image-type-png').html(res.statistics.images.type.png);
			$('#statistics-image-type-bmp').html(res.statistics.images.type.bmp);
			$('#statistics-image-type-webp').html(res.statistics.images.type.webp);
			$('#statistics-image-type-avif').html(res.statistics.images.type.avif);

			// GIF
			$('#statistics-image-type-gif').html(res.statistics.images.type.gif.full);
			$('#statistics-image-type-gif-thumb').html(res.statistics.images.type.gif.thumbnails);
			if(res.statistics.images.type.gif.total_average_frames){
				$('#statistics-image-type-gif-average-frame-count').html(res.statistics.images.type.gif.total_average_frames.toFixed(2));
			}else{
				$('#statistics-image-type-gif-average-frame-count').html('-');
			}
			if(res.statistics.images.type.gif.total_average_duration){
				$('#statistics-image-type-gif-average-duration').html(res.statistics.images.type.gif.total_average_duration.toFixed(2)+' ms');
			}else{
				$('#statistics-image-type-gif-average-duration').html('-');
			}

			$('#statistics-video-total').html(res.statistics.videos.total);
			$('#statistics-video-frames').html(res.statistics.videos.frames);
			$('#statistics-local-files-total').html(res.statistics.local_files.total);
			$('#statistics-batch-converter-total').html(res.statistics.batch_converter.total);

			$('#statistics-klasses').empty();
			let total = 0;
			let max_entry_total = -Infinity;
			for (const entry in res.statistics.klasses.label) {
				total += res.statistics.klasses.label[entry].total;
				if(res.statistics.klasses.label[entry].total > max_entry_total){
					max_entry_total = res.statistics.klasses.label[entry].total;
				}
			}
			for (const entry in res.statistics.klasses.label) {
				if (entry === 'NONE') {
					continue;
				}

				let entry_total = res.statistics.klasses.label[entry].total;
				let entry_avg = entry_total > 0 ? (100 / total * entry_total).toFixed(2) + "%" : '-';

				let $info = $(`.stat-bodypart-info[label="${entry}"]`);
				$info.children('.stat-bodypart-count').html(entry_total);
				$info.children('.stat-bodypart-percent').html(entry_avg);
				if (0 < total) {
					$info.children('.stat-bodypart-bar').css('width', (100 / total * entry_total) / (max_entry_total / total) + '%');
				} else {
					$info.children('.stat-bodypart-bar').css('width', '100%');
				}
			}
		});
	});
}

function showModalWindow(selector){
	$(selector).show();
	$('#overlay').show();
}

function hideModalWindow(selector){
	$(selector).hide();
	$('#overlay').hide();
}

let updateInterval;

function startUpdateInterval(){
	let lockd = null;
	let dur = null;
	updateInterval = window.setInterval(function(){
		// Check if extension got disabled
		let o = browser.storage.sync.get(['lock_configuration']);
		o.then((res) => {
			if(res.lock_configuration) {
				if (lockd === null) {
					lockd = res.lock_configuration.enabled;
					dur = res.lock_configuration.duration;
				} else if (lockd !== res.lock_configuration.enabled || dur !== res.lock_configuration.duration) {
					lockd = res.lock_configuration.enabled;
					dur = res.lock_configuration.duration;
					if(!res.lock_configuration.enabled){
						window.location.reload();
					}else{
						restoreLockUnlock();
					}
				}
			}
		});
	}, 1000);
}

$('#refresh-cache').click(function(){
	updateHelp();
});

$('#refresh-statistics').click(function(){
	updateStatistics();
});

$('#reset-statistics').click(function(){
	let sending = browser.runtime.sendMessage({
		resetstatistics: true
	});
	sending.then(function(message){
		updateStatistics();
	});
});

$('#clear-cache').click(function(){
	let sending = browser.runtime.sendMessage({
		clearcache: true
	});
	sending.then(function(message){
		updateHelp();
	});
});

$('.file_max_min').on("change keyup paste", function(evt) {
	let trnslt = bytesToSize(this.value*1000);
	if(this.value == 0 && $(this).hasClass("max_file")){
		trnslt = "<b style='font-size: 120%'>∞</b>";
	}
	let linked = $(this).attr("link");
	$('.file_size_translated[link="'+linked+'"]').html(" = "+trnslt);
});

$('.zero_look_disabled').on("change keyup paste", function(evt) {
	if(this.value == 0){
		$(this).addClass( "look_disabled" );
	}else{
		$(this).removeClass( "look_disabled" );
	}
});

function saveConfig(){
	let conf = browser.storage.sync.get(['bar_configuration','blur_configuration','pixel_configuration',
		'gif_configuration','png_configuration', 'jpg_configuration', 'bmp_configuration', 'webp_configuration', 'avif_configuration',
		'glitch_configuration', 'triangle_configuration','box_configuration', 'sticker_configuration', 'sobel_configuration', 'splatter_configuration',
		'mixed_configuration', 'random_configuration', 'reverse_mode_configuration', 'word_wall_configuration', 'lock_configuration',
		'whiteblacklist_configuration','clustering_configuration','only_once_mode_configuration']);
	conf.then((res) => {
		const censor_configs_by_type = {
			1: res.pixel_configuration,
			2: res.blur_configuration,
			3: res.bar_configuration,
			4: res.triangle_configuration,
			5: res.box_configuration,
			6: res.glitch_configuration,
			7: res.sticker_configuration,
			8: res.sobel_configuration,
			9: res.splatter_configuration,
			10: res.mixed_configuration,
			11: res.random_configuration,
		}

		for (const censor_type in censor_configs_by_type) {
			const censor_config = censor_configs_by_type[censor_type];

			const $tab_presets = $(`.presets[data-censor-type=${censor_type}] .tab-preset`);
			const selected_preset_index = $tab_presets.filter('.tab-preset-selected').attr("data-index");
			saveCensorPresetConfig[censor_type](censor_presets[censor_type][selected_preset_index]);
	
			for (let i = 0; i < $tab_presets.length; i++) {
				censor_presets[censor_type][i].name = $tab_presets.eq(i).children('input').val();
			}
	
			censor_config.selected_preset_index = selected_preset_index;
			censor_config.presets = censor_presets[censor_type];
		}

		res.clustering_configuration.mode = parseInt($('[name="clustering_mode"]:checked').val());
		res.reverse_mode_configuration.mode = parseInt($('[name="reverse_mode_mode"]:checked').val());

		res.word_wall_configuration.word_pool = $('#word_wall_word_pool').val().split('\n').filter(line => line.length !== 0);
		res.word_wall_configuration.draw_mode = parseFloat($('[name="word_wall_draw_mode"]:checked').val());
		res.word_wall_configuration.size = parseFloat($('#word_wall_size').val());
		res.word_wall_configuration.horizontal_spacing = parseFloat($('#word_wall_horizontal_spacing').val());
		res.word_wall_configuration.vertical_spacing = parseFloat($('#word_wall_vertical_spacing').val());
		res.word_wall_configuration.offset = parseFloat($('#word_wall_offset').val());
		res.word_wall_configuration.angle = parseFloat($('#word_wall_angle').val());
		res.word_wall_configuration.font = $('#word_wall_font').val();
		res.word_wall_configuration.color_mode = parseFloat($("#word_wall_color_mode").val());
		res.word_wall_configuration.color = $('#word_wall_color').val();

		res.whiteblacklist_configuration.mode = parseInt($("input:radio[name='wblist']:checked").val());
		let white_list = [];
		$.each($('#whitelist').val().split(/\n/), function (i, line) {
			if (line) {
				white_list.push(line);
			}
		});
		res.whiteblacklist_configuration.white_list = white_list;
		let black_list = [];
		$.each($('#blacklist').val().split(/\n/), function (i, line) {
			if (line) {
				black_list.push(line);
			}
		});
		res.whiteblacklist_configuration.black_list = black_list;

		res.only_once_mode_configuration.precision = Math.min(64,Math.max(58,$('#only_once_mode_precision').val()));
		res.only_once_mode_configuration.message = $('#only_once_mode_message').val();
		res.only_once_mode_configuration.date_time_format = $('#only_once_mode_date_time_format').val()? $('#only_once_mode_date_time_format').val() : "";
		res.only_once_mode_configuration.mode = parseInt($('.oomModeRadio').filter(":checked").val());

		res.only_once_mode_configuration.timer = $('#only_once_mode_timer').is(':checked')? true : false;
		res.only_once_mode_configuration.timer_autorefresh = $('#only_once_mode_timer_refresh').is(':checked')? true : false;

		res.only_once_mode_configuration.timer_animation = $('#only_once_mode_timer_animation_progressbar').is(':checked')? 1 :
			($('#only_once_mode_timer_animation_black').is(':checked')? 2 :
				($('#only_once_mode_timer_animation_blur').is(':checked')? 3 : 0
				));

		res.only_once_mode_configuration.mode_configuration[1].transparency =  Math.min(0.08,Math.max(0.01,$('#only_once_mode_mode_see_trough_advanced_transparency').val()/100));

		res.only_once_mode_configuration.mode_configuration[2].distance =  Math.min(0.2,Math.max(0.01,$('#only_once_mode_mode_border_advanced_distance').val()/100));
		res.only_once_mode_configuration.mode_configuration[2].radius =  Math.min(300,Math.max(0,$('#only_once_mode_mode_border_advanced_radius').val()));

		res.only_once_mode_configuration.mode_configuration[3].blur =  Math.min(0.1,Math.max(0.0,$('#only_once_mode_mode_thumbnail_advanced_blur').val()/100));

		res.only_once_mode_configuration.mode_configuration[4].strength =  Math.min(1,Math.max(0.1,$('#only_once_mode_mode_grid_advanced_strength').val()));
		res.only_once_mode_configuration.mode_configuration[4].color_1 = $('#only_once_mode_mode_grid_advanced_color_1').val();
		res.only_once_mode_configuration.mode_configuration[4].color_2 = $('#only_once_mode_mode_grid_advanced_color_2').val();

		res.only_once_mode_configuration.mode_configuration[5].blur =  Math.min(0.2,Math.max(0.01,$('#only_once_mode_mode_blur_advanced_blur').val()/100));
		res.only_once_mode_configuration.mode_configuration[5].grayscale = $
		('#only_once_mode_mode_blur_advanced_grayscale').is(':checked') ? true : false;

		res.only_once_mode_configuration.mode_configuration[6].allow_faces = $('#only_once_mode_mode_box_tease_advanced_allow_faces').is(':checked')? true : false;

		if(!res.only_once_mode_configuration.mode){
			res.only_once_mode_configuration.mode = 0;
		}
		res.only_once_mode_configuration.width_min = Math.max(0,$('#only_once_mode_min_width').val());
		res.only_once_mode_configuration.height_min = Math.max(0,$('#only_once_mode_min_height').val());
		let triggers = [];
		$('.oom-label').each(function() {
			if($(this).is(':checked')) {
				let key = $(this).attr("label");
				let r = Object.values(klasses).find((e) => e.key == key);
				triggers.push(r.index);
			}
		});
		res.only_once_mode_configuration.trigger = triggers;
		res.only_once_mode_configuration.display_classes = $('#only_once_mode_display_classes').is(':checked')? true : false;

		let save_obj = {
			bar_configuration: res.bar_configuration,
			blur_configuration: res.blur_configuration,
			pixel_configuration: res.pixel_configuration,
			glitch_configuration: res.glitch_configuration,
			triangle_configuration: res.triangle_configuration,
			box_configuration: res.box_configuration,
			sticker_configuration: res.sticker_configuration,
			sobel_configuration: res.sobel_configuration,
			splatter_configuration: res.splatter_configuration,
			mixed_configuration: res.mixed_configuration,
			random_configuration: res.random_configuration,
			gif_configuration: res.gif_configuration,
			png_configuration: res.png_configuration,
			jpg_configuration: res.jpg_configuration,
			bmp_configuration: res.bmp_configuration,
			avif_configuration: res.avif_configuration,
			webp_configuration: res.webp_configuration,
			whiteblacklist_configuration: res.whiteblacklist_configuration,
			reverse_mode_configuration: res.reverse_mode_configuration,
			word_wall_configuration: res.word_wall_configuration,
			clustering_configuration: res.clustering_configuration,
			only_once_mode_configuration: res.only_once_mode_configuration,
		}

		if(res.lock_configuration.enabled){
			delete save_obj.bar_configuration;
			delete save_obj.blur_configuration;
			delete save_obj.pixel_configuration;
			delete save_obj.glitch_configuration;
			delete save_obj.triangle_configuration;
			delete save_obj.box_configuration;
			delete save_obj.sticker_configuration;
			delete save_obj.sobel_configuration;
			delete save_obj.splatter_configuration;
			delete save_obj.mixed_configuration;
			delete save_obj.random_configuration;
			delete save_obj.gif_configuration;
			delete save_obj.png_configuration;
			delete save_obj.jpg_configuration;
			delete save_obj.bmp_configuration;
			delete save_obj.avif_configuration;
			delete save_obj.webp_configuration;
			//delete save_obj.whiteblacklist_configuration;
			delete save_obj.word_wall_configuration;
			delete save_obj.clustering_configuration;
			delete save_obj.only_once_mode_configuration;
		}
		browser.storage.sync.set(save_obj).then(function(){
			updateOptionsContentScript();
		});
	});
}

$(':radio[hide]').click(function(){
	$('.'+$(this).attr('hide')).hide();
});

$('button[toggle]').click(function(){
	$('#'+$(this).attr('toggle')).toggle();
	$('.'+$(this).attr('toggle')).toggle();
});
$('input[toggle]').click(function(){
	$('#'+$(this).attr('toggle')).toggle();
	$('.'+$(this).attr('toggle')).toggle();
});
$(':radio[toggle]').click(function(){
	$('#'+$(this).attr('toggle')).show();
	$('.'+$(this).attr('toggle')).show();
});

$('#radio-nolist[toggle]').click(function(){
	if($(this).is(':checked')){
		$('#whitelist-container').hide();
		$('#blacklist-container').hide();
	}
});
$('#radio-whitelist[toggle]').click(function(){
	if($(this).is(':checked')){
		$('#whitelist-container').show();
		$('#blacklist-container').hide();
	}
});
$('#radio-blacklist[toggle]').click(function(){
	if($(this).is(':checked')){
		$('#whitelist-container').hide();
		$('#blacklist-container').show();
	}
});

$('.tab').on('click',function(){
	let id = $(this).attr('id');
	if(id !== 'gallery_viewer'){
		$('.tab').removeClass('tab-selected');
		$(this).addClass('tab-selected');
		$('.content').removeClass('content-displayed');
		$('#'+$(this).attr('content')).addClass('content-displayed');
		browser.storage.sync.set({
			option_page_view: $(this).attr('content'),
		});
	}
	setTimeout(function() {
		const url = new URL(window.location);
		url.hash = '';
		window.history.pushState({}, '', url);
	}, 10);
});

$('io-toggle').click(function(e) {
	toggleIoToggle($(this).add($(this).children('button')));
});

$('.onoff').click(function(e) {
	e.stopImmediatePropagation();
	if($(this).attr('disabled') === 'disabled'){
		return;
	}
	$('#onoff').html($(this).attr('check')? 'ON' : 'OFF');
	activatePuryFi($(this).attr('check')? true : false);
});

$('io-toggle.labelToggle').click(function(e){
	let labels = browser.storage.sync.get(['labels', 'lock_configuration']);
	labels.then((res) => {
		if(res.lock_configuration.enabled) {
			return;
		}
		
		let labellist = res.labels || [];
		const label = $(this).attr('label');
		if($(this).attr('check')){
			if (labellist.indexOf(label) === -1) {
				labellist.push(label);
				$(`button.labelButton[id="${label}"]`).addClass('selected');
				setMixedLabelTypeRowSelected(parseKlassFromKey(label).index, true);
			}
		}else{
			if (labellist.indexOf(label) !== -1) {
				labellist = removeItemAll(labellist, label);
				$(`button.labelButton[id="${label}"]`).removeClass('selected');
				setMixedLabelTypeRowSelected(parseKlassFromKey(label).index, false);
			}
		}
		browser.storage.sync.set({
			labels: labellist
		}).then(function(){
			updateOptionsContentScript();
		});
	});
});


$('io-toggle.base64Toggle').click(function(e){
	//e.stopImmediatePropagation();
	browser.storage.sync.set({
		base64_scanner: $(this).attr('check')  ? true : false
	}).then(function(){
		updateOptionsContentScript();
	});
});

$('io-toggle.statisticsToggle').click(function(e){
	//e.stopImmediatePropagation();
	browser.storage.sync.set({
		statistics_enabled: $(this).attr('check')  ? true : false
	}).then(function(){
		updateOptionsContentScript();
	});
});

$('.prescale').click(function(e){
	browser.storage.sync.set({
		prescale: $(this).attr('check')  ? true : false
	}).then(function(){
		updateOptionsContentScript();
	});
});

$('.ai-warm-up').click(function(e){
	//e.stopImmediatePropagation();
	browser.storage.sync.set({
		ai_warm_up: $(this).attr('check')  ? true : false
	}) // We do not need to update the content script here
});

$('button.typecm').click(function(e){
	browser.storage.sync.get(['clustering_configuration','lock_configuration']).then((res) => {
		if(res.lock_configuration.enabled) return;
		let value = $(this).hasClass('selected');
		if (value) {
			$('.clusteringToggle').removeAttr('check');
			$(this).removeClass('selected');
		} else {
			$('.clusteringToggle').attr('check', true);
			$(this).addClass('selected');
		}
		res.clustering_configuration.enabled = !value;
		browser.storage.sync.set({
				clustering_configuration: res.clustering_configuration
			}).then(function(){
				updateOptionsContentScript();
			});
	});
});

$('io-toggle.clusteringToggle').click(function(e){
	browser.storage.sync.get(['clustering_configuration','lock_configuration']).then((res) => {
		if(res.lock_configuration.enabled) return; 
		let value = $(this).attr('check') ? true : false;
		if (value) {
			$('button.typecm').addClass('selected');
		} else {
			$('button.typecm').removeClass('selected');
		}
		res.clustering_configuration.enabled = value;
		browser.storage.sync.set({
			clustering_configuration: res.clustering_configuration
		}).then(function(){
			updateOptionsContentScript();
		});
	});
});

$('#sticker_enabled_sources').on('focus', function() {
    if (!$(this).data('autocomplete')) {
        browser.storage.local.get(['sticker_collections']).then((res) => {
            var availableStickers = res.sticker_collections.map(collection => collection._name);
            $(this).autocomplete({
                source: function(request, response) {
                    var term = request.term.split(';').pop().trim();
                    var currentStickers = request.term.split(';').map(item => item.trim());
                    var suggestions = availableStickers.filter(sticker => !currentStickers.includes(sticker));
                    var matches = $.ui.autocomplete.filter(suggestions, term);
                    response(matches);
                },
                focus: function() {
                    return false;
                },
                select: function(event, ui) {
                    var terms = this.value.split(';');
                    terms.pop();
                    terms.push(ui.item.value);
                    terms.push("");
                    this.value = terms.join(";");
                    return false;
                },
            });
        });
    }
});

$('#sticker_enabled_sources').on('change focusout', function() {
    // TODO: validate the inputted names refer to existing collections or groups
    $(this).val($(this).val().toLowerCase());
})
function updateOnTriangleFillColorModeChange() {
	if ($(this).val() == 0) {
		$('#triangle_fill_color').closest('.general-setting').hide();
	} else {
		$('#triangle_fill_color').closest('.general-setting').show();
	}
}

function updateOnTriangleStrokeColorModeChange() {
	if ($(this).val() == 0) {
		$('#triangle_stroke_color').closest('.general-setting').hide();
	} else {
		$('#triangle_stroke_color').closest('.general-setting').show();
	}
}

function updateOnTriangleGradientsChange() {
	if (!$(this).is(':checked')) {
		$('#triangle_gradient_stops').closest('.general-setting').hide();
	} else {
		$('#triangle_gradient_stops').closest('.general-setting').show();
	}
}

function updateOnGlitchTypeChange() {
	$('#glitch_multiple_panels_preview').find('#glitch_multiple_panels_preset_name').html('');
	if($(this).val() == 0){
		$('[id^="glitch_chromatic_aberration"]').closest('.general-setting').show();
		$('[id^="glitch_multiple_panels"]').closest('.general-setting').hide();
	}else if($('#glitch_type').val() == 1){
		$('[id^="glitch_multiple_panels"]').closest('.general-setting').show();
		$('[id^="glitch_chromatic_aberration"]').closest('.general-setting').hide();
		previewGlitchMultiplePanelsPreset();
	}
}

$('#triangle_fill_color_mode').change(updateOnTriangleFillColorModeChange);
$('#triangle_stroke_color_mode').change(updateOnTriangleStrokeColorModeChange);
$('#triangle_gradients').change(updateOnTriangleGradientsChange);
$('#glitch_type').change(updateOnGlitchTypeChange);

function toggleIoToggle($toggle) {
	if($toggle.attr('disabled') === 'disabled' || $toggle.hasClass('patreon_locked')) return;
	
	if ($toggle.hasClass('lock-on-toggle')) {
		$toggle.attr('check', true);	
	} else {
		$toggle.toggleAttr('check', true);
	}
}

$('#batch_converter').click(function(e){
	let file_types = browser.storage.sync.get(['file_types','gif_configuration']);
	file_types.then((res) => {
		let file_typeslist = res.file_types || [];
		$('#batch_converter_png').addClass('filetype-inactive');
		$('#batch_converter_jpg').addClass('filetype-inactive');
		$('#batch_converter_bmp').addClass('filetype-inactive');
		$('#batch_converter_webp').addClass('filetype-inactive');
		$('#batch_converter_gif').addClass('filetype-inactive');
		file_typeslist.forEach(function(type){
			$('#batch_converter_'+type).removeClass('filetype-inactive');
		});
		if(res.gif_configuration._thumbnails){
			$('#batch_converter_gif').removeClass('filetype-inactive');
		}
	});
});

$('io-toggle.overlayToggle').click(function(e){
	let sync = browser.storage.sync.get(['lock_configuration']);
	sync.then((res) => {
		if(res.lock_configuration.enabled && res.lock_configuration.locked_options.includes('video'))return;

		browser.storage.sync.set({
			video_overlay: $(this).attr('check') ? true : false
		}).then(function(){
			updateOptionsContentScript();
		});

	});
});

$('io-toggle.debugToggle').click(function(e){
	$('#'+$(this).attr('label')).toggleClass("selected", $(this).attr('check') ? true : false);
	let debug = browser.storage.sync.get('debug');
	debug.then((res) => {
		let debugging = res.debug || false;
		browser.storage.sync.set({
			debug: ($(this).attr('check')? true : false)
		}).then(function(){
			updateOptionsContentScript();
		});
	});
});

$('io-toggle.do_cacheToggle').click(function(e){
	$('#'+$(this).attr('label')).toggleClass("selected", $(this).attr('check') ? true : false);
	let censor = browser.storage.sync.get('do_cache');
	censor.then((res) => {
		browser.storage.sync.set({
			do_cache:  ($(this).attr('check') ? true : false)
		}).then(function(){
			updateOptionsContentScript();
		});
	});
});

// jquery toggle whole attribute
$.fn.toggleAttr = function(attr, val) {
	var test = $(this).attr(attr);
	if ( test ) {
		// if attrib exists with ANY value, still remove it
		$(this).removeAttr(attr);
	} else {
		$(this).attr(attr, val);
	}
	return this;
};

// jquery toggle just the attribute value
$.fn.toggleAttrVal = function(attr, val1, val2) {
	var test = $(this).attr(attr);
	if ( test === val1) {
		$(this).attr(attr, val2);
		return this;
	}
	if ( test === val2) {
		$(this).attr(attr, val1);
		return this;
	}
	// default to val1 if neither
	$(this).attr(attr, val1);
	return this;
};

$(".labelButton").on('click', function(event){
	let labels = browser.storage.sync.get(['labels', 'lock_configuration']);
	labels.then((res) => {
		if(res.lock_configuration.enabled){
			return;
		}
		$(this).toggleClass("selected");
		const id = $(this).attr('id');
		if($(this).hasClass("selected")){
			$('button[label='+id+']').attr('check', true);
			$('button[label='+id+']').parent().attr('check', true);
		}else{
			$('button[label='+id+']').removeAttr('check');
			$('button[label='+id+']').parent().removeAttr('check');
		}
		let labellist = res.labels || [];
		if($(this).hasClass("selected")){
			if(labellist.indexOf(this.id) === -1){
				labellist.push(this.id)
				setMixedLabelTypeRowSelected(parseKlassFromKey(id).index, true);
			}
		}else{
			if(labellist.indexOf(this.id) !== -1){
				labellist = removeItemAll(labellist, this.id);
				setMixedLabelTypeRowSelected(parseKlassFromKey(id).index, false);
			}
		}
		browser.storage.sync.set({
			labels: labellist
		}).then(function(){
			updateOptionsContentScript();
		});
	});
});

function expandGeneralAccordion() {
	let $settings_cont = $(this).find('.general-settings-container');
	$settings_cont.removeClass('collapsed');
	let ver_padding = parseInt($settings_cont.css('padding-top')) + parseInt($settings_cont.css('padding-bottom'));
	let scroll_height = $settings_cont.prop('scrollHeight');

	const top_margin = 35;
	const bottom_margin = 50;
	$settings_cont.stop().animate({
			height: scroll_height - ver_padding + 'px'
		}, { 
			duration: scroll_height / 3,
			step: function(now, fx) {
				let bottom = $settings_cont[0].getBoundingClientRect().bottom;
				if (bottom + bottom_margin > window.innerHeight) {
					window.scrollBy({ top: bottom + bottom_margin - window.innerHeight, behavior: 'smooth' });
				}
			},
			complete: function() {
				$settings_cont.css('overflow', 'visible');
				$settings_cont.css('height', 'auto');
			}
		} 
	);
	let top = $(this)[0].getBoundingClientRect().top;
	if (top - top_margin < 0) {
		window.scrollBy({ top: top - top_margin, behavior: 'smooth' });
	}

	$(this).next('.general-accordion-buttons').show();
	$(this).find('.general-accordion-collapsible').html('-');
}

function collapseGeneralAccordion() {
	let $settings_cont = $(this).find('.general-settings-container');
	$settings_cont.addClass('collapsed');
	let scroll_height = $settings_cont.prop('scrollHeight');
	
	$settings_cont.css('overflow', 'hidden');
	$settings_cont.stop().animate({
			height: 0
		}, {
			duration: scroll_height / 3,
			easing: 'linear',
		}
	);

	$(this).find('.general-accordion-collapsible').html('+');
	$(this).next('.general-accordion-buttons').hide();
}

function toggleGeneralAccordion() {
	if (!$(this).find('.general-settings').length) return;

	if ($(this).find('.general-accordion-collapsible').html() === "+") {
		expandGeneralAccordion.call(this);
	} else {
		collapseGeneralAccordion.call(this);
	}
}

// collapseGeneralAccordion.call($('.general-accordion-container'));

$(".general-accordion").on('click', function(e) {
	if ($(e.target).is('.icon-button-l, .icon-button-l *, io-toggle, io-toggle *')) return;
	let res = browser.storage.sync.get(['lock_configuration']);
	res.then((res) => {
		if(res.lock_configuration.enabled) return;
		toggleGeneralAccordion.call($(this).closest('.general-accordion-container')) 
	});
});

function enableCensorType($accordion) {
	$('.censor-type-toggle').removeAttr('check');
	$('.type-button-l').removeClass('selected');
	$accordion.find('.censor-type-toggle').attr('check', true);
	$accordion.find('.type-button-l').addClass('selected');

	const censor_type = parseInt($accordion.attr('data-censor-type'));
	browser.storage.sync.set({
		censor_type: parseEffectFromIndex(censor_type).name
	}).then(function(){
		updateOptionsContentScript();
	});
}

$('.censor-type-toggle, .type-button-l').on('click', function(e) {
	let res = browser.storage.sync.get(['lock_configuration']);
	res.then((res) => {
		if (res.lock_configuration.enabled) return;
		enableCensorType($(this).closest('.general-accordion-container'));
	})
});

$(".debugButton").on('click', function(e){
	if(!$(this).hasClass("selected")){
		$(this).addClass("selected");
		$('.debugToggle').attr('check', true);
	}else{
		$(this).removeClass("selected");
		$('.debugToggle').removeAttr('check');
	}
	let debug = browser.storage.sync.get('debug');
	debug.then((res) => {
		let debugging = res.debug || false;
		browser.storage.sync.set({
			debug: $(this).hasClass("selected")
		}).then(function(){
			updateOptionsContentScript();
		});
	});
});

$("#blacklist-locked-add").on('click', function(event){
	let whiteblacklist = browser.storage.sync.get(['whiteblacklist_configuration']);
	whiteblacklist.then((res) => {
		let val = $("#blacklist-locked-input").val();
		if (!val) return;
		$("#blacklist-locked-input").val('');

		let black_list = res.whiteblacklist_configuration.black_list;
		black_list.push(val);
		res.whiteblacklist_configuration.black_list = black_list;
		browser.storage.sync.set({
			whiteblacklist_configuration: res.whiteblacklist_configuration,
		}).then(function(){
			updateOptionsContentScript();
			$('#blacklist_locked').val(res.whiteblacklist_configuration.black_list.join('\n'));
		});
	});
});

$('#restore_default_settings-button').on('click',function() {
	if (window.confirm("Warning: This will reset all configurations to its default parameters.")) {
		let clearStorage = browser.storage.sync.clear();
		clearStorage.then(e => {
			resetOptions();
			window.location.reload();
		});
	}
});

let censor_preset_config_clipboard = null;

$('.copy-button').click(function(e) {
	let censor_type = parseInt($(this).parent().attr('data-censor-type'));
	let selected_preset_index = parseInt($(`.presets[data-censor-type="${censor_type}"] .tab-preset-selected`).attr('data-index'));
	censor_preset_config_clipboard = {
		type: censor_type,
		preset_config: structuredClone(censor_presets[censor_type][selected_preset_index].config),
	}
})

$('.paste-button').click(function(e) {
	if (censor_preset_config_clipboard == null) return;
	
	let censor_type = parseInt($(this).parent().attr('data-censor-type'));
	
	let selected_preset_index = parseInt($(`.presets[data-censor-type="${censor_type}"] .tab-preset-selected`).attr('data-index'));

	let censor_preset = censor_presets[censor_type][selected_preset_index];
	for (key in censor_preset_config_clipboard.preset_config) {
		if (key === 'type' && censor_type !== censor_preset_config_clipboard.type) continue;
		if (censor_preset.config[key] !== undefined && censor_preset.config[key] !== censor_preset_config_clipboard.preset_config[key]) {
			censor_preset.config[key] = censor_preset_config_clipboard.preset_config[key];
		}
	}

	updateCensorPresetConfig[censor_type](censor_preset, true);
	saveConfig();
})

function animateHighlight($elem) {
	$elem.stop();
	$({ brightness: 1 }).animate({ brightness: 2 }, {
		duration: 50,
		step: function(now) {
			$elem.css('filter', `brightness(${now})`);
		},
		complete: function() {
			$elem.css('filter', 'brightness(2)');
			$({ brightness: 2 }).animate({ brightness: 1 }, {
				duration: 550,
				step: function(now) {
					$elem.css('filter', `brightness(${now})`);
				},
				complete: function() {
					$elem.css('filter', '');
				},
				easing: 'easeInQuad',
				queue: false,
			});
		},
		easing: "linear",
		queue: false,
	})
}