$("a[content='content-cloud']").click(function() {
    updateRemoteTokenDisplay();
});

$('#remote-stored-tokens').on('click', '#remote-new-token', function(){
    $(this).prop("disabled", true);
    createRemoteToken();
});

$('#remote-stored-tokens').on('click', '.remote-token-delete', function(){
    let identifier = $(this).attr('remote_identifier');
    if(confirm("Do you wish to delete remote token '"+identifier+"'?")) {
        identifier = identifier.replace('puryfi-remote:', '');
        deleteRemoteToken(identifier);
    }
});

function createRemoteToken(){
    let sync = browser.storage.sync.get(['user']);
    sync.then((res) => {
        if(res.user == null){
            return;
        }
        let xmlhttp = new XMLHttpRequest();
        let url = "https://pury.fi/site/wp-json/remote_control/new_token";
        let params = 'username=' + encodeURIComponent(res.user.username) + '&password=' + encodeURIComponent(res.user.password);
        xmlhttp.onreadystatechange = function () {
            if (this.readyState == 4 && this.status == 200) {
                try {
                    let data = JSON.parse(this.responseText);
                    if (data) {
                        if (data.hasOwnProperty('error')) {
                            alert(data.error);
                        } else {
                            updateRemoteTokenDisplay();
                        }
                    }
                    reEnableRemoteCreate();
                } catch (e) {
                    log(e);
                    reEnableRemoteCreate();
                }
            }
        };
        xmlhttp.ontimeout = function (e) {
            log(e);
            reEnableRemoteCreate();
        };
        xmlhttp.open('POST', url, true);
        xmlhttp.timeout = 1000;
        xmlhttp.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
        xmlhttp.send(params);
    });
}

function reEnableRemoteCreate(){
    setTimeout(()=> {
        $('#remote-new-token').prop("disabled",false);
    },1000);
}

function updateRemoteTokenDisplay(){
    let elem = $("#remote-new-token-tr");
    let tbody = $("#remote-stored-tokens").find('tbody');
    tbody.empty();
    tbody.append(elem);
    let sync = browser.storage.sync.get(['user']);
    sync.then((res) => {
        if(res.user == null){
            return;
        }
        let xmlhttp = new XMLHttpRequest();
        let url = "https://pury.fi/site/wp-json/remote_control/info";
        let params = 'username=' + encodeURIComponent(res.user.username) + '&password=' + encodeURIComponent(res.user.password);
        xmlhttp.onreadystatechange = function () {
            if (this.readyState == 4 && this.status == 200) {
                try {
                    let data = JSON.parse(this.responseText);
                    if (data) {
                        if (data.hasOwnProperty('error')) {
                            alert(data.error);
                        } else {
                            $("#remote-token-current").html(data.info.storage_current);
                            $("#remote-token-max").html(data.info.storage_max);
                            data['token'].forEach(function (settings_entry, index) {
                                $("#remote-stored-tokens").find('tbody')
                                    .prepend($('<tr>')
                                        .append($('<td>')
                                            .text('puryfi-remote:'+settings_entry.identifier)
                                            .append($('<i remote_identifier="'+settings_entry.identifier+'" class="fas fa-copy remote-token-copy remote-token-copy-identifier" style="float:right;" ></i>'))
                                        ).append($('<td style="display: flex;">')
                                            .append($('<input remote_identifier="'+settings_entry.identifier+
                                                '" type="text" value="'+(settings_entry.cloud_settings != null?'puryfi-settings:'+settings_entry.cloud_settings:"")+
                                                '" class="cloud-setting-remote-input" placeholder="Enter a settings cloud id" disabled>')) //todo: put style in css
                                            .append(
                                                $('<button remote_identifier="'+settings_entry.identifier+'" class="icon-btn remote-token-edit-settings" style="width: 22px;">')
                                                    .append($('<i class="fa fa-edit"></i>'))
                                            )
                                            .append(
                                                $('<button remote_identifier="'+settings_entry.identifier+'" class="icon-btn remote-token-save-settings" style="width: 22px;display: none;">')
                                                    .append($('<i class="fa fa-save"></i>'))
                                            )
                                        ).append($('<td>')
                                                .text(settings_entry.users).append($('<i remote_identifier="'+settings_entry.identifier+'" class="remote-user-info fas fa-users" style="margin-left: 5px;"></i>'))
                                        ).append($('<td>')
                                            .append($('<button remote_identifier="'+settings_entry.identifier+'" class="icon-btn remote-token-delete">')
                                            .append($('<i class="fa fa-trash"></i>'))))
                                    );
                            });
                        }
                    }
                    reEnableCloudUpload();
                } catch (e) {
                    log(e);
                    reEnableCloudUpload();
                }
            }
        };
        xmlhttp.ontimeout = function (e) {
            log(e);
            reEnableCloudUpload();
        };
        xmlhttp.open('POST', url, true);
        xmlhttp.timeout = 1000;
        xmlhttp.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
        xmlhttp.send(params);
    });
}

function deleteRemoteToken(identifier){
    let sync = browser.storage.sync.get(['user']);
    sync.then((res) => {
        if(res.user == null){
            alert("Please login to use this feature!");
            return;
        }
        let xmlhttp = new XMLHttpRequest();
        let url = "https://pury.fi/site/wp-json/remote_control/delete_token";
        let params = 'username=' + encodeURIComponent(res.user.username) + '&password=' + encodeURIComponent(res.user.password);
        params += '&identifier=' + encodeURIComponent(identifier);
        xmlhttp.onreadystatechange = function () {
            if (this.readyState == 4 && this.status == 200) {
                try {
                    let data = JSON.parse(this.responseText);
                    updateRemoteTokenDisplay();
                } catch (e) {
                    updateRemoteTokenDisplay();
                    log(e);
                }
            }
        };
        xmlhttp.ontimeout = function (e) {
            updateRemoteTokenDisplay();
            log(e);
        };
        xmlhttp.open('POST', url, true);
        xmlhttp.timeout = 1000;
        xmlhttp.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
        xmlhttp.send(params);
    });
}

function bindCloudSettingsToToken(token_identifier, cloud_identifier){
        let sync = browser.storage.sync.get(['user']);
        sync.then((res) => {
            if(res.user == null){
                return;
            }
            let xmlhttp = new XMLHttpRequest();
            let url = "https://pury.fi/site/wp-json/remote_control/bind";
            let params = 'username=' + encodeURIComponent(res.user.username) + '&password=' + encodeURIComponent(res.user.password);
            params += '&identifier=' + encodeURIComponent(token_identifier);
            params += '&cloud_settings=' + encodeURIComponent(cloud_identifier);
            xmlhttp.onreadystatechange = function () {
                if (this.readyState == 4 && this.status == 200) {
                    try {
                        let data = JSON.parse(this.responseText);
                        if (data) {
                            if (data.hasOwnProperty('error')) {
                                alert(data.error);
                            } else {
                                updateRemoteTokenDisplay();
                            }
                        }
                    } catch (e) {
                        log(e);
                    }
                }
            };
            xmlhttp.ontimeout = function (e) {
                log(e);
            };
            xmlhttp.open('POST', url, true);
            xmlhttp.timeout = 1000;
            xmlhttp.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
            xmlhttp.send(params);
        });
}

function updateRemoteSubscription(){
    return new Promise((resolve, reject) => {
        let sync = browser.storage.sync.get(['remote_configuration', 'lock_configuration']);
        sync.then((res) => {
            $("#remote-error-name").html("&nbsp;");
            $("#remote-error-info").html("&nbsp;");
            $("#remote-error").hide();
            let remote_lock_elem = $('#remote-lock');
            if (res.remote_configuration.remote_token_identifier === null) {
                $('#remote-settings-identifier').val('');
                $("#remote-settings-subscribe").show();
                $("#remote-settings-unsubscribe").hide();
                $("#remote-timestamps").hide();
                if (!res.lock_configuration.enabled) {
                    $("#remote-settings-identifier").prop('disabled', false);
                }
                if (res.lock_configuration.enabled) {
                    $('#remote-settings').hide();
                }
                if (res.remote_configuration.lock) {
                    $("#remote-settings-subscribe").prop('disabled', true);
                }
                remote_lock_elem.prop("checked", false);
                remote_lock_elem.prop('disabled', false);
                $('#remote-statistic').prop('disabled', false);
                $('#remote-allow-old-settings').prop('disabled', false);
            } else {
                $('#remote-settings-identifier').val("puryfi-remote:" + res.remote_configuration.remote_token_identifier);
                $("#remote-settings-identifier").prop('disabled', true);
                $("#remote-last-updated").html(new Date(res.remote_configuration.last_updated).toLocaleString());
                $("#remote-last-checked").html(new Date(res.remote_configuration.last_checked).toLocaleString());
                $("#remote-timestamps").show();
                if (res.remote_configuration.lock) {
                    $('#remote-statistic').prop('disabled', true);
                    $('#remote-allow-old-settings').prop('disabled', true);
                    $('.remote-check-period').prop('disabled', true);
                    $('#remote-check-periodic-minutes').prop('disabled', true);
                    $("#remote-settings-subscribe").hide();
                    $("#remote-settings-unsubscribe").show();
                    $("#remote-settings-unsubscribe").addClass('locked');
                    remote_lock_elem.prop("checked", true);
                } else {
                    $('#remote-statistic').prop('disabled', false);
                    $('#remote-allow-old-settings').prop('disabled', false);
                    $('.remote-check-period').prop('disabled', false);
                    $('#remote-check-periodic-minutes').prop('disabled', false);
                    $("#remote-settings-subscribe").hide();
                    $("#remote-settings-unsubscribe").show();
                }
                remote_lock_elem.prop('disabled', true);
                if (res.remote_configuration.send_statistics_data) {
                    $('#remote-statistic').prop("checked", true);
                }
            }
            resolve();
        });
    });
}

function subscribeToRemoteControl(res, remote_identifier, lock, send_statistic, check_period){
    registerRemoteSubscription(remote_identifier).then((data) => {
        res.remote_configuration.lock = lock;
        res.remote_configuration.send_statistics_data = send_statistic;
        res.remote_configuration.remote_token_identifier = remote_identifier;
        res.remote_configuration.check_period = check_period;
        browser.storage.sync.set({
            remote_configuration: res.remote_configuration
        }).then(() => {
            updateRemoteSubscription().then(()=>{
                loadRemoteSettings();
            }).catch((e) => {
                console.log(e);
            });
        });
    }).catch((e) => {
        alert(e);
        console.log(e);
    });
}

function unsubscribeToRemoteControl(res, ignore_error = false){
    cancelRemoteSubscription(res.remote_configuration.remote_token_identifier).then((data) => {
        resetRemoteControl(res).then(() => {}).catch((e) => {console.log(e)});
    }).catch((e) => {
        if(e === 'UNKNOWN_TOKEN'){
            resetRemoteControl(res).then(() => {}).catch((e) => {console.log(e)});
        }else if(ignore_error || window.confirm("Pury.Fi encountered an error ("+e+"),\n" +
                   "do you wish to unsubscribe locally only?")){
            resetRemoteControl(res).then(() => {}).catch((e) => {console.log(e)});
        }
    }).finally(() => {

    });
}

function resetRemoteControl(res){
    return new Promise((resolve, reject) => {
        res.remote_configuration.lock = false;
        res.remote_configuration.send_statistics_data = false;
        res.remote_configuration.remote_token_identifier = null;
        res.remote_configuration.last_updated = null;
        res.remote_configuration.last_checked = null;
        browser.storage.sync.set({
            remote_configuration: res.remote_configuration
        }).then(() => {
            updateOptionsContentScript().then(() => {
                updateRemoteSubscription().then(() => {
                    resolve();
                    if(res.lock_configuration.enabled && res.lock_configuration.remote_lock){
                        unlockExtension();
                    }
                }).catch((e) => {console.log(e)});
            });
        }).catch((e) => {console.log(e)});
    });
}

function registerRemoteSubscription(identifier) {
    return new Promise((resolve, reject) => {
        let sync = browser.storage.sync.get(['user']);
        sync.then((res) => {
            if (res.user == null) {
                reject("No user data");
                return;
            }
            let xmlhttp = new XMLHttpRequest();
            let url = "https://pury.fi/site/wp-json/remote_control/subscribe";
            let params = 'username=' + encodeURIComponent(res.user.username) + '&password=' + encodeURIComponent(res.user.password);
            params += '&identifier=' + encodeURIComponent(identifier);
            xmlhttp.onreadystatechange = function () {
                if (this.readyState == 4 && this.status == 200) {
                    try {
                        let data = JSON.parse(this.responseText);
                        if (data) {
                            if (data.hasOwnProperty('error')) {
                                reject(data.error);
                            } else {
                                resolve(data);
                            }
                        }
                    } catch (e) {
                        reject(e);
                    }
                }
            };
            xmlhttp.ontimeout = function (e) {
                reject(e);
            };
            xmlhttp.open('POST', url, true);
            xmlhttp.timeout = 1000;
            xmlhttp.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
            xmlhttp.send(params);
        });
    });
}

function cancelRemoteSubscription(identifier) {
    return new Promise((resolve, reject) => {
        let sync = browser.storage.sync.get(['user']);
        sync.then((res) => {
            if (res.user == null) {
                reject("No user data");
                return;
            }
            let xmlhttp = new XMLHttpRequest();
            let url = "https://pury.fi/site/wp-json/remote_control/unsubscribe";
            let params = 'username=' + encodeURIComponent(res.user.username) + '&password=' + encodeURIComponent(res.user.password);
            params += '&identifier=' + encodeURIComponent(identifier);
            xmlhttp.onreadystatechange = function () {
                if (this.readyState == 4 && this.status == 200) {
                    try {
                        let data = JSON.parse(this.responseText);
                        if (data) {
                            if (data.hasOwnProperty('error')) {
                                reject(data.error);
                            } else {
                                resolve(data);
                            }
                        }
                    } catch (e) {
                        reject(e);
                    }
                }
            };
            xmlhttp.ontimeout = function (e) {
                reject(e);
            };
            xmlhttp.open('POST', url, true);
            xmlhttp.timeout = 1000;
            xmlhttp.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
            xmlhttp.send(params);
        });
    });
}

function cancelRemoteUserSubscription(identifier, user_id) {
    return new Promise((resolve, reject) => {
        let sync = browser.storage.sync.get(['user']);
        sync.then((res) => {
            if (res.user == null) {
                reject("No user data");
                return;
            }
            let xmlhttp = new XMLHttpRequest();
            let url = "https://pury.fi/site/wp-json/remote_control/unsubscribe_user";
            let params = 'username=' + encodeURIComponent(res.user.username) + '&password=' + encodeURIComponent(res.user.password);
            params += '&identifier=' + encodeURIComponent(identifier);
            params += '&user_id=' + encodeURIComponent(user_id);
            xmlhttp.onreadystatechange = function () {
                if (this.readyState == 4 && this.status == 200) {
                    try {
                        let data = JSON.parse(this.responseText);
                        if (data) {
                            if (data.hasOwnProperty('error')) {
                                reject(data.error);
                            } else {
                                resolve(data);
                            }
                        }
                    } catch (e) {
                        reject(e);
                    }
                }
            };
            xmlhttp.ontimeout = function (e) {
                reject(e);
            };
            xmlhttp.open('POST', url, true);
            xmlhttp.timeout = 1000;
            xmlhttp.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
            xmlhttp.send(params);
        });
    });
}

$("#reload-remote-settings").click(function() {
    loadRemoteSettings();
});

function loadRemoteSettings(){
        let sync = browser.storage.sync.get(['remote_configuration', 'lock_configuration', 'user']);
        sync.then((res) => {
            if(res.remote_configuration.remote_token_identifier != null){
                let remote_identifier = res.remote_configuration.remote_token_identifier;
                checkRemoteSettings(res.user, remote_identifier).then((data) => {
                    let last_updated = new Date(data.last_updated).getTime();
                    let last_checked = res.remote_configuration.last_checked? res.remote_configuration.last_checked : 0;
                    if(data.identifier && (last_updated > last_checked ||
                        window.confirm( "Your remote settings seem to be up to date.\n" +
                                        "Last sync: "+new Date(res.remote_configuration.last_updated).toLocaleString()+"\n" +
                                        "However, depending on your configuration, local changes may have overwritten the remote settings.\n" +
                                        "Do you wish to re-sync?")
                    )){
                        res.remote_configuration.last_updated = last_checked;
                        if(!res.remote_configuration.last_updated){
                            res.remote_configuration.last_updated = Date.now();
                        }
                        // settings might have been changed
                        cloud_import_settings(data.identifier, true, true);
                    }else if(data.identifier === null){
                        let remote_error_fix = $("#remote-error-fix");
                        let remote_error_clear = $("#remote-error-clear");
                        remote_error_fix.hide();
                        remote_error_clear.hide();
                        $("#remote-error-name").html("No Remote Settings Deposited");
                        $("#remote-error").show();
                    }
                    last_checked = Date.now();
                    res.remote_configuration.last_checked = last_checked;
                    browser.storage.sync.set({
                        remote_configuration: res.remote_configuration
                    }).then(() => {
                        updateOptionsContentScript();
                     });
                }).catch(e => {
                    if(e === 'INVALIDATED'){
                        if(window.confirm("Your Remote Subscription has ended!\n" +
                                          "You may clear your Subscription now and subscribe to another Remote Token.")){
                            unsubscribeToRemoteControl(res);
                        }
                    }else{
                        console.log(e);
                    }
                });
            }
        });
}

function checkRemoteSettings(user, remote_identifier){
    return new Promise((resolve, reject) => {
        if(user == null){
            return;
        }
        let xmlhttp = new XMLHttpRequest();
        let url = "https://pury.fi/site/wp-json/remote_control/settings_info";
        let params = 'username=' + encodeURIComponent(user.username) + '&password=' + encodeURIComponent(user.password);
        params += '&identifier=' + encodeURIComponent(remote_identifier);
        xmlhttp.onreadystatechange = function () {
            if (this.readyState == 4 && this.status == 200) {
                try {
                    let data = JSON.parse(this.responseText);
                    if (data) {
                        if (data.hasOwnProperty('error')) {
                            reject(data.error);
                            return;
                        } else {
                            resolve(data);
                        }
                    }
                } catch (e) {
                    reject(e);
                    return;
                }
            }
        };
        xmlhttp.ontimeout = function (e) {
            reject(e);
            return;
        };
        xmlhttp.open('POST', url, true);
        xmlhttp.timeout = 1000;
        xmlhttp.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
        xmlhttp.send(params);
    });
}

function checkRemoteSubscription(remote_configuration , user){
    //Checks if token and membership exist
    return new Promise((resolve, reject) => {
            if (user == null) {
                reject("No user data");
                return;
            }
            let xmlhttp = new XMLHttpRequest();
            let url = "https://pury.fi/site/wp-json/remote_control/subscription_info";
            let params = 'username=' + encodeURIComponent(user.username) + '&password=' + encodeURIComponent(user.password);
            params += '&identifier=' + encodeURIComponent(remote_configuration.remote_token_identifier);
            xmlhttp.onreadystatechange = function () {
                if (this.readyState == 4 && this.status == 200) {
                    try {
                        let data = JSON.parse(this.responseText);
                        if (data) {
                            if (data.hasOwnProperty('error')) {
                                reject(data.error);
                                return;
                            } else {
                                // todo: check version and generate error if mismatch
                                resolve(data);
                            }
                        }
                    } catch (e) {
                        reject(e);
                        return;
                    }
                }
            };
            xmlhttp.ontimeout = function (e) {
                reject(e);
                return;
            };
            xmlhttp.open('POST', url, true);
            xmlhttp.timeout = 1000;
            xmlhttp.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
            xmlhttp.send(params);
        });
}

function remoteCheckOnData(res, data){
    if(data.identifier === null){
        let remote_error_fix = $("#remote-error-fix");
        let remote_error_clear = $("#remote-error-clear");
        remote_error_fix.hide();
        remote_error_clear.hide();
        $("#remote-error-name").html("No Remote Settings Deposited");
        $("#remote-error").show();
    }else if(res.remote_configuration.remote_token_identifier !== data.remote_token){
        if(res.remote_configuration.remote_token_identifier === null){
            let remote_identifier = data.remote_token;
            $("#remote-error-name").html("Missing local Data:");
            $("#remote-error-info").html("puryfi-remote:"+ remote_identifier);
            let remote_error_fix = $("#remote-error-fix");
            let remote_error_clear = $("#remote-error-clear");
            remote_error_fix.unbind('click');
            remote_error_clear.unbind('click');
            remote_error_fix.show();
            remote_error_clear.show();
            remote_error_fix.html('Subscribe');
            remote_error_clear.html('Clear Cloud Data');
            remote_error_fix.click(function() {
                let lock = $('#remote-lock').is(':checked');
                let send_statistic = $('#remote-statistic').is(':checked');
                let check_period = Math.min(10080, Math.max(1,$('#remote-check-periodic-minutes').val()))*60000;

                let t = remoteConfirmText(lock, send_statistic);
                if(window.confirm("Please confirm subscription to: "+remote_identifier+"\n"+t)) {
                    subscribeToRemoteControl(res, remote_identifier, lock, send_statistic, check_period);
                }
            });
            remote_error_clear.click(function() {
                cancelRemoteSubscription(data.remote_token).then(() => {
                    resetRemoteControl(res).then(() => {
                        $("#menu-entry-settings").click();
                    });
                });
            });
        }else{
            let remote_identifier = data.remote_token;
            $("#remote-error-title").html("Data Missmatch:");
            $("#remote-error-info").html("<b>Data Missmatch:</b><br>" +
                "Local puryfi-remote:"+ res.remote_configuration.remote_token_identifier + "<br>" +
                "<div class='puryfi-settings-space'><b>Cloud puryfi-remote:</b><br>" + remote_identifier + "</div>"
            );
            let remote_error_fix = $("#remote-error-fix");
            let remote_error_clear = $("#remote-error-clear");
            remote_error_fix.html('Subscribe Local');
            remote_error_clear.html('Subscribe Cloud');
            remote_error_fix.unbind('click');
            remote_error_clear.unbind('click');
            remote_error_fix.show();
            remote_error_clear.show();
            remote_error_fix.click(function() {
                let lock = $('#remote-lock').is(':checked');
                let send_statistic = $('#remote-statistic').is(':checked');
                let check_period = Math.min(10080, Math.max(1,$('#remote-check-periodic-minutes').val()))*60000;
                let t = remoteConfirmText(lock, send_statistic);
                if(window.confirm("Please confirm subscription to: "+res.remote_configuration.remote_token_identifier+"\n"+t)) {
                    subscribeToRemoteControl(res, res.remote_configuration.remote_token_identifier, lock, send_statistic, check_period);
                }
            });
            remote_error_clear.click(function() {
                let lock = $('#remote-lock').is(':checked');
                let send_statistic = $('#remote-statistic').is(':checked');
                let check_period = Math.min(10080, Math.max(1,$('#remote-check-periodic-minutes').val()))*60000;
                let t = remoteConfirmText(lock, send_statistic);
                if(window.confirm("Please confirm subscription to: "+remote_identifier+"\n"+t)) {
                    subscribeToRemoteControl(res, remote_identifier, lock, send_statistic, check_period);
                }
            });
        }
        $("#remote-error").show();
    }
}

function remoteConfirmText(lock, send_statistic){
    let t = "Attention:\n" +
            "• Your current settings will be overwritten!\n";
    if(lock){
        t += "• Your remote subscription will be LOCKED!\n";
    }
    if(send_statistic){
        t += "• You will send your statistic data!\n";
    }
    return t;
}

function remoteCheckOnError(remote_configuration, error){
    if(remote_configuration.remote_token_identifier != null){
        if(error === 'NO_DATA'){
            $("#remote-error-name").html("Missing Cloud Data!");
            $("#remote-error").show();
            let remote_error_fix = $("#remote-error-fix");
            let remote_error_clear = $("#remote-error-clear");
            remote_error_fix.unbind('click');
            remote_error_clear.unbind('click');
            remote_error_fix.html('Synchronize Cloud');
            remote_error_clear.hide();
            remote_error_fix.show();
            remote_error_fix.click(function() {
                registerRemoteSubscription(remote_configuration.remote_token_identifier).then((data) => {
                    updateRemoteSubscription().then(()=>{}).catch((e) => {console.log(e)});
                }).catch((e) => {console.log(e)});
            });
        }else if(error === 'INVALIDATED'){
            $("#remote-error-name").html("Your subscription has ended!");
			$("#remote-timestamps").hide();
            $("#remote-error").show();
            let remote_error_fix = $("#remote-error-fix");
            let remote_error_clear = $("#remote-error-clear");
            remote_error_fix.unbind('click');
            remote_error_clear.unbind('click');
            remote_error_fix.html('Accept');
            remote_error_clear.hide();
            remote_error_fix.show();
            remote_error_fix.click(function() {
                let sync = browser.storage.sync.get(['remote_configuration', 'lock_configuration', 'user']);
                sync.then((res) => {
                    unsubscribeToRemoteControl(res, true);
                });
            });
        }else{
            console.log(error);
        }
    }
    log(error);
}

function getRemoteSubscriberData(remote_identifier, user){
    return new Promise((resolve, reject) => {
        if (user == null) {
            reject("No user data");
            return;
        }
        let xmlhttp = new XMLHttpRequest();
        let url = "https://pury.fi/site/wp-json/remote_control/subscriber_info";
        let params = 'username=' + encodeURIComponent(user.username) + '&password=' + encodeURIComponent(user.password);
        params += '&identifier=' + encodeURIComponent(remote_identifier);
        xmlhttp.onreadystatechange = function () {
            if (this.readyState == 4 && this.status == 200) {
                try {
                    let data = JSON.parse(this.responseText);
                    if (data) {
                        if (data.hasOwnProperty('error')) {
                            reject(data.error);
                            return;
                        } else {
                            resolve(data);
                        }
                    }
                } catch (e) {
                    reject(e);
                    return;
                }
            }
        };
        xmlhttp.ontimeout = function (e) {
            reject(e);
            return;
        };
        xmlhttp.open('POST', url, true);
        xmlhttp.timeout = 1000;
        xmlhttp.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
        xmlhttp.send(params);
    });
}

let checkRemoteSubscriptionTimestamp = performance.now();

$("#menu-entry-settings").click(function(e) {
    if (e.originalEvent !== undefined) {
        if (performance.now() - checkRemoteSubscriptionTimestamp < 2000) {
            return;
        }
    }
    let sync = browser.storage.sync.get(['remote_configuration', 'lock_configuration', 'user']);
    sync.then((res) => {
        if (res.user != null) {
            checkRemoteSubscription(res.remote_configuration, res.user).then((data) => {
                remoteCheckOnData(res, data);
            }).catch((e) => {
                remoteCheckOnError(res.remote_configuration, e);
            }).finally(() => {
                checkRemoteSubscriptionTimestamp = performance.now();
            });
        }
    });
});


$("#remote-settings-subscribe").click(function() {
    let remote_identifier =  $('#remote-settings-identifier').val();
    if(!remote_identifier){
        return;
    }
    let sync = browser.storage.sync.get(['remote_configuration','lock_configuration']);
    sync.then((res) => {
        if((res.remote_configuration.remote_token_identifier === null || !res.remote_configuration.lock) && !res.lock_configuration.enabled) {
            remote_identifier = remote_identifier.replace('puryfi-remote:', '');
            let lock = $('#remote-lock').is(':checked');
            let send_statistic = $('#remote-statistic').is(':checked');
            let check_period = Math.min(10080, Math.max(1,$('#remote-check-periodic-minutes').val()))*60000;
            let t = remoteConfirmText(lock, send_statistic);
            if(window.confirm("Please confirm subscription to: "+remote_identifier+"\n"+t)){
                subscribeToRemoteControl(res, remote_identifier, lock, send_statistic, check_period);
            }
        }else if(res.remote_configuration.lock || res.lock_configuration.enabled){
            window.alert("Subscription is locked");
        }
    });
});

$("#remote-settings-unsubscribe").click(function() {
    let sync = browser.storage.sync.get(['remote_configuration', 'lock_configuration', 'user']);
    sync.then((res) => {
        if (!res.remote_configuration.lock) {
            unsubscribeToRemoteControl(res);
        } else if (res.remote_configuration.lock) {
            window.alert("Unsubscribing is locked");
        }
    });
});


$('#remote-stored-tokens').on('click', '.remote-token-copy', function(){
    let identifier = $(this).attr('remote_identifier');
    navigator.clipboard.writeText('puryfi-remote:'+identifier);
    let elem = $('#settings-cloud-copy-tooltip');
    elem.tooltip("open");
    elem.attr('tooltip', 'copied-message');
    elem.trigger('mouseover');
    setTimeout(function() {
        elem.tooltip("close");
        elem.removeAttr('tooltip');
    }, 1000);
});

$("#refresh-remote-settings").click(function() {
    updateRemoteTokenDisplay();
});

$('#remote-stored-tokens').on('click', '.remote-token-edit-settings', function(){
    let identifier = $(this).attr('remote_identifier');
    $('input[remote_identifier="'+identifier+'"]').prop("disabled", false);
    $('button[remote_identifier="'+identifier+'"]').show();
    $(this).hide();
});

$('#remote-stored-tokens').on('click', '.remote-token-save-settings', function(){
    let token_identifier = $(this).attr('remote_identifier');
    token_identifier = token_identifier.replace('puryfi-remote:', '');
    let cloud_identifier = $('input[remote_identifier="'+token_identifier+'"]').val();
    cloud_identifier = cloud_identifier.replace('puryfi-settings:', '');
    bindCloudSettingsToToken(token_identifier, cloud_identifier);
});

$('#remote-stored-tokens').on('click', '.remote-user-info', function(){
    let token_identifier = $(this).attr('remote_identifier');
    let sync = browser.storage.sync.get(['user']);
    sync.then((res) => {
        showRemoteUsersModal(token_identifier, res.user);
    });
});

function showRemoteUsersModal(token_identifier, user){
    getRemoteSubscriberData(token_identifier, user).then(data => {
        let tbody = $('#remote-users-modal-table').find('tbody');
        tbody.empty();
        data.forEach(function (user_entry, index) {
            tbody.prepend($('<tr>')
                .append($('<td>').append(user_entry.name))
                .append($('<td>').append(new Date(user_entry.timestamp).toLocaleString()))
                .append($('<td>').append(
                        $('<button remote_identifier="'+token_identifier+'" user_identifier="'+user_entry.uid+'" class="icon-btn remote-subscriber-info" style="color: gray;width: 22px;" disabled>')
                            .append($('<i class="fa fa-info"></i>'))
                    )
                ).append($('<td>').append(
                        $('<button remote_identifier="'+token_identifier+'" user_name="'+user_entry.name+'" user_identifier="'+user_entry.uid+'" class="icon-btn remote-subscriber-kick" style="width: 22px;">')
                            .append($('<i class="fa fa-user-slash"></i>'))
                    )
                )
            );
        });
        showModalWindow('#remote-users-modal');
    }).catch((e) => {
        console.log(e);
    });
}

$('#remote-users-modal-close').click(function() {
    hideModalWindow('#remote-users-modal');
});

$('#remote-users-modal-table').on('click', '.remote-subscriber-kick', function(){
    let user_identifier = $(this).attr('user_identifier');
    let remote_identifier = $(this).attr('remote_identifier');
    let user_name = $(this).attr('user_name');
    if(confirm("Do you wish to unsubscribe '"+user_name+"'?")) {
        cancelRemoteUserSubscription(remote_identifier, user_identifier).then((data) => {
            let sync = browser.storage.sync.get(['user']);
            sync.then((res) => {
                updateRemoteTokenDisplay();
                showRemoteUsersModal(remote_identifier, res.user);
            });
        }).catch((e) => {console.log(e)});
    }
});

$('.remote-check-period').change(function() {
    let n = $(this).val();
    let sync = browser.storage.sync.get(['remote_configuration']);
    sync.then((res) => {
        if (!res.remote_configuration.lock) {
            res.remote_configuration.check_mode = parseInt(n);
            browser.storage.sync.set({
                remote_configuration: res.remote_configuration
            }).then(() => {
                updateOptionsContentScript();
            });
        }
    });
});

$('#remote-allow-old-settings').change(function() {
    let checked = $(this).is(':checked');
    let sync = browser.storage.sync.get(['remote_configuration']);
    sync.then((res) => {
        if (!res.remote_configuration.lock) {
            res.remote_configuration.allow_older_settings = checked;
            browser.storage.sync.set({
                remote_configuration: res.remote_configuration
            }).then(() => {
                updateOptionsContentScript();
            });
        }
    });
});

document.addEventListener('dragstart', function(event) {
    let draggedElement = event.target;
    if($(draggedElement).hasClass('cloud-settings-draggable')){
        let elem = $("input[remote_identifier]");
        if(!elem.hasClass("class-name")){
            elem.addClass("cloud-settings-drop-point");
            $(draggedElement).addClass("cloud-settings-dragged");
        }
    }
});

document.addEventListener('dragend', function(event) {
    let draggedElement = event.target;
    if($(draggedElement).hasClass('cloud-settings-draggable')){
        let elem = $("input[remote_identifier]");
        elem.removeClass("cloud-settings-drop-point");
        $(draggedElement).removeClass("cloud-settings-dragged");
        let x = event.clientX;
        let y = event.clientY;
        let droppedElement = document.elementFromPoint(x, y);
        if($(droppedElement).attr('remote_identifier')){
            let cloud_identifier = $(draggedElement).html();
            cloud_identifier = cloud_identifier.replace('puryfi-settings:', '');
            if(window.confirm("Do you wish to upload puryfi-settings:"+cloud_identifier+" ?")){
                let token_identifier = $(droppedElement).attr('remote_identifier');
                token_identifier = token_identifier.replace('puryfi-remote:', '');
                bindCloudSettingsToToken(token_identifier, cloud_identifier);
            }
        }
    }
});

