

function updateQuickSettings() {
    $('#settings-select option').remove();
    let full_menu = $('#login_form').length;
    let conf = browser.storage.local.get(['settings']);
    conf.then((res) => {
        let full_menu = $('#login_form').length;
        if(res.settings && Object.keys(res.settings).length >= 1 && !full_menu) {
            if(!$('#quick-setting-container').hasClass('locked')){
                $('#quick-setting-container').show();
            }
            $('#settings-select').append($('<option>', {
                value: '',
                text: '',
            }));
        }

        $('#settings-select').append($('<option>', {
            value: 'default',
            text: 'Default',
        }));

        if(res.settings){
            for (const setting_name in res.settings) {
                $('#settings-select').append($('<option>', {
                    value: setting_name.toLowerCase(),
                    text: setting_name,
                }));
            }
        }
    });
}

function loadSettings(setting_name){
    if(setting_name.toLowerCase() === ''){
        return;
    }
    let full_menu = $('#login_form').length;
    if(setting_name.toLowerCase() === 'default'){
        let skip_settings_reset = skip_settings.slice(0);
        skip_settings_reset.push('lock_configuration');
        resetOptions(skip_settings_reset).then(function (){
                restoreOptions();
        });
        return;
    }
    let s = browser.storage.local.get(['settings']);
    s.then((set) => {
        if (setting_name) {
            let keys = Object.keys(set.settings);
            for (const key of keys) {
                if(setting_name.toLowerCase() === key.toLowerCase()) {
                    let setting = {};
                    for (const property in set.settings[key]) {
                        if (!skip_settings.includes(property) && property != 'lock_configuration') {
                            setting[property] = set.settings[key][property];
                        }
                    }
                    browser.storage.sync.set(setting).then(function () {
                        if (full_menu) {
                            $('#settings-name').val("");
                            $('#settings_info_box').html("Setting <b>" + setting_name + "</b> loaded");
                        }
                        // Wait for the repaired config to be set in storage before restoring options
                        updateOptionsContentScript(false, true, set.settings[setting_name].version ?? null).then(() => restoreOptions());
                    });
                    break;
                }
            }
        }
    });
}

function encodeSettings(obj){
    let v = getVersion();
    const jsonString = JSON.stringify(obj);
    const encoder = new TextEncoder();
    const uint8array = encoder.encode(jsonString);
    let base64String = btoa(String.fromCharCode.apply(null, uint8array));
    base64String = crypt(v,base64String);
    return base64String;
}

function exportSettings(filename, obj){
    let v = getVersion();
    let base64String = encodeSettings(obj);
    const byteNumbers = new Array(base64String.length);
    for (let i = 0; i < base64String.length; i++) {
        byteNumbers[i] = base64String.charCodeAt(i);
    }
    const uint8Array = new Uint8Array(byteNumbers);
    const blob = new Blob([uint8Array], { type: "text/json" });
    const link = document.createElement("a");
    link.download = filename+'_'+new Date().valueOf()+'_'+v;
    link.href = window.URL.createObjectURL(blob);
    link.dataset.downloadurl = ["text/json", link.download, link.href].join(":");
    const evt = new MouseEvent("click", {
        view: window,
        bubbles: true,
        cancelable: true,
    });
    link.dispatchEvent(evt);
    link.remove()
}

function importSettingsFile(file){
    let reader = new FileReader();
    reader.onload = function(event) {
        const dec = event.target.result;
        const s = file.name.split("_").pop();
        if(s){
            importSettings(dec, s);
        }else{
            alert("Incompatible file");
        }
    }
    reader.readAsText(file);
}

function importSettings(data, version, silent = false, remote_import = false){
    let v = version.replace("B", "");
    let compare = compareVersions( browser.runtime.getManifest().version, v);
    if(compare < 0){
        alert("Error:\n" +
            "The settings you are trying to import was created by a newer version of the software.\n" +
            "Please update your software to the latest version or obtain a compatible settings file.");
        return;
    }else if(compare == 0 || confirm("Warning:\n" +
        "The settings file you are trying to upload was created by an older version of the software and may not be fully compatible with the current version.\n" +
        "Some settings may not work as intended.\n" +
        "Are you sure you want to proceed?")){
        const base64String = decrypt(version,data);
        try {
            const decodedString = atob(base64String);
            const settings_import = JSON.parse(decodedString);
            let c = browser.storage.sync.get(null);
            c.then((res) => {
                let setting = {};
                let lock = false;
                let keys = Object.keys(res);
                for (const property in settings_import) {
                    if(shape_settings.includes(property)){
                        settingsCheck(res.user, [settings_import[property]]);
                    }
                    if (keys.includes(property) && !skip_settings.includes(property)) {
                        setting[property] = settings_import[property];
                        if(property === 'lock_configuration' && settings_import[property].enabled){
                            lock = true;
                            if(remote_import){
                                setting[property].remote_lock = true;
                            }
                            if(setting[property].duration_timestamp !== null &&
                                parseInt(setting[property].timer_mode) !== 1 &&
                                res.lock_configuration.timer_mode !== setting[property].duration_timestamp
                                ){
                                setting[property].duration_timestamp = Date.now();
                            }
                        }
                    }
                }
                if(lock){
                    lock = "\nWARNING: These settings will lock your extension!\n";
                }else{
                    lock = "";
                }
                let affected_settings = Object.keys(setting).join(', ');
                if(silent || confirm("The following settings will be overwritten:\n"
                    +affected_settings +"\n" + lock +
                    "\nDo you want to continue?")){
                    browser.storage.sync.set(setting).then(function () {
                        $('#settings_info_box').html("Setting <b>" + name + "</b> imported");
                        // Wait for the repaired config to be set in storage before restoring options
                        updateOptionsContentScript(false, true, version).then(() => restoreOptions());
                    });
                }
            });
        }catch(e){
            alert("Could not read settings: Corrupted file");
        }
    }
}

$('#settings-delete').click(function(){
    let c = browser.storage.local.get(['settings']);
    c.then((res) => {
        let setting_name = $('#settings-select').find(":selected").val();
        if(setting_name.toLowerCase() === 'default'){
            window.alert("This setting can't be deleted!")
            return;
        }
        if(setting_name){
            let keys = Object.keys(res.settings);
            for (const key of keys) {
                if(setting_name.toLowerCase() === key.toLowerCase() && window.confirm("Do you really want to delete this setting: " + setting_name + " ?")) {
                    delete res.settings[key];
                    browser.storage.local.set({
                        settings: res.settings,
                    });
                    $('#settings-name').val("");
                    $('#settings_info_box').html("Setting <b>"+setting_name+"</b> deleted");
                    updateQuickSettings();
                    break;
                }
            }
        }
    });
});

$('#settings-load').click(function(){
    let setting_name = $('#settings-select').find(":selected").val();
    loadSettings(setting_name);
});

$('#settings-save').click(function(){
    let take = [];
    $('.puryfi-settings-toggle-grid input').each(function (i, elem){
        if($(elem).is(':checked')){
            take.push($(elem).attr("id"));
        }
    });
    let c = browser.storage.sync.get(null);
    c.then((res) => {
        let s = browser.storage.local.get(['settings']);
        s.then((set) => {
            let settings = set['settings'] ? set['settings'] : {};
            let setting_name = $('#settings-name').val() ? $('#settings-name').val() : 'settings-' + performance.now();
            if(setting_name.length > 20){
                window.alert("Setting name too long!")
                return;
            }
            if(setting_name.length < 1){
                window.alert("Setting name too short!")
                return;
            }
            if(setting_name.toLowerCase() === 'default'){
                window.alert("This setting name can't be used!")
                return;
            }
            let setting_exists = false;
            let settings_key = null;
            if(set.settings != null) {
                let keys = Object.keys(set.settings);
                for (const key of keys) {
                    if (setting_name.toLowerCase() === key.toLowerCase()) {
                        setting_exists = true;
                        settings_key = key;
                        break;
                    }
                }
            }
            if(settings_key != null) {
                if(window.confirm("A setting with that name already exists! Do you want to overwrite it?")) {
                    delete set.settings[settings_key];
                }else{
                   return;
                }
            }
            let setting = {};
            for (const property in res) {
                if (!skip_settings.includes(property) && take.includes(property)) {
                    setting[property] = res[property];
                }
            }
            setting.version = browser.runtime.getManifest().version;
            settings[setting_name] = setting;
            browser.storage.local.set({
                settings: settings,
            });
            $('#settings-name').val("");
            $('#settings_info_box').html("Setting <b>"+setting_name+"</b> added");

            updateQuickSettings();
        });
    });
});

$('#settings-export-current').click(function(){
    let setting = {};
    let c = browser.storage.sync.get(null);
    c.then((res) => {
        let take = [];
        $('.puryfi-settings-toggle-grid input').each(function (i, elem){
            if($(elem).is(':checked')){
                take.push($(elem).attr("id"));
            }
        });
        for (const property in res) {
            if (!skip_settings.includes(property) && take.includes(property)) {
                setting[property] = res[property];
            }
        }
        let setting_name = $('#settings-name').val() ? $('#settings-name').val() : 'settings';
        exportSettings(setting_name, setting);
    });
});

$('#settings-export-saved').click(function(){
    let setting_name = $('#settings-select').find(":selected").val();
    if(setting_name.toLowerCase() === ''){
        return;
    }
    if(setting_name.toLowerCase() === 'default'){
        alert("Not supported:\n" +
              "Please export default settings by loading it and then exporting the current settings!");
        return;
    }
    let s = browser.storage.local.get(['settings']);
    s.then((set) => {
        if (setting_name) {
            let keys = Object.keys(set.settings);
            for (const key of keys) {
                if(setting_name.toLowerCase() === key.toLowerCase()) {
                    let setting = {};
                    for (const property in set.settings[key]) {
                        if (!skip_settings.includes(property)) {
                            setting[property] = set.settings[key][property];
                        }
                    }
                    exportSettings(setting_name, setting);
                }
            }
        }
    });
});

$('#settings-import').on("change", function() {
    let files = $( this ).prop('files');
    if (files && files[0]) {
        importSettingsFile(files[0]);
    }
});

$('.puryfi-settings-toggle-header').on('click', function() {
    let $checkboxes = $(this).next('.puryfi-settings-toggle-grid').find('input');
    let all_checked = $checkboxes.toArray().every(checkbox => checkbox.checked);
    $checkboxes.prop("checked", !all_checked);
});

$('#settings-import-cloud').click(function(){
    let val = $('#settings-import-cloud-identifier').val();
    val = val.replace('puryfi-settings:','');
    if(val){
        cloud_import_settings(val);
    }
});

$('#settings-upload-current').click(function(){
    if(confirm("Do you want to upload your current settings?")) {
        $('.settings-upload').prop("disabled", true);
        let setting_name = $('#settings-name').val() ? $('#settings-name').val() : 'settings';
        let version = getVersion();
        let setting = {};
        let c = browser.storage.sync.get(null);
        c.then((res) => {
            let take = [];
            $('.puryfi-settings-toggle-grid input').each(function (i, elem) {
                if ($(elem).is(':checked')) {
                    take.push($(elem).attr("id"));
                }
            });
            for (const property in res) {
                if (!skip_settings.includes(property) && take.includes(property)) {
                    setting[property] = res[property];
                }
            }
            let configs = Object.keys(setting).join(',');
            let settings = encodeSettings(setting);
            cloud_upload_settings(setting_name, version, configs, settings);
        });
    }
});

$('#settings-upload-saved').click(function(){
    let setting_name = $('#settings-select').find(":selected").val();
    if(setting_name.toLowerCase() === ''){
        return;
    }
    if(setting_name.toLowerCase() === 'default'){
        alert("Not supported:\n" +
            "Please choose non-default settings!");
        return;
    }
    if(confirm("Do you want to upload your settings: '"+setting_name+"'?")) {
        $('.settings-upload-saved').prop("disabled", true);

        let version = getVersion();
        let s = browser.storage.local.get(['settings']);
        s.then((set) => {
            if (setting_name) {
                let keys = Object.keys(set.settings);
                for (const key of keys) {
                    if(setting_name.toLowerCase() === key.toLowerCase()) {
                        let setting = {};
                        for (const property in set.settings[key]) {
                            if (!skip_settings.includes(property)) {
                                setting[property] = set.settings[key][property];
                            }
                        }
                        let configs = Object.keys(setting).join(',');
                        let settings = encodeSettings(setting);
                        cloud_upload_settings(setting_name, version, configs, settings);
                    }
                }
            }
        });
    }
});

function cloud_upload_settings(name, version, configs, settings){
        let sync = browser.storage.sync.get(['user']);
        sync.then((res) => {
            if(res.user == null){
                alert("Please login to use this feature!");
                return;
            }
            let xmlhttp = new XMLHttpRequest();
            let url = "https://pury.fi/site/wp-json/cloud_settings/upload";
            let params = 'username=' + encodeURIComponent(res.user.username) + '&password=' + encodeURIComponent(res.user.password);
            params += '&name=' + encodeURIComponent(name);
            params += '&version=' + encodeURIComponent(version);
            params += '&configs=' + encodeURIComponent(configs);
            params += '&settings=' + encodeURIComponent(settings);
            xmlhttp.onreadystatechange = function () {
                if (this.readyState == 4 && this.status == 200) {
                    try {
                        let identifier = JSON.parse(this.responseText);
                        if (identifier) {
                            if(identifier.hasOwnProperty('error')){
                                alert(identifier.error);
                            }else{
                                $('#settings-cloud-copy-identifier').val('puryfi-settings:'+identifier);
                                showModalWindow('#settings_upload_modal');
                            }
                        }
                        reEnableCloudUpload();
                    } catch (e) {
                        log(e);
                        reEnableCloudUpload();
                    }
                }
            };
            xmlhttp.ontimeout = function (e) {
                log(e);
                reEnableCloudUpload();
            };
            xmlhttp.open('POST', url, true);
            xmlhttp.timeout = 1000;
            xmlhttp.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
            xmlhttp.send(params);
        });
}

function cloud_import_settings(identifier, silent = false, remote_import = false){
        let xmlhttp = new XMLHttpRequest();
        let url = "https://pury.fi/site/wp-json/cloud_settings/load";
        let params = 'identifier=' + encodeURIComponent(identifier);
        xmlhttp.onreadystatechange = function () {
            if (this.readyState == 4 && this.status == 200) {
                try {
                    let data = JSON.parse(this.responseText);
                    if (data) {
                        if(data.hasOwnProperty('error')){
                            alert(data.error);
                        }else{
                            importSettings(data['settings'], data['version'], silent, remote_import);
                        }
                    }
                } catch (e) {
                    log(e);
                }
            }
        };
        xmlhttp.ontimeout = function (e) {
            log(e);
        };
        xmlhttp.open('POST', url, true);
        xmlhttp.timeout = 1000;
        xmlhttp.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
        xmlhttp.send(params);
}

function cloud_delete_settings(identifier){
    let sync = browser.storage.sync.get(['user']);
    sync.then((res) => {
        if(res.user == null){
            alert("Please login to use this feature!");
            return;
        }
        let xmlhttp = new XMLHttpRequest();
        let url = "https://pury.fi/site/wp-json/cloud_settings/delete";
        let params = 'username=' + encodeURIComponent(res.user.username) + '&password=' + encodeURIComponent(res.user.password);
        params += '&identifier=' + encodeURIComponent(identifier);
        xmlhttp.onreadystatechange = function () {
            if (this.readyState == 4 && this.status == 200) {
                try {
                    let data = JSON.parse(this.responseText);
                    updateCloudSettingsOverview();
                } catch (e) {
                    updateCloudSettingsOverview();
                    log(e);
                }
            }
        };
        xmlhttp.ontimeout = function (e) {
            updateCloudSettingsOverview();
            log(e);
        };
        xmlhttp.open('POST', url, true);
        xmlhttp.timeout = 1000;
        xmlhttp.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
        xmlhttp.send(params);
    });
}

function reEnableCloudUpload(){
    setTimeout(()=> {
            $('.settings-upload').prop("disabled",false);
        },1000);
}

$('#settings_upload_modal_close').click(function() {
    hideModalWindow('#settings_upload_modal');
});

$('.settings-cloud-copy-identifier').click(function() {
    let cp = document.getElementById("settings-cloud-copy-identifier");
    cp.setSelectionRange(0, 99999);
    navigator.clipboard.writeText(cp.value);
    $('#settings-cloud-copy-icon-container').attr('tooltip', 'copied-message');
    $('#settings-cloud-copy-icon-container').tooltip("open");
    setTimeout(function() {
        $('#settings-cloud-copy-icon-container').tooltip("close");
        $('#settings-cloud-copy-icon-container').removeAttr('tooltip');
    }, 1000);
});

function updateCloudSettingsOverview(){
    $("#cloud-stored-settings").find('tbody').empty();
    let sync = browser.storage.sync.get(['user']);
    sync.then((res) => {
        if(res.user == null){
            return;
        }
        let xmlhttp = new XMLHttpRequest();
        let url = "https://pury.fi/site/wp-json/cloud_settings/info";
        let params = 'username=' + encodeURIComponent(res.user.username) + '&password=' + encodeURIComponent(res.user.password);
        xmlhttp.onreadystatechange = function () {
            if (this.readyState == 4 && this.status == 200) {
                try {
                    let data = JSON.parse(this.responseText);
                    if (data) {
                        if (data.hasOwnProperty('error')) {
                            alert(data.error);
                        } else {
                            data['settings'].forEach(function (settings_entry, index) {
                                $("#settings-storage-current").html(data.info.storage_current);
                                $("#settings-storage-max").html(data.info.storage_max);
                                $("#cloud-stored-settings").find('tbody')
                                    .append($('<tr>')
                                        .append($('<td style="display: flex;" draggable="true">')
                                            .append($('<div class="cloud-settings-draggable" draggable="true">')
                                                .text('puryfi-settings:'+settings_entry.identifier))
                                            .append($('<div style="margin-left: 5px;">' +
                                                '<i cloud_identifier="'+settings_entry.identifier+'" class="fas fa-copy cloud-settings-copy settings-cloud-copy-identifier"></i>' +
                                                '</div>'))
                                        ).append($('<td>')
                                            .text(settings_entry.name)
                                        ).append($('<td>')
                                            .text(settings_entry.version)
                                        ).append($('<td>')
                                            .text(settings_entry.downloads)
                                        ).append($('<td>')
                                            .append($('<i class="fas fa-info-circle cloud-config-tooltip"></i><div cloud_config_data="'+settings_entry.configs+'"></div>'))
                                        ).append($('<td>').append($('<button cloud_identifier="'+settings_entry.identifier+'" class="icon-btn cloud-settings-delete">')
                                            .append($('<i class="fa fa-trash"></i>'))))
                                    );
                            });
                        }
                    }
                    reEnableCloudUpload();
                } catch (e) {
                    log(e);
                    reEnableCloudUpload();
                }
            }
        };
        xmlhttp.ontimeout = function (e) {
            log(e);
            reEnableCloudUpload();
        };
        xmlhttp.open('POST', url, true);
        xmlhttp.timeout = 1000;
        xmlhttp.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
        xmlhttp.send(params);
    });
}

$("a[content='content-cloud']").click(function() {
    updateCloudSettingsOverview();
});
$("#refresh-cloud-settings").click(function() {
    updateCloudSettingsOverview();
});
$('#cloud-stored-settings').on('click', '.cloud-settings-delete', function(){
    let identifier = $(this).attr('cloud_identifier');
    if(confirm("Do you wish to delete cloud settings '"+identifier+"'?")) {
        identifier = identifier.replace('puryfi-settings:', '');
        cloud_delete_settings(identifier);
    }
});
$('#cloud-stored-settings').on('click', '.cloud-settings-copy', function(){
    let identifier = $(this).attr('cloud_identifier');
    navigator.clipboard.writeText('puryfi-settings:'+identifier);
    $('#settings-cloud-copy-tooltip').tooltip("open");
    $('#settings-cloud-copy-tooltip').attr('tooltip', 'copied-message');
    $('#settings-cloud-copy-tooltip').trigger('mouseover');
    setTimeout(function() {
        $('#settings-cloud-copy-tooltip').tooltip("close");
        $('#settings-cloud-copy-tooltip').removeAttr('tooltip');
    }, 1000);
});

let stp_displayed = null;
$('#cloud-stored-settings').on('mouseenter', '.cloud-config-tooltip', function(){
    if(stp_displayed != $(this)) {
        let data = $(this).parent().find('div').attr('cloud_config_data');
        data = data.split(',');
        let html = '<ul>';
        for(let i=0; i <data.length; i++) {
            html += '<li>' + data[i] + '</li>';
        }
        html += '</ul>';
        $('#settings-cloud-copy-tooltip').tooltip("open");
        $('#settings-cloud-copy-tooltip').attr('tooltip', 'cloud-config-data');
        $('#settings-cloud-copy-tooltip').trigger('mouseover');
        $('#settings-affected-configurations').html(html);

        setTimeout(function () {
            if(stp_displayed == $(this)) {
                $('#settings-cloud-copy-tooltip').tooltip("close");
                $('#settings-cloud-copy-tooltip').removeAttr('tooltip');
            }
        }, 1000);

    }
});

$('#cloud-stored-settings').on('mouseleave', '.cloud-config-tooltip', function(){
    $('#settings-cloud-copy-tooltip').tooltip({hide: false});
    $('#settings-cloud-copy-tooltip').tooltip("close");
    $('#settings-cloud-copy-tooltip').removeAttr('tooltip');
    $('#settings-cloud-copy-tooltip').tooltip({hide: true});
});

$('#settings_import_dropzone').on('dragover', function(e) {
    e.preventDefault();
})

$('#settings_import_dropzone').on('dragenter', function(e) {
    e.preventDefault();
})

$('#settings_import_dropzone').on('drop', function(e) {
    let files = e.originalEvent.dataTransfer?.files;
    if (files == null) return;
    e.preventDefault();
    $('#settings-import').prop('files', files);
    $('#settings-import').trigger('change');
})