function getSelectedPreset(config) {
    return config.presets[config.selected_preset_index];
}

/**********************************************************************************************************************
 * Censor Types
 **********************************************************************************************************************/
const MIXED_CENSOR_LAYERS_COUNT = 3;
const MIXED_LABEL_TYPE_ROWS_COUNT = 20;

class CensorTypePresetConfig {
  constructor(scale = 1.0, shape = 0, feathering= 0) {
      this.scale = scale;
      this.shape = shape;
      this.feathering = feathering;
  }
}

class CensorTypePreset {
    constructor(config, name = "Regular") {
        this.config = config;
        this.name = name;
    }
}
  
class CensorTypeConfig {
    constructor(_name, presets = [], selected_preset_index = 0,) {
        this._name = _name;
        this.presets = presets;
        this.selected_preset_index = selected_preset_index;
    }
}

class StickerCensorTypeConfig extends CensorTypeConfig {
    constructor(_name, cached, presets = [], selected_preset_index = 0,) {
        super(_name, presets, selected_preset_index);
        this.cached = cached
    }
}
  
class BarCensorTypePresetConfig extends CensorTypePresetConfig {
  constructor(shape = 0, rounding = 0, scale = 1, feathering = 0, border_width = 0, border_color = '#FFFFFF', type = 0, color = '#000000', opacity = 1) {
      super(scale, shape, feathering);
      this.rounding = rounding;
      this.border_width = border_width;
      this.border_color = border_color;
      this.type = type;
      this.color = color;
      this.opacity = opacity;
  }
}

class PixelCensorTypePresetConfig extends CensorTypePresetConfig {
  constructor(shape = 0, rounding = 0, scale = 1, feathering = 0, border_width = 0, border_color = '#FFFFFF', type = 0, strength = 20, spacing = 0, channel_colors = 256, grayscale = false, resolution_aware = true, detection_size_aware = true, accurate_sampling = true) {
      super(scale, shape, feathering);
      this.rounding = rounding;
      this.border_width = border_width;
      this.border_color = border_color;
      this.spacing = spacing;
      this.channel_colors = channel_colors;
      this.type = type;
      this.resolution_aware = resolution_aware;
      this.detection_size_aware = detection_size_aware;
      this.strength = strength;
      this.grayscale = grayscale;
      this.accurate_sampling = accurate_sampling;
  }
}

class BlurCensorTypePresetConfig extends CensorTypePresetConfig {
    constructor(shape = 0, rounding = 0, scale = 1, feathering = 0, border_width = 0, border_color = '#FFFFFF', strength = 20, grayscale = false, resolution_aware = true) {
        super(scale, shape, feathering);
        this.rounding = rounding;
        this.border_width = border_width;
        this.border_color = border_color;
        this.strength = strength;
        this.grayscale = grayscale;
        this.resolution_aware = resolution_aware;
    }
}

class TriangleCensorTypePresetConfig extends CensorTypePresetConfig {
  constructor(shape = 0, rounding = 0, scale = 1, feathering = 0, border_width = 0, border_color = '#FFFFFF', resolution_aware = false, vertex_count = 700, threshold = 50, accuracy = 0.7, blurring = 1, fill_color_mode = 0, fill_color = '#000000', stroke_color_mode = 0, stroke_color = '#000000', stroke_width = 0.5, gradients = false, gradient_stops = 4, line_join = 0, transparent_color = false) {
        super(scale, shape, feathering);
        this.rounding = rounding;
        this.border_width = border_width;
        this.border_color = border_color;
        this.accuracy = accuracy;
        this.blurring = blurring;
        this.threshold = threshold;
        this.resolution_aware = resolution_aware;
        this.vertex_count = vertex_count;
        this.fill_color_mode = fill_color_mode;
        this.fill_color = fill_color;
        this.stroke_color_mode = stroke_color_mode;
        this.stroke_color = stroke_color;
        this.stroke_width = stroke_width;
        this.gradients = gradients;
        this.gradient_stops = gradient_stops;
        this.line_join = line_join;
        this.transparent_color = transparent_color;
  }
}

class BoxCensorTypePresetConfig extends CensorTypePresetConfig {
    constructor(scale = 1, display = 0, fill = 1, fill_color = '#000000', border = 1, border_color = '#FFFFFF') {
        super(scale, 0, 0);
        this.display = display;
        this.fill = fill;
        this.fill_color = fill_color;
        this.border = border;
        this.border_color = border_color;
    }
}
  
class GlitchCensorTypePresetConfig extends CensorTypePresetConfig {
    constructor(type = 0, scale = 1, chromatic_aberration_color_intensity = 1,chromatic_aberration_shift_intensity = 1, chromatic_aberration_vertical_shift = true, chromatic_aberration_horizontal_shift = true, multiple_panels_amount = 4, multiple_panels_min_size = 0.85, multiple_panels_max_size = 1.45, multiple_panels_scatter = 0.25, multiple_panels_split_chance = 0.2, multiple_panels_border = false, multiple_panels_gradient = [ {color: "#0000007F", offset: 0 }, {color: "#0000007F", offset: 0 }, {color: "#0000007F", offset: 0 }, {color: "#0000007F", offset: 0 } ]) {
        super(scale, 0, 0);
        this.type = type;
        this.chromatic_aberration_shift_intensity = chromatic_aberration_shift_intensity;
        this.chromatic_aberration_color_intensity = chromatic_aberration_color_intensity;
        this.chromatic_aberration_vertical_shift = chromatic_aberration_vertical_shift;
        this.chromatic_aberration_horizontal_shift = chromatic_aberration_horizontal_shift;
        this.multiple_panels_amount = multiple_panels_amount;
        this.multiple_panels_min_size = multiple_panels_min_size;
        this.multiple_panels_max_size = multiple_panels_max_size;
        this.multiple_panels_scatter = multiple_panels_scatter;
        this.multiple_panels_split_chance = multiple_panels_split_chance;
        this.multiple_panels_gradient = multiple_panels_gradient.map(item => ({ color: item.color, offset: item.offset }));
        this.multiple_panels_border = multiple_panels_border;
    }
}
  
class StickerCensorTypePresetConfig extends CensorTypePresetConfig {
  constructor(scale = 1, draw_mode = 0, sample_single_source = true,
              enabled_sources = ["emoji smilies", "emoji hearts", "emoji hands", "emoji other", "emoji signs","fruits"]) {
      super(scale, 0, 0);
      this.draw_mode = draw_mode;
      this.sample_single_source = sample_single_source;
      this.enabled_sources = enabled_sources;
  }
}

class SobelCensorTypePresetConfig extends CensorTypePresetConfig {
  constructor(shape = 0, rounding = 0, scale = 1, feathering = 0, border_width = 0, border_color = '#FFFFFF', inverted_color = false) {
      super(scale, shape, feathering);
      this.rounding = rounding;
      this.border_width = border_width;
      this.border_color = border_color;
      this.inverted_color = inverted_color;
  }
}

class SplatterCensorTypePresetConfig extends CensorTypePresetConfig {
    constructor(scale = 1, color_mode = 0, opacity = 1, color_pool = [ {color: "#000000", weight: 0 }, {color: "#000000", weight: 0 }, {color: "#000000", weight: 0 }, {color: "#000000", weight: 0 }, {color: "#000000", weight: 0 }, {color: "#000000", weight: 0 } ], centering = 4, size = 1, amount = 5, sub_size = 1, sub_amount = 10, recursions = 2) {
        super(scale, 0, 0);
        this.color_mode = color_mode;
        this.opacity = opacity;
        this.color_pool = color_pool.map(item => ({ color: item.color, weight: item.weight }));;
        this.centering = centering; 
        this.size = size;
        this.amount = amount;
        this.sub_size = sub_size;
        this.sub_amount = sub_amount;
        this.recursions = recursions;
  }
}

class MixedCensorTypePresetConfig extends CensorTypePresetConfig {
    constructor(censor_types = { 
    0:[],1:[],2:[],3:[],4:[],5:[],6:[],7:[],8:[],9:[],10:[],11:[],12:[],13:[],14:[],15:[],16:[],17:[],18:[],19:[]
}, censor_preset_indexes = {
    0:[],1:[],2:[],3:[],4:[],5:[],6:[],7:[],8:[],9:[],10:[],11:[],12:[],13:[],14:[],15:[],16:[],17:[],18:[],19:[]
}, reverse_censor_types = [], reverse_censor_preset_indexes = []) {
        super(null, null, null);
        this.censor_types = censor_types;
        this.censor_preset_indexes = censor_preset_indexes;
        this.reverse_censor_types = reverse_censor_types;
        this.reverse_censor_preset_indexes = reverse_censor_preset_indexes;
    }
}

class RandomCensorTypePresetConfig extends CensorTypePresetConfig {
    constructor(mode = 0, censor_weights = {
        0: 0,
        1: 1,
        2: 1,
        3: 1,
        4: 1,
        5: 0,
        6: 1,
        7: 1,
        8: 1,
        9: 1,
        10: 0,
    }) {
        super(null, null, null);
        this.mode = mode;
        this.censor_weights = censor_weights;
    }
}

/**********************************************************************************************************************
 * File Types
 **********************************************************************************************************************/
class FileTypeConfiguration {
    get file_output_type() {
        return this._file_output_type;
    }

    set file_output_type(value) {
        this._file_output_type = value;
    }

    constructor(name, filesize_min = 1000, filesize_max= 0, width_min= 50, width_max= 0, height_min= 50, height_max= 0, file_output_type) {
        this._name = name;
        this._filesize_min = filesize_min;
        this._filesize_max = filesize_max;
        this._width_min = width_min;
        this._width_max = width_max;
        this._height_min = height_min;
        this._height_max = height_max;
        this._file_output_type = file_output_type;
    }

    get name() {
        return this._name;
    }

    set name(value) {
        this._name = value;
    }

    get filesize_min() {
        return this._filesize_min;
    }

    set filesize_min(value) {
        this._filesize_min = value;
    }

    get filesize_max() {
        return this._filesize_max;
    }

    set filesize_max(value) {
        this._filesize_max = value;
    }

    get width_min() {
        return this._width_min;
    }

    set width_min(value) {
        this._width_min = value;
    }

    get width_max() {
        return this._width_max;
    }

    set width_max(value) {
        this._width_max = value;
    }

    get height_min() {
        return this._height_min;
    }

    set height_min(value) {
        this._height_min = value;
    }

    get height_max() {
        return this._height_max;
    }

    set height_max(value) {
        this._height_max = value;
    }
}

class PNGTypeConfiguration extends FileTypeConfiguration {
    constructor(name,
                filesize_min = 5000, filesize_max= 0,
                width_min= 50, width_max= 0, height_min= 50, height_max= 0,
                file_output_type = "image/png") {
        super(name, filesize_min, filesize_max, width_min, width_max, height_min, height_max, file_output_type);
    }
}

class JPGTypeConfiguration extends FileTypeConfiguration {
    constructor(name,
                filesize_min = 5000, filesize_max= 0,
                width_min= 50, width_max= 0, height_min= 50, height_max= 0,
                file_output_type = "image/jpeg") {
        super(name,filesize_min,filesize_max,width_min, width_max, height_min, height_max, file_output_type);
    }
}

class BMPTypeConfiguration extends FileTypeConfiguration {
    constructor(name,
                filesize_min = 5000, filesize_max= 0,
                width_min= 50, width_max= 0, height_min= 50, height_max= 0,
                file_output_type = "image/bmp") {
        super(name,filesize_min,filesize_max,width_min, width_max, height_min, height_max, file_output_type);
    }
}

class WEBPTypeConfiguration extends FileTypeConfiguration {
    constructor(name,
                filesize_min = 5000, filesize_max= 0,
                width_min= 50, width_max= 0, height_min= 50, height_max= 0,
                file_output_type = "image/webp") {
        super(name,filesize_min,filesize_max,width_min, width_max, height_min, height_max, file_output_type);
    }
}

class AVIFTypeConfiguration extends FileTypeConfiguration {
    constructor(name,
                filesize_min = 5000, filesize_max= 0,
                width_min= 50, width_max= 0, height_min= 50, height_max= 0,
                file_output_type = "image/png") {
        super(name,filesize_min,filesize_max,width_min, width_max, height_min, height_max, file_output_type);
    }
}

class GIFTypeConfiguration extends FileTypeConfiguration {
    constructor(name, memory_method, thumbnails, thumbnail_fallback, frame_count_max = 0,
                filesize_min = 10000, filesize_max = 100000000,
                width_min= 0, width_max= 0, height_min= 0, height_max= 0, ai_frame_skipping = 1, detection_extending = 4, detection_similarity_threshold = 0.3
                ) {
        super(name,filesize_min,filesize_max,width_min, width_max, height_min, height_max,"image/gif");
        this._frame_count_max = frame_count_max;
        this._memory_method = memory_method;
        this._thumbnails = thumbnails;
        this._thumbnail_fallback = thumbnail_fallback;
        this._ai_frame_skipping = ai_frame_skipping;
        this._detection_extending = detection_extending;
        this._detection_similarity_threshold = detection_similarity_threshold;
    }

    get memory_method() {
        return this._memory_method;
    }

    set memory_method(value) {
        this._memory_method = value;
    }

    get thumbnails() {
        return this._thumbnails;
    }

    set thumbnails(value) {
        this._thumbnails = value;
    }


    get frame_count_max() {
        return this._frame_count_max;
    }

    set frame_count_max(value) {
        this._frame_count_max = value;
    }

    get thumbnail_fallback() {
        return this._thumbnail_fallback;
    }

    set thumbnail_fallback(value) {
        this._thumbnail_fallback = value;
    }
}

/**********************************************************************************************************************
 * White & Blacklist
 **********************************************************************************************************************/

class WhiteBlackListConfiguration {
    constructor(mode = 0, white_list = [], black_list = []) {
        this.mode = mode;
        this.white_list = white_list;
        this.black_list = black_list;
    }
}

/**********************************************************************************************************************
 * ReverseMode
 **********************************************************************************************************************/

class ReverseModeConfiguration {
    constructor(enabled, mode) {
        this.enabled = enabled;
        this.mode = mode;
    }
}

/**********************************************************************************************************************
 * Clustering
 **********************************************************************************************************************/

class ClusteringConfiguration {
    constructor(enabled, mode) {
        this.enabled = enabled;
        this.mode = mode;
    }
}

/**********************************************************************************************************************
 * Word Wall
 **********************************************************************************************************************/

class WordWallConfiguration {
    constructor(enabled, word_pool, draw_mode, font, size, horizontal_spacing, vertical_spacing, offset, angle, color_mode, color) {
        this.enabled = enabled;
        this.word_pool = word_pool;
        this.draw_mode = draw_mode;
        this.size = size;
        this.horizontal_spacing = horizontal_spacing;
        this.vertical_spacing = vertical_spacing;
        this.offset = offset;
        this.angle = angle;
        this.color_mode = color_mode;
        this.color = color;
        this.font = font;
    }
}
/**********************************************************************************************************************
 * Remote Configuration
 **********************************************************************************************************************/

class RemoteConfiguration {
    constructor(lock = false, send_statistics_data = false, remote_token_identifier = null, last_updated = null,
                last_checked = null, check_mode = 0, check_period = 600000, allow_older_settings = false) {
        this.remote_token_identifier = remote_token_identifier;
        this.last_updated = last_updated;
        this.last_checked = last_checked;
        this.lock = lock;
        this.send_statistics_data = send_statistics_data;
        this.check_mode = check_mode;
        this.check_period = check_period;
        this.allow_older_settings = allow_older_settings;
    }
}

/**********************************************************************************************************************
 * Only Once Mode
 **********************************************************************************************************************/

class OnlyOnceModeConfiguration {
    constructor(enabled = false, mode = 0, precision = 62, width_min = 50, height_min = 50,
    message = 'This image has been permanently blocked!', date_time_format = 'local', display_classes = false) {
        this.enabled = enabled;
        this.mode = mode;
        this.mode_configuration = {
            0: { // Full Block

            },
            1: { // See Trough
                transparency: 0.02,
            },
            2: { // Borderless
                distance: 0.5,
                radius: 0,
            },
            3: { // Thumbnail
                blur: 0.0,
            },
            4: { // Grid
                strength: 0.3,
                color_1: "#000000",
                color_2: "#FFFFFF8C",
            },
            5: { // Strong Blur
                blur: 0.05,
                grayscale: false,
            },
            6: { // Box Tease
                allow_faces: false,
            },
        };
        this.precision = precision;
        this.message = message;
        this.date_time_format = date_time_format;
        this.display_classes = display_classes;
        this.width_min = width_min;
        this.height_min = height_min;
        this.trigger = [0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19];
        this.timer = false;
        this.timer_autorefresh = true;
        this.timer_min_duration = 1000*10;
        this.timer_max_duration = 1000*60*5;
        this.timer_animation = 1;
    }
}

class OnlyOnceModeTree {
    constructor() {
        this.tree = {};
        this.count = 0;
    }
}

class OnlyOnceModeStorageManager {
    constructor() {
        this.trees = [];
    }
}


/**********************************************************************************************************************
 * Security
 **********************************************************************************************************************/
class LockConfiguration {
    constructor(enabled = false, password = null, duration= null, timer_plus = false,
                timer_plus_weight_box = true, timer_plus_weight_box_size = 50000, locked_options = [], timer_mode = 0) {
        this.enabled = enabled;
        this.password = password;
        this.duration = duration;
        this.duration_timestamp = null;
        this.token = null;
        this.locked_options = locked_options;
        this.timer_plus = timer_plus;
        this.timer_plus_data = {
            BELLY_EXPOSED: 0,
            BELLY_COVERED: 0,
            BUTTOCKS_EXPOSED: 0,
            BUTTOCKS_COVERED: 0,
            FEMALE_BREAST_EXPOSED: 0,
            FEMALE_BREAST_COVERED: 0,
            FEMALE_GENITALIA_EXPOSED: 0,
            FEMALE_GENITALIA_COVERED: 0,
            MALE_GENITALIA_EXPOSED: 0,
            MALE_GENITALIA_COVERED: 0,
            MALE_BREAST_EXPOSED: 0,
            MALE_BREAST_COVERED: 0,
            FACE_FEMALE: 0,
            FACE_MALE: 0,
            FEET_EXPOSED: 0,
            FEET_COVERED: 0,
            ARMPITS_EXPOSED: 0,
            ARMPITS_COVERED: 0,
            ANUS_EXPOSED: 0,
            ANUS_COVERED: 0,
        };
        this.timer_plus_weight_box = timer_plus_weight_box;
        this.timer_plus_weight_box_size = timer_plus_weight_box_size;
        this.timer_mode = timer_mode;
        this.remote_lock = false;
    }
}

const generateDefaultPreset = {
    0: () => null,
    1: () => new CensorTypePreset(new PixelCensorTypePresetConfig()),
    2: () => new CensorTypePreset(new BlurCensorTypePresetConfig()),
    3: () => new CensorTypePreset(new BarCensorTypePresetConfig()),
    4: () => new CensorTypePreset(new TriangleCensorTypePresetConfig()),
    5: () => new CensorTypePreset(new BoxCensorTypePresetConfig()),
    6: () => new CensorTypePreset(new GlitchCensorTypePresetConfig()),
    7: () => new CensorTypePreset(new StickerCensorTypePresetConfig()),
    8: () => new CensorTypePreset(new SobelCensorTypePresetConfig()),
    9: () => new CensorTypePreset(new SplatterCensorTypePresetConfig()),
    10: () => new CensorTypePreset(new MixedCensorTypePresetConfig()),
    11: () => new CensorTypePreset(new RandomCensorTypePresetConfig()),
}

function extractImgConfig(config, content_type){
    let img_config = config.png_configuration;
    switch (content_type) {
        case 'image/png':
            img_config = config.png_configuration;
            break;
        case 'image/jpg':
            img_config = config.jpg_configuration;
            break;
        case 'image/jpeg':
            img_config = config.jpg_configuration;
            break;
        case 'image/bmp':
            img_config = config.bmp_configuration;
            break;
        case 'image/webp':
            img_config = config.webp_configuration;
            break;
        case 'image/avif':
            img_config = config.avif_configuration;
            break;
        case 'image/gif':
            img_config = config.gif_configuration;
            break;
    }
    return img_config;
}

function checkImageDimensions(conf, width, height){
    if( width < conf._width_min ||
        (conf._width_max > 0 && width > conf._width_max) ||
        height < conf._height_min ||
        (conf._height_max > 0 && height > conf._height_max)){
        return false;
    }
    return true;
}

function checkImageFileSize(conf, filesize){
    if( filesize < conf._filesize_min ||
        (conf._filesize_max > 0 && filesize > conf._filesize_max)
        ){
        return false;
    }
    return true;
}

/**********************************************************************************************************************
 * Look & Feel
 **********************************************************************************************************************/

class LookAndFeelConfiguration {
    constructor() {
        this.main_bg_color = '#0B0E11';
        this.secondary_bg_color = '#151719';
        this.secondary_bg_color_variant_1 = '#2C3135';
        this.secondary_bg_color_variant_2 = '#212428';
        this.secondary_bg_color_variant_3 = '#14161a';
        this.secondary_bg_color_variant_4 = '#15181B';

        this.main_accent_color = '#009879';
        this.main_accent_color_variant_1 = '#017A62';
        this.main_accent_color_variant_2 = '#0B5E50';
        this.main_accent_color_variant_3 = '#213E3A';
        this.main_accent_color_variant_4 = '#4AC193';

        this.main_input_color = '#2C3135';
        this.main_input_color_variant_1 = '#1F2A2D';

        this.main_text_color = '#BBC9DF';
        this.main_text_color_disabled = '#BBC9DF99';
        this.tab_text_color = '#BBC9DF';

        this.main_border_color = '#BBC9DF';
        this.border_color_variant_1 = '#646D78';
        this.border_color_variant_2 = '#3A3F45';
        this.border_color_variant_3 = '#111111';

    }
}
