const effects = {
    NONE: {name: "normal",  index: 0, config: null},
    PIXEL: {name: "pixel",  index: 1, config: 'pixel_configuration'},
    BLUR: {name: "blur",    index: 2, config: 'blur_configuration'},
    BAR: {name: "black",  index: 3, config: 'bar_configuration'},
    TRIANGLE: {name: "triangle", index: 4,  config: 'triangle_configuration'},
    BOX: {name: "box", index: 5, config: 'box_configuration'},
    GLITCH: {name: "glitch", index: 6, config: 'glitch_configuration'},
    STICKER: {name: "sticker", index: 7, config: 'sticker_configuration'},
    SOBEL: {name: "sobel", index: 8, config: 'sobel_configuration'},
    SPLATTER: {name: "splatter", index: 9, config: 'splatter_configuration'},
    MIXED: {name: "mixed", index: 10, config: 'mixed_configuration'},
    RANDOM: {name: "random", index: 11, config: 'random_configuration'},
}

function parseEffectFromName(effect_string) {
    for (const entry in effects) {
        const effect = effects[entry];
        if (effect.name === effect_string) {
            return effect;
        }
    }
    console.error("Returning empty effect!", effect_string);
    return effects.NONE;
}

function parseEffectFromIndex(index) {
    for (const entry in effects) {
        const effect = effects[entry];
        if (effect.index === index) {
            return effect;
        }
    }
    console.error("Returning empty effect!", index);
    return effects.NONE;
}


// If they haven't been drawn for this preset index, draw a random censor type and preset config and store it in the cache. Secondary refers to the draw made when random is a censor type inside mixed, rather than when random is the selected censor type
function drawRandomModeSecondary(preset_index, config, cache) {
    if (cache[`random_preset_index_${preset_index}_secondary_drawn_censor_type`] != null) return;

    let random_preset_config = config.random_configuration.presets[preset_index].config;
    let weights = {...random_preset_config.censor_weights};
    weights[effects.MIXED.index] = 0;
    let effect = parseEffectFromIndex(randomWeighted(weights));

    cache[`random_preset_index_${preset_index}_secondary_drawn_censor_type`] = effect.index;
    if (random_preset_config.mode === 1 && effect.index !== effects.NONE.index) {
        cache[`random_preset_index_${preset_index}_secondary_drawn_censor_preset_config`] = randomizeConfigObject(getSelectedPreset(config[effect.config]).config);
    }
}

// If they haven't been drawn, draw a random censor type and preset config and store it in the cache. Primary refers to the draw made when random is the selected censor type, rather than when random is a censor type inside mixed
function drawRandomModePrimary(config, cache) {
    if (cache['random_primary_drawn_censor_type'] != null) return;

    let random_preset_config = getSelectedPreset(config.random_configuration).config;
    let effect = parseEffectFromIndex(randomWeighted(random_preset_config.censor_weights));

    cache['random_primary_drawn_censor_type'] = effect.index;
    if (random_preset_config.mode === 1 && effect.index !== effects.NONE.index) {
        cache['random_primary_drawn_censor_preset_config'] = randomizeConfigObject(getSelectedPreset(config[effect.config]).config)
    }
}

function randomWeighted(weights) {
    const total_weight = Object.values(weights).reduce((weight_sum, weight) => weight_sum + weight, 0);
    const random_value = Math.random() * total_weight;
    
    let curr_weight_sum = 0;
    for (const [index, weight] of Object.entries(weights)) {
        curr_weight_sum += weight;
        if (random_value <= curr_weight_sum) {
            return parseInt(index);
        }
    }

    return 0;
}

const klasses = {
    BELLY_EXPOSED: {key: "BELLYEXPOSED", name: "Belly (nude)", index: 0, body_part_index: 0, nude: true, erotic: false},
    BELLY_COVERED: {key: "BELLYCOVERED", name: "Belly", index: 1, body_part_index: 0, nude: false, erotic: false},
    BUTTOCKS_EXPOSED: {key: "BUTTOCKSEXPOSED", name: "Buttocks (nude)", index: 2, body_part_index: 1, nude: true, erotic: false},
    BUTTOCKS_COVERED: {key: "BUTTOCKSCOVERED", name: "Buttocks", index: 3, body_part_index: 1,  nude: false, erotic: false},
    FEMALE_BREAST_EXPOSED: {key: "FEMALEBREASTEXPOSED", name: "Breast F (nude)", index: 4, body_part_index: 2, nude: true, erotic: true},
    FEMALE_BREAST_COVERED: {key: "FEMALEBREASTCOVERED", name: "Breast F", index: 5, body_part_index: 2, nude: false, erotic: false},
    FEMALE_GENITALIA_EXPOSED: {key: "FEMALEGENITALIAEXPOSED", name: "Genitalia F (nude)", index: 6, body_part_index: 3, nude: true, erotic: true},
    FEMALE_GENITALIA_COVERED: {key: "FEMALEGENITALIACOVERED", name: "Genitalia F", index: 7, body_part_index: 3, nude: false, erotic: false},
    MALE_GENITALIA_COVERED: {key: "MALEGENITALIACOVERED", name: "Genitalia M", index: 8, body_part_index: 4, nude: false, erotic: false},
    MALE_GENITALIA_EXPOSED: {key: "MALEGENITALIAEXPOSED", name: "Genitalia M (nude)", index: 9, body_part_index: 4, nude: true, erotic: true},
    MALE_BREAST_EXPOSED: {key: "MALEBREASTEXPOSED", name: "Breast M (nude)", index: 10, body_part_index: 5, nude: true, erotic: false},
    MALE_BREAST_COVERED: {key: "MALEBREASTCOVERED", name: "Breast M", index: 11, body_part_index: 5, nude: false, erotic: false},
    FACE_FEMALE: {key: "FACEFEMALE", name: "Female", index: 12, body_part_index: 6, nude: false, erotic: false},
    FACE_MALE: {key: "FACEMALE", name: "Male", index: 13, body_part_index: 7, nude: false, erotic: false},
    FEET_COVERED: {key: "FEETCOVERED", name: "Feet", index: 14, body_part_index: 8, nude: false, erotic: false},
    FEET_EXPOSED: {key: "FEETEXPOSED", name: "Feet (nude)", index: 15, body_part_index: 8, nude: true, erotic: false},
    ARMPITS_COVERED: {key: "ARMPITSCOVERED", name: "Armpit", index: 16, body_part_index: 9, nude: false, erotic: false},
    ARMPITS_EXPOSED: {key: "ARMPITSEXPOSED", name: "Armpit (nude)", index: 17, body_part_index: 9, nude: true, erotic: false},
    ANUS_COVERED: {key: "ANUSCOVERED", name: "Anus", index: 18, body_part_index: 10, nude: true, erotic: false},
    ANUS_EXPOSED: {key: "ANUSEXPOSED", name: "Anus (nude)", index: 19, body_part_index: 10, nude: true, erotic: true},
    NONE: {key: null, name: "Invalid", index: -1, nude: false, erotic: false},
}

function parseKlassFromIndex(index) {
    for (const entry in klasses) {
        const klass = klasses[entry];
        if (klass.index === index) {
            return klass;
        }
    }
    console.error("Returning empty klass!", index);
    return klasses.NONE;
}

function parseKlassFromKey(key) {
    for (const entry in klasses) {
        const klass = klasses[entry];
        if (klass.key === key) {
            return klass;
        }
    }
    console.error("Returning empty klass!", key);
    return klasses.NONE;
}

function parseEntryFromIndex(index) {
    for (const entry in klasses) {
        const klass = klasses[entry];
        if (klass.index === index) {
            return entry;
        }
    }
    console.error("Returning empty klass!", index);
    return 'NONE';
}