function deduceAIInfos(frame_count, infos, ai_frequency, det_similarity_threshold, det_extending) {
    let deduced_infos = populateArray(frame_count, () => ({ cboxes: [], ctranslated_klasses: [], cscores: [], detections_count: 0 }));

    for (let info_i = 0; info_i < infos.length; info_i++) {
        let info = infos[info_i];
        for (let entry_i = 0; entry_i < info.cboxes.length; entry_i++) {
            deduceNextAIEntries(info_i, entry_i);
            deducePrevAIEntries(info_i, entry_i);
        }
    }

    return deduced_infos;

    function deduceNextAIEntries(info_i, entry_i) {
        let info = infos[info_i];
        for (let n_info_i = info_i + 1; n_info_i < Math.min(infos.length, info_i + det_extending * 2); n_info_i++) {
            let n_info = infos[n_info_i];
            for (let n_entry_i = 0; n_entry_i < n_info.cboxes.length; n_entry_i++) {
                if (compareAIEntries(info, entry_i, n_info, n_entry_i))  {
                    createIntermediateAIEntries((n_info_i - info_i) * ai_frequency, info_i, entry_i, n_info_i, n_entry_i);
                    return;
                }
            }
        }

        for (let i = info_i * ai_frequency; i < Math.min(frame_count, (info_i + det_extending) * ai_frequency + 1); i++) {
            deduced_infos[i].cboxes.push(info.cboxes[entry_i]);
            deduced_infos[i].ctranslated_klasses.push(info.ctranslated_klasses[entry_i]);
            deduced_infos[i].cscores.push(info.cscores[entry_i]);
            deduced_infos[i].detections_count++;
        }
    }

    function deducePrevAIEntries(info_i, entry_i) {
        if (info_i < 1) return;

        let info = infos[info_i];
        let p_info = deduced_infos[(info_i - 1) * ai_frequency];
        for (let p_entry_i = 0; p_entry_i < p_info.cboxes.length; p_entry_i++) {
            if (compareAIEntries(info, entry_i, p_info, p_entry_i)) {
                return;
            }
        }

        for (let i = info_i * ai_frequency - 1; i > Math.max(-1, (info_i + det_extending) * ai_frequency); i--) {
            deduced_infos[i].cboxes.push(info.cboxes[entry_i]);
            deduced_infos[i].ctranslated_klasses.push(info.ctranslated_klasses[entry_i]);
            deduced_infos[i].cscores.push(info.cscores[entry_i]);
            deduced_infos[i].detections_count++;
        }
    }

    function compareAIEntries(info, entry_i, p_info, p_entry_i) {
        if (info.ctranslated_klasses[entry_i].body_part_index !== p_info.ctranslated_klasses[p_entry_i].body_part_index) return false;
        let box = info.cboxes[entry_i], next_box = p_info.cboxes[p_entry_i];

        let avg_box_area = Math.abs((box[2] - box[0]) * (box[3] - box[1]) + (next_box[2] - next_box[0]) * (next_box[3] - next_box[1])) / 2;

        let x_overlap = Math.max(box[0], next_box[0]);
        let y_overlap = Math.max(box[1], next_box[1]);
        let end_x_overlap = Math.min(box[2], next_box[2]);
        let end_y_overlap = Math.min(box[3], next_box[3]);

        let overlap_width = end_x_overlap - x_overlap;
        let overlap_height = end_y_overlap - y_overlap;
        if (overlap_width <= 0 || overlap_height <= 0) return false;
        else return det_similarity_threshold <= overlap_width * overlap_height / avg_box_area;1
    }

    function createIntermediateAIEntries(frames_covered, info_i, entry_i, n_info_i, n_entry_i) {
        let info = infos[info_i], box = info.cboxes[entry_i];
        let n_info = infos[n_info_i], n_box = n_info.cboxes[n_entry_i];

        let intermediate_boxes = populateArray(frames_covered, () => new Array(4));
        for (let coord_i = 0; coord_i < 4; coord_i++) {
            let diff = (n_box[coord_i] - box[coord_i]) / frames_covered;
            for (let i = 0; i < frames_covered; i++) {
                intermediate_boxes[i][coord_i] = Math.round(box[coord_i] + diff * i);
            }
        }

        let intersection = frames_covered;
        if (info.ctranslated_klasses[entry_i].nude && !n_info.ctranslated_klasses[n_entry_i].nude) {
            intersection = Math.min(frames_covered, det_extending * ai_frequency);
        } else if (n_info.ctranslated_klasses[n_entry_i].nude && !info.ctranslated_klasses[entry_i].nude){
            intersection = Math.max(0, frames_covered - det_extending * ai_frequency);
        }

        let i = 0;
        for (; i < intersection; i++) {
            deduced_infos[info_i * ai_frequency + i].cboxes.push(intermediate_boxes[i]);
            deduced_infos[info_i * ai_frequency + i].ctranslated_klasses.push(info.ctranslated_klasses[entry_i]);
            deduced_infos[info_i * ai_frequency + i].cscores.push(info.cscores[entry_i]);
            deduced_infos[info_i * ai_frequency + i].detections_count++;
        }
        for (; i < frames_covered; i++) {
            deduced_infos[info_i * ai_frequency + i].cboxes.push(intermediate_boxes[i]);
            deduced_infos[info_i * ai_frequency + i].ctranslated_klasses.push(n_info.ctranslated_klasses[n_entry_i]);
            deduced_infos[info_i * ai_frequency + i].cscores.push(n_info.cscores[n_entry_i]);
            deduced_infos[info_i * ai_frequency + i].detections_count++;
        }
    }
}

function paintCensorWithParsedAIInfo(out, outctx, censor_type, labels, cboxes, ctranslated_klasses, cscores, detections_count, configuration, cache = {}) {
    const image_processor = new ImageProcessor(out, outctx, configuration);
    paintImageProcessor(image_processor, censor_type, labels, cboxes, ctranslated_klasses, cscores, configuration, cache, detections_count);
}

function paintCensorRequest(out, outctx, processedRequest, configuration, cache = {}) {
    const labels = processedRequest.labels;
    const valid_detections = processedRequest.valid_detections;
    const classes = processedRequest.classes;
    const scores = processedRequest.scores;
    const boxes = processedRequest.boxes;
    const censor_type = processedRequest.censortype;

    const image_processor = new ImageProcessor(out, outctx, configuration);

    let { cboxes, ctranslated_klasses, cscores, detections_count } = parseAIInfo(image_processor.natural_width, image_processor.natural_height, censor_type, labels, valid_detections, classes, boxes, scores);

    paintImageProcessor(image_processor, censor_type, labels, cboxes, ctranslated_klasses, cscores, configuration, cache, detections_count);
}

function paintCensor(out, outctx, censor_type, labels, valid_detections, classes, scores, boxes, configuration, cache = {}) {
    const image_processor = new ImageProcessor(out, outctx, configuration);

    let { cboxes, ctranslated_klasses, cscores, detections_count } = parseAIInfo(image_processor.natural_width, image_processor.natural_height, censor_type, labels, valid_detections, classes, boxes, scores);
    
    paintImageProcessor(image_processor, censor_type, labels, cboxes, ctranslated_klasses, cscores, configuration, cache, detections_count);
}

function parseAIInfo(natural_w, natural_h, censor_type, labels, valid_detections, classes, boxes, scores)  {
    let cboxes = [];
    let ctranslated_klasses = [];
    let cscores = [];
    let detections_count = valid_detections[0]
    for (let i = 0; i < detections_count; i++) {
        let translated = parseKlassFromIndex(classes[i]);
        // TODO: box censors in mixed won't trigger this, are we ok with that?
        if (censor_type === 5 && getSelectedPreset(configuration.box_configuration).config.border === 0) {
            // Do nothing
        } else {
            // Skip if user is not interested
            if (!labels.includes(translated.key)) continue;
        }
        ctranslated_klasses.push(translated);
        let [x1, y1, x2, y2] = boxes.slice(i * 4, (i + 1) * 4);
        x1 *= natural_w;
        x2 *= natural_w;
        y1 *= natural_h;
        y2 *= natural_h;
        cboxes.push([Math.ceil(x1), Math.ceil(y1), Math.ceil(x2), Math.ceil(y2)]);
        cscores.push(scores[i]);
    }

    return { cboxes, ctranslated_klasses, cscores, detections_count };
}

function paintImageProcessor(image_processor, censor_type, labels, cboxes, ctranslated_klasses, cscores,  configuration, cache, detections_count) {
    let censor_effect = parseEffectFromName(censor_type);

    cache.detections_count = detections_count;
    cache.reverse_mode = configuration.reverse_mode_configuration.enabled;

    // TODO: this can be done before parsing AI info
    if ((!cache.reverse_mode || (configuration.reverse_mode_configuration.mode !== 0 && !cache.video_frame)) && cache.detections_count <= 0) return;

    cache.enabled_detections_count = cboxes.length;
    cache.labels = labels;

    cache.random_mode = censor_effect.name === configuration.random_configuration._name;
    if (cache.random_mode) {
        drawRandomModePrimary(configuration, cache);
        censor_effect = parseEffectFromIndex(cache['random_primary_drawn_censor_type']);
    }

    if (censor_effect === effects.NONE) return;

    cache.censor_config = configuration[censor_effect.config];
    if (cache['random_primary_drawn_censor_preset_config']) {
        cache.censor_preset_config = cache['random_primary_drawn_censor_preset_config'];
        cache.censor_preset_ident = configuration.random_configuration.selected_preset_index;
    } else {
        cache.censor_preset_config = getSelectedPreset(cache.censor_config).config;
        cache.censor_preset_ident = cache.censor_config.selected_preset_index;
    }

    cache.mixed_mode = censor_effect.name === configuration.mixed_configuration._name;

    if (cache.mixed_mode) {
        collectMixedEntries(censor_effect.index, configuration, ctranslated_klasses, cache);
    }
    
    if (!cache.reverse_mode || (configuration.reverse_mode_configuration.mode === 2 && !cache.video_frame)) {
        if (cache.mixed_mode) {
            let layer = 0;
            while (true) {
                let reverse_entry = cache.mixed_reverse_entries[layer];
                if (0 < cache.detections_to_censor_per_layer[layer]
                    || (reverse_entry != null 
                        && reverse_entry.type !== effects.NONE.index 
                        && cache.detections_to_normal_censor_per_layer[layer] < ctranslated_klasses.length)) break;
                layer++;
                if (MIXED_CENSOR_LAYERS_COUNT <= layer) {
                    return;
                }
            }
        } else if (cache.reverse_mode && cache.enabled_detections_count <= 0) return;
    }

    paintMultiImageProcessor(image_processor, censor_effect, configuration, cache,
        cboxes,
        ctranslated_klasses,
        cscores);
}

function collectMixedEntries(censor_type, configuration, ctranslated_klasses, cache) {
    cache.mixed_reverse_entries = new Array(MIXED_CENSOR_LAYERS_COUNT);
    if (cache.reverse_mode) {
        for (let layer = 0; layer < MIXED_CENSOR_LAYERS_COUNT; layer++) {
            let reverse_censor_type = cache.censor_preset_config.reverse_censor_types[layer];
            if (reverse_censor_type == null) continue;
            let reverse_censor_preset_index = cache.censor_preset_config.reverse_censor_preset_indexes[layer];

            if (reverse_censor_type === effects.RANDOM.index) {
                drawRandomModeSecondary(reverse_censor_preset_index, configuration, cache);

                reverse_censor_type = cache[`random_preset_index_${reverse_censor_preset_index}_secondary_drawn_censor_type`];
                // Will contain something if the random mode is super and the drawn censor type isn't normal
                if (cache[`random_preset_index_${reverse_censor_preset_index}_secondary_drawn_censor_preset_config`]) {
                    cache.mixed_reverse_entries[layer] = {
                        type: reverse_censor_type,
                        preset_ident: `random_${reverse_censor_preset_index}`,
                        preset_config: cache[`random_preset_index_${reverse_censor_preset_index}_secondary_drawn_censor_preset_config`],
                    };
                    // An entry has been stored, we're done here
                    continue;
                }
                censor_preset_index = configuration[parseEffectFromIndex(censor_type).config]?.selected_preset_index ?? 0;
            }

            if (cache.video_frame && (reverse_censor_type === effects.TRIANGLE.index || reverse_censor_type === effects.GLITCH.index)) continue;

            let reverse_censor_config = configuration[parseEffectFromIndex(reverse_censor_type).config];
            cache.mixed_reverse_entries[layer] = {
                type: reverse_censor_type,
                preset_ident: reverse_censor_preset_index,
                preset_config: reverse_censor_config?.presets[reverse_censor_preset_index].config,
            };
        }
    }

    cache.detections_to_normal_censor_per_layer = new Array(MIXED_CENSOR_LAYERS_COUNT).fill(0);
    cache.detections_to_censor_per_layer = new Array(MIXED_CENSOR_LAYERS_COUNT).fill(0);
    cache.mixed_entries = {};
    // Only collect label entries of enabled labels
    for (let ctranslated_klass of ctranslated_klasses) {
        let klass = ctranslated_klass.index;
        cache.mixed_entries[klass] = new Array(MIXED_CENSOR_LAYERS_COUNT);
        for (let layer = 0; layer < MIXED_CENSOR_LAYERS_COUNT; layer++) {
            let censor_type = cache.censor_preset_config.censor_types[klass][layer];
            if (censor_type == null) continue;
            let censor_preset_index = cache.censor_preset_config.censor_preset_indexes[klass][layer];

            if (censor_type === effects.RANDOM.index) {
                drawRandomModeSecondary(censor_preset_index, configuration, cache);

                censor_type = cache[`random_preset_index_${censor_preset_index}_secondary_drawn_censor_type`];
                // Will contain something if the random mode is super and the drawn censor type isn't normal
                if (cache[`random_preset_index_${censor_preset_index}_secondary_drawn_censor_preset_config`]) {
                    cache.detections_to_censor_per_layer[layer]++;
                    cache.mixed_entries[klass][layer] = {
                        type: censor_type,
                        preset_ident: `random_${censor_preset_index}`,
                        preset_config: cache[`random_preset_index_${censor_preset_index}_secondary_drawn_censor_preset_config`],
                    };
                    // An entry has been stored, we're done here
                    continue;
                }
                censor_preset_index = configuration[parseEffectFromIndex(censor_type).config]?.selected_preset_index ?? 0;
            }

            if (cache.video_frame && (censor_type === effects.TRIANGLE.index || censor_type === effects.GLITCH.index)) continue;

            if (censor_type === effects.NONE.index) {
                cache.detections_to_normal_censor_per_layer[layer]++;
                if (cache.mixed_reverse_entries[layer] == null) continue;
                cache.mixed_entries[klass][layer] = {
                    type: censor_type,
                    preset_ident: cache.mixed_reverse_entries[layer].preset_ident,
                    preset_config: cache.mixed_reverse_entries[layer].preset_config,
                };
            } else {
                cache.detections_to_censor_per_layer[layer]++;
                let censor_config = configuration[parseEffectFromIndex(censor_type).config];
                cache.mixed_entries[klass][layer] = {
                    type: censor_type,
                    preset_ident: censor_preset_index,
                    preset_config: censor_config.presets[censor_preset_index].config,
                };
            }
        }
    }
}

function paintMultiImageProcessor(image_processor, censor_effect, configuration, cache,
                                  cboxes,
                                  ctranslated_klasses,
                                  cscores) {
    cache.box_count = cboxes.length;

    image_processor.before(cache);
    if (cache.mixed_mode) {
        let results_by_censor_type_by_layer = processIntoMixedResults(cboxes,
            ctranslated_klasses,
            cscores,
            configuration.clustering_configuration,
            cache);

        for (let layer = 0; layer < MIXED_CENSOR_LAYERS_COUNT; layer++) {
            let results_by_censor_type = results_by_censor_type_by_layer[layer];
            if (!Object.keys(results_by_censor_type).length && !(cache.reverse_mode || cache.mixed_reverse_entries[layer] != null)) continue;
            
            image_processor.beforeLayer(cache);
            
            if (cache.reverse_mode && cache.mixed_reverse_entries[layer] != null) {
                image_processor.applyReverseEffect(parseInt(cache.mixed_reverse_entries[layer].type),cache.mixed_reverse_entries[layer].preset_config, cache.mixed_reverse_entries[layer].preset_ident, cache);

                let normal_results = results_by_censor_type[effects.NONE.index];
                if (normal_results != null) {
                    image_processor.applyNormalEffects(parseInt(cache.mixed_reverse_entries[layer].type), normal_results, cache);
                }
            }

            for (const censor_type in results_by_censor_type) {
                image_processor.applyEffects(parseInt(censor_type), results_by_censor_type[censor_type], cache);
            }
        }
    } else {
        let results = processIntoSimpleResults(cboxes,
            ctranslated_klasses,
            cscores,
            configuration.clustering_configuration,
            cache);

        image_processor.beforeLayer(cache);

        if (cache.reverse_mode) {
            image_processor.applyReverseEffect(censor_effect.index, cache.censor_preset_config, cache.censor_preset_ident, cache);

            if (results.length) {
                image_processor.applyNormalEffects(censor_effect.index, results, cache);
            }
        } else if (results.length) {
            image_processor.applyEffects(censor_effect.index, results, cache);
        }
    }
}

function processIntoSimpleResults(cboxes, ctranslated_klasses, cscores, clustering_config, cache) {
    let results = [];
    if (clustering_config?.enabled) {
        cluster(null, ctranslated_klasses, cscores, cboxes, clustering_config, storeIntoSimpleResult.bind(null, results), cache);
    } else {
        for (let i = 0; i < cboxes.length; i++) {
            storeIntoSimpleResult(results, ctranslated_klasses[i], cscores[i], cboxes[i], cache);
        }
    }
    return results;
}

function processIntoMixedResults(cboxes, ctranslated_klasses, cscores, clustering_config, cache) {
    let results_by_censor_type_by_layer = new Array(MIXED_CENSOR_LAYERS_COUNT);
    if (clustering_config.enabled) {
        for (let layer = 0; layer < MIXED_CENSOR_LAYERS_COUNT; layer++) {
            let cboxes_copy = Object.assign([], cboxes);
            let ctranslated_klasses_copy = Object.assign([], ctranslated_klasses);
            let cscores_copy = Object.assign([], cscores);
            results_by_censor_type_by_layer[layer] = {};
            
            cluster(layer, ctranslated_klasses_copy, cscores_copy, cboxes_copy, clustering_config, storeIntoMixedResult.bind(null, results_by_censor_type_by_layer, layer), cache);
        }
    } else {
        for (let layer = 0; layer < MIXED_CENSOR_LAYERS_COUNT; layer++) {
            results_by_censor_type_by_layer[layer] = {};
            for (let i = 0; i < cboxes.length; i++) {
                if (cache.mixed_entries[ctranslated_klasses[i].index][layer] != null) {
                    storeIntoMixedResult(results_by_censor_type_by_layer, layer, ctranslated_klasses[i], cscores[i], cboxes[i], cache);
                }
            }
        }
    }
    return results_by_censor_type_by_layer;
}

function cluster(layer, ctranslated_klasses, cscores, cboxes, clustering_config, collect, cache) {
    for (let k = 0; k < cboxes.length; k++) {
        // Copy index so we can change the value of i without restarting this loop
        let i = k;
        
        let preset_config = cache.mixed_mode ? cache.mixed_entries[ctranslated_klasses[i].index][layer]?.preset_config : cache.censor_preset_config;
        if (preset_config == null) continue;

        let [x1, y1, x2, y2] = cboxes[i];
        let rect_sc = new Rectangle(x1, y1, x2 - x1, y2 - y1);
        rect_sc.scale(preset_config.scale, preset_config.scale);

        let x1_sc = rect_sc.x;
        let y1_sc = rect_sc.y;
        let x2_sc = rect_sc.x + rect_sc.w;
        let y2_sc = rect_sc.y + rect_sc.h;

        for (let j = i + 1; j < cboxes.length; j++) {
            let other_preset_config = cache.mixed_mode ? cache.mixed_entries[ctranslated_klasses[j].index][layer]?.preset_config : cache.censor_preset_config;
            if (other_preset_config == null) continue;

            let [other_x1, other_y1, other_x2, other_y2] = cboxes[j];
            let other_rect_sc = new Rectangle(other_x1, other_y1, other_x2 - other_x1, other_y2 - other_y1);
            other_rect_sc.scale(other_preset_config.scale, other_preset_config.scale);

            let other_x1_sc = other_rect_sc.x;
            let other_y1_sc = other_rect_sc.y;
            let other_x2_sc = other_rect_sc.x + other_rect_sc.w;
            let other_y2_sc = other_rect_sc.y + other_rect_sc.h;

            if (preset_config !== other_preset_config
                || (clustering_config.mode === 1 && ctranslated_klasses[i].index !== ctranslated_klasses[j].index)
                || !intersects(x1_sc, y1_sc, x2_sc, y2_sc, other_x1_sc, other_y1_sc, other_x2_sc, other_y2_sc)) continue;

            // Make a new box, remove the merged ones and restart
            let newbox = merge(x1, y1, x2, y2,
                other_x1, other_y1, other_x2, other_y2);
            // TODO: iterate from back to front, so we can push here instead of unshifting
            cboxes.unshift(newbox);
            cboxes.splice(i + 1, 1);
            cboxes.splice(j, 1);

            // cdetections -= 1;
            ctranslated_klasses.unshift({
                key: (ctranslated_klasses[i].key),
                name: (ctranslated_klasses[i].name === ctranslated_klasses[j].name ? ctranslated_klasses[i].name : ctranslated_klasses[i].name + " & " + ctranslated_klasses[j].name),
                index: (ctranslated_klasses[i].index),
                nude: ctranslated_klasses[i].nude || ctranslated_klasses[j].nude,
                erotic: ctranslated_klasses[i].erotic || ctranslated_klasses[j].erotic
            });
            ctranslated_klasses.splice(i + 1, 1);
            ctranslated_klasses.splice(j, 1);

            cscores.unshift((cscores[i] + cscores[j]) / 2);
            cscores.splice(i + 1, 1);
            cscores.splice(j, 1);

            if (j < k) k--;
            i = 0;
            j = 0;

            ([x1, y1, x2, y2] = cboxes[i]);
            rect_sc = new Rectangle(x1, y1, x2 - x1, y2 - y1);
            rect_sc.scale(preset_config.scale, preset_config.scale);

            x1_sc = rect_sc.x;
            y1_sc = rect_sc.y;
            x2_sc = rect_sc.x + rect_sc.w;
            y2_sc = rect_sc.y + rect_sc.h;
        }

        collect(ctranslated_klasses[i], cscores[i], cboxes[i], cache);
    }
}

function storeIntoSimpleResult(results, ctranslated_klass, cscore, cbox, cache) {
    let result = {
        'klass': ctranslated_klass,
        'score': cscore.toFixed(2),
        'rect': new Rectangle(cbox[0], cbox[1], cbox[2] - cbox[0], cbox[3] - cbox[1]),
        'preset_config': cache.censor_preset_config,
        'preset_ident': cache.censor_preset_ident,
    };
    results.push(result);
}

function storeIntoMixedResult(results_by_censor_type_by_layer, layer, ctranslated_klass, cscore, cbox, cache) {
    let censor_preset_config = cache.mixed_entries[ctranslated_klass.index][layer].preset_config;
    let result = {
        'klass': ctranslated_klass,
        'score': cscore.toFixed(2),
        'rect': new Rectangle(cbox[0], cbox[1], cbox[2] - cbox[0], cbox[3] - cbox[1]),
        'preset_config': censor_preset_config,
        'preset_ident': cache.mixed_entries[ctranslated_klass.index][layer].preset_ident,
    };
    
    let censor_type = cache.mixed_entries[ctranslated_klass.index][layer].type;
    if (results_by_censor_type_by_layer[layer][censor_type] == null) {
        results_by_censor_type_by_layer[layer][censor_type] = [result];
    } else {
        // Group by matching preset configs
        let matching_config_index = results_by_censor_type_by_layer[layer][censor_type].findIndex(item => item.preset_config === censor_preset_config);
        if (matching_config_index !== -1) {
            results_by_censor_type_by_layer[layer][censor_type].splice(matching_config_index, 0, result);
        } else {
            results_by_censor_type_by_layer[layer][censor_type].push(result);
        }
    }
}

function merge(ix1, iy1, ix2, iy2, jx1, jy1, jx2, jy2) {
    return [Math.min(ix1, jx1),
        Math.min(iy1, jy1),
        Math.max(ix2, jx2),
        Math.max(iy2, jy2)
    ];
}

function intersects(ix1, iy1, ix2, iy2,
                    jx1, jy1, jx2, jy2) {
    return !(jx1 > ix2
        || jx2 < ix1
        || jy1 > iy2
        || jy2 < iy1
    );
}

